#ifndef QSAS_QDU_H_
#define QSAS_QDU_H_

#define QDUTIL_OK 0
#define QDUTIL_ERR 1
#define QDUTIL_WARN 2

#include <stdlib.h>
#include "DvObj.h"
#include "Xrefs.h"


using namespace std;

typedef vector <int> dimTriple;
typedef vector <dimTriple> dimTripleStack;

#define EVENT_0_STR " < 0 >"
#define R_STR_CMP " [ r (mag) ]"
#define X_STR_CMP " [ x ]"
#define Y_STR_CMP " [ y ]"
#define Z_STR_CMP " [ z ]"
#define Ld_STR_CMP " [ Lat (deg) ]"
#define Lr_STR_CMP " [ Lat (rad) ]"
#define Td_STR_CMP " [ theta (deg) ]"
#define Tr_STR_CMP " [ theta (rad) ]"
#define Pd_STR_CMP " [ phi (deg) ]"
#define Pr_STR_CMP " [ phi (rad) ]"
#define Lt_STR_CMP " [ Local Time ]"
#define T_STR_START " [ Start Time ]"
#define T_STR_CENTRE " [ Centre Time ]"
#define T_STR_END " [ End Time ]"
#define T_STR_0_START " <0> [ Start Time ]"

typedef enum QguComp_e
{
  UNKNOWN_COMP=-1,
  WL_OBJ=0,
  R_COMP=1,
  X_COMP=2,
  Y_COMP=3,
  Z_COMP=4,
  LAT_DEG_COMP=5,
  THETA_DEG_COMP=6,
  PHI_DEG_COMP=7,
  LAT_RAD_COMP=8,
  THETA_RAD_COMP=9,
  PHI_RAD_COMP=10,
  ARRAY_COMP=11,
  XREF_COMP=12,
  T_START_COMP=13,
  T_CENTRE_COMP=14,
  T_END_COMP=15,
  XREF_OBJ=16,
  SLICE_COMP=17,
  T_0_START_COMP=18,
  LOCAL_T=19
} QguComp;




int QuEpochType(DvObject_var &obj);
bool QuIsMatch(double value, double fill);
bool QuSlotDebug();
string QuCommonLabl(string str1, string str2);
DvString baseObjectText(const char *);
DvString getCompChange(const char *oldComp, const char *newComp);
DvString getNewComp(const char *oldComp, const char *changes);
QguComp compEntry(const char*,const char*);
QguComp compEntry(const char*);
DvString compText(const char*);
dimTriple getDimTriple(DvString &compTxt);
dimTriple getSliceTriple(DvString &compTxt);
dimTripleStack getDimTripleStack(const char* compTxt);
bool QuIsBadData(double value, double fill);
void QuMsgBar(const char * text, bool error);
void QuMsgBar(const char* text1, const char* text2, bool error);
void QuSetOK();
int QuStrcpy(char * To, const char * From, int ToLen);
int QuStrcat(char * To, const char * From, int ToLen);
string QuToUpper(const char *InStr);
string QuToUpper(string InStr);
double ConvertThetaRad(double theta);
double ConvertThetaDeg(double theta);
string QuSanitizeName ( const char * oldName);
bool QuStrCmp(const char *A, const char *B);
void QuNow(const char * txt, bool reset=false);
bool QuSet_Array(int ndata, double *floatArray, DvObject_var &obj, DvTime offset=DvTime());
bool QuSetObjByName(DvObject_var ToSetobj, const char* name);
void QuRenameObject(const char* oldName, const char* newName);
void QuDeleteObject(const char* name);
void QuReplaceObject(const char* name, DvObject_var &new_obj);
bool QuIsBad(double value, double fill);
void setRestoreVersion(const char *version);
void QuParseSIC(string &str, double &multiplier);

void cleanWLtext(string &text);

bool validObjName(DvString objName);
DvObject *getWLObjExact(DvString objName);
DvObject *getWLObject(DvString objName);
DvObject *getWLObject(const char * objName);

DvObject_var getObjectAndComp(DvString objName);
DvObject_var getObjectAndComp(const char * objName);
# endif // QSAS_QDU_H_
