#include "qcalcContext.h"
#include "qcalcSlots.h"
#include "qcalcCalcItem.h"
#include "qcalcInputItem.h"
#include "qcalcOperatorItem.h"
#include "qcalcOutputItem.h"
#include "qcalcExtractionItem.h"

TimeLineHandler timeLineHandler;

ContextWidget::ContextWidget(QWidget* parent, CalcView* view):
    QWidget(parent)
{	
//     setMinimumWidth(300);
	_view = view;
	
    pluginView=new PluginContextWidget(this);
    bbView=new BlackBoxContextWidget(this, view);
    ioView=new IOContextWidget(this, view);
	defaultView=new DefaultContextWidget(this, view);
	selectionView=new SelectionContextWidget(this, view);
	extractorView=new ExtractorContextWidget(this);
	joinView=new JoinContextWidget(this, view);

	connect(view, SIGNAL(componentSelected(const char *, const char *)), extractorView, SLOT(userSetComponent(const char *, const char *)));
	
    QVBoxLayout* layout=new QVBoxLayout;

	tabWidget=new QTabWidget(this);
	tabWidget->addTab(defaultView, "New Item");
	tabWidget->addTab(selectionView, "Aspect");
	tabWidget->addTab(pluginView, "Description");
	tabWidget->addTab(bbView, "Black Box");
	tabWidget->addTab(extractorView, "Extract");
    tabWidget->addTab(joinView, "Timeline/Join");
    tabWidget->addTab(ioView, "Input/Output");

	setView(false);
	layout->addWidget(tabWidget);
    setLayout(layout);
}


void ContextWidget::setView(bool withItems)
{
	
	QList<QGraphicsItem*> items;
	if( withItems ) items = _view->scene()->selectedItems(); // otherwise empty list
	
	QWidget* current=tabWidget->currentWidget();
	tabWidget->clear();

	tabWidget->addTab(defaultView, "Insertion");

	
	if(!items.isEmpty())
	{
		// Add plugin view if and only if there is 1 plugin in the selection
		int counter=0;
		VOperatorItem* op=NULL;
		foreach(QGraphicsItem* item, items)
		{
			OperatorItem* tempItem=qgraphicsitem_cast<OperatorItem*>(item);
			if(tempItem!=NULL)
			{
				if(tempItem->parentItem->isPlugin())
				{
					op=tempItem->parentItem;
					++counter;
				}
			}
		}
		if(counter==1 && op!=NULL)
		{
			pluginView->setPluginInfo(op);
			tabWidget->addTab(pluginView, "Description");
		}

        counter=0;
        VCalcItem* inout=NULL;
        foreach(QGraphicsItem* item, items)
        {
            InputItem* tempItemi=qgraphicsitem_cast<InputItem*>(item);
            if(tempItemi!=NULL)
            {
                inout=tempItemi->parentItem;
                ++counter;
            }
            OutputItem* tempItemo=qgraphicsitem_cast<OutputItem*>(item);
            if(tempItemo!=NULL)
            {
                inout=tempItemo->parentItem;
                ++counter;
            }
        }
        if(counter==1 && inout!=NULL)
        {
            ioView->setInfo(inout);
            tabWidget->addTab(ioView, "Input/Output");
        }

		counter=0;
		op=NULL;
		foreach(QGraphicsItem* item, items)
		{
			OperatorItem* tempItem=qgraphicsitem_cast<OperatorItem*>(item);
			if(tempItem!=NULL)
			{
				if(tempItem->parentItem->isBlackBox)
				{
					op=tempItem->parentItem;
					++counter;
				}
			}
		}
		if(counter==1 && op!=NULL)
		{
			bbView->setBlackBoxInfo(op);
			tabWidget->addTab(bbView, "Black Box");
		}
		updateGeometry();
		
		
		QList<VExtractionItem*> extList;
		foreach(QGraphicsItem* item, items)
		{
			ExtractionItem* tempItem=qgraphicsitem_cast<ExtractionItem*>(item);
			if(tempItem!=NULL)
			{
				extList.push_back(tempItem->parentItem);
			}
		}
		if(!extList.isEmpty())
		{
			extractorView->setExtractorParameters(extList);
			tabWidget->addTab(extractorView, "Extractor");
		}
		
		joinView->setJoinParameters(withItems);
		tabWidget->addTab(joinView, "Timeline/Join");
		
		
		tabWidget->addTab(selectionView, "Selection");
	}
	
	int index=tabWidget->indexOf(current);
	if(index>=0)
	{
		tabWidget->setCurrentWidget(current);
	}
	updateGeometry();
}


void ContextWidget::showItemTab(QList<QGraphicsItem *> items)
{
	bool joinItem=false;
    bool extractItem=false;
    bool bbItem=false;
	
	foreach(QGraphicsItem* item, items)
	{
		InputItem* tempItem=qgraphicsitem_cast<InputItem*>(item);
		if(tempItem!=NULL) joinItem = true;
	
		JoinItem* tempItem2=qgraphicsitem_cast<JoinItem*>(item);
		if(tempItem2!=NULL) joinItem = true;
		
		ExtractionItem* tempItem3=qgraphicsitem_cast<ExtractionItem*>(item);
		if(tempItem3!=NULL) extractItem = true;
	
        OperatorItem* tempItem4=qgraphicsitem_cast<OperatorItem*>(item);
        if(tempItem4!=NULL && tempItem4->parentItem->isBlackBox) bbItem = true;
	}
	if(joinItem){
		tabWidget->setCurrentWidget(joinView);		
    }
    else if(extractItem){
        tabWidget->setCurrentWidget(extractorView);
    }
    else if(bbItem){
        tabWidget->setCurrentWidget(bbView);
    }
	
	
}


void ContextWidget::setTimeLine( QString& name)
{
    joinView->setTimeLineTab(name);
	
}

void ContextWidget::newInput(const QString& text)
{
	if(joinView->_TagSource->text().isEmpty()) joinView->_TagSource->setText(text);
	
}

void ContextWidget::clearTT()
{
	joinView->_TagSource->setText("");
	
}

//-------------------------------


BlackBoxContextWidget::BlackBoxContextWidget(QWidget* parent, CalcView* view):
    QScrollArea(parent)
{
	
    QWidget* mainWidget=new QWidget;
    blackBox=NULL;
    inputDescriptions.clear();
    outputDescriptions.clear();

    inputWidget=new QWidget;
    outputWidget=new QWidget;
    inputLayout=new QVBoxLayout;
    outputLayout=new QVBoxLayout;
    inputWidget->setLayout(inputLayout);
    outputWidget->setLayout(outputLayout);

    QVBoxLayout* layout=new QVBoxLayout;

    QTabWidget* tab=new QTabWidget;
    
    tab->addTab(inputWidget, "Inputs");
    tab->addTab(outputWidget, "Outputs");

	QHBoxLayout* hLayout=new QHBoxLayout;
	hLayout->addWidget(new QLabel("Name:"));
	nameEditor=new QLineEdit;
	hLayout->addWidget(nameEditor);
	layout->addLayout(hLayout);
    connect(nameEditor, SIGNAL(editingFinished()), this, SLOT(updateName()));

    QPushButton * addToMenuButton=new QPushButton("Save to Library");
    connect(addToMenuButton, SIGNAL(clicked()), view, SLOT(saveBlackBoxToLibrary()));
    layout->addWidget(addToMenuButton);


    layout->addWidget(tab);
	

    mainWidget->setLayout(layout);
    setWidgetResizable(true);
    setWidget(mainWidget);
}

void BlackBoxContextWidget::setBlackBoxInfo(VOperatorItem* bb)
{

    foreach(QTextEdit* edit, inputDescriptions)
    {
        inputLayout->removeWidget(edit);
        delete edit;
    }
    foreach(QTextEdit* edit, outputDescriptions)
    {
        outputLayout->removeWidget(edit);
        delete edit;
    }
    foreach(QLabel* label, inputLabels)
    {
        inputLayout->removeWidget(label);
        delete label;
    }
    foreach(QLabel* label, outputLabels)
    {
        outputLayout->removeWidget(label);
        delete label;
    }
    foreach(QGroupBox* box, outputBoxes)
    {
        outputLayout->removeWidget(box);
        delete box;
    }
    foreach(QGroupBox* box, inputBoxes)
    {
        inputLayout->removeWidget(box);
        delete box;
    }

    inputDescriptions.clear();
    outputDescriptions.clear();
    inputLabels.clear();
    outputLabels.clear();
    inputBoxes.clear();
    outputBoxes.clear();
	
    if(bb->isBlackBox)
    {
        blackBox=bb;
        
        for(int i=0; i<blackBox->inputs.size(); ++i)
        {
            QTextEdit* text=new QTextEdit;
    		connect(text, SIGNAL(textChanged()), this, SLOT(updateInputTexts()));

            text->setText(blackBox->inputs[i]->getDescription());
            inputDescriptions.push_back(text);

			QGroupBox* box=new QGroupBox(QString("Input #%1").arg(i));
			QVBoxLayout* layout=new QVBoxLayout;
			layout->addWidget(new QLabel("Description"));
            layout->addWidget(text);
			box->setLayout(layout);
			inputLayout->addWidget(box);
			inputBoxes.push_back(box);
        }

        for(int i=0; i<blackBox->outputs.size(); ++i)
        {
           QTextEdit* text=new QTextEdit;
    		connect(text, SIGNAL(textChanged()), this, SLOT(updateOutputTexts()));

            text->setText(blackBox->outputs[i]->getDescription());
            outputDescriptions.push_back(text);

			QGroupBox* box=new QGroupBox(QString("Output #%1").arg(i));
			QVBoxLayout* layout=new QVBoxLayout;
			layout->addWidget(new QLabel("Description"));
            layout->addWidget(text);
			box->setLayout(layout);
			outputLayout->addWidget(box);
			outputBoxes.push_back(box);
        }
		nameEditor->setText(bb->getName());
    }
}


void BlackBoxContextWidget::updateName()
{
   	blackBox->setName(nameEditor->text());
	if(blackBox->hasGui())
	{
		blackBox->getGUI()->setTitle(nameEditor->text());
	}
}


void BlackBoxContextWidget::updateOutputTexts()
{
    if(outputDescriptions.size()!=blackBox->outputs.size()) return;
    
    for(int i=0; i<blackBox->outputs.size(); ++i)
    {
        blackBox->outputs[i]->setDescription(outputDescriptions[i]->toPlainText());
    }
}


void BlackBoxContextWidget::updateInputTexts()
{
    if(inputDescriptions.size()!=blackBox->inputs.size()) return;
    
    for(int i=0; i<blackBox->inputs.size(); ++i)
    {
        blackBox->inputs[i]->setDescription(inputDescriptions[i]->toPlainText());
    }

}

//-----------------------------------



IOContextWidget::IOContextWidget(QWidget* parent, CalcView* view):
QScrollArea(parent)
{
    
    QWidget* mainWidget=new QWidget;
    IOitem=NULL;
    inputDescriptions.clear();
    outputDescriptions.clear();
    
    inputWidget=new QWidget;
    outputWidget=new QWidget;
    inputLayout=new QVBoxLayout;
    outputLayout=new QVBoxLayout;
    inputWidget->setLayout(inputLayout);
    outputWidget->setLayout(outputLayout);
    
    QVBoxLayout* layout=new QVBoxLayout;
    
    QTabWidget* tab=new QTabWidget;
    
    tab->addTab(inputWidget, "Inputs");
    tab->addTab(outputWidget, "Outputs");
    
    QHBoxLayout* hLayout=new QHBoxLayout;
    hLayout->addWidget(new QLabel("Name:"));
    nameEditor=new QLineEdit;
    hLayout->addWidget(nameEditor);
    layout->addLayout(hLayout);
    connect(nameEditor, SIGNAL(editingFinished()), this, SLOT(updateName()));
    
    layout->addWidget(tab);
    
    
    mainWidget->setLayout(layout);
    setWidgetResizable(true);
    setWidget(mainWidget);
}

void IOContextWidget::setInfo(VCalcItem* item)
{
    
    foreach(QTextEdit* edit, inputDescriptions)
    {
        inputLayout->removeWidget(edit);
        delete edit;
    }
    foreach(QTextEdit* edit, outputDescriptions)
    {
        outputLayout->removeWidget(edit);
        delete edit;
    }
    foreach(QLabel* label, inputLabels)
    {
        inputLayout->removeWidget(label);
        delete label;
    }
    foreach(QLabel* label, outputLabels)
    {
        outputLayout->removeWidget(label);
        delete label;
    }
    foreach(QGroupBox* box, outputBoxes)
    {
        outputLayout->removeWidget(box);
        delete box;
    }
    foreach(QGroupBox* box, inputBoxes)
    {
        inputLayout->removeWidget(box);
        delete box;
    }
    
    inputDescriptions.clear();
    outputDescriptions.clear();
    inputLabels.clear();
    outputLabels.clear();
    inputBoxes.clear();
    outputBoxes.clear();
    
    VInputItem * Vin = dynamic_cast <VInputItem*>(item);
    VOutputItem * Vout = dynamic_cast <VOutputItem*>(item);
    
    if(Vin || Vout)
    {
        IOitem = item;
        
        for(int i=0; i<IOitem->inputs.size(); ++i)
        {
            QTextEdit* text=new QTextEdit;
            connect(text, SIGNAL(textChanged()), this, SLOT(updateInputTexts()));
            
            text->setText(IOitem->inputs[i]->getDescription());
            inputDescriptions.push_back(text);
            
            QGroupBox* box=new QGroupBox(QString("Input #%1").arg(i));
            QVBoxLayout* layout=new QVBoxLayout;
            layout->addWidget(new QLabel("Description"));
            layout->addWidget(text);
            box->setLayout(layout);
            inputLayout->addWidget(box);
            inputBoxes.push_back(box);
        }
        
        for(int i=0; i<IOitem->outputs.size(); ++i)
        {
            QTextEdit* text=new QTextEdit;
            connect(text, SIGNAL(textChanged()), this, SLOT(updateOutputTexts()));
            
            text->setText(IOitem->outputs[i]->getDescription());
            outputDescriptions.push_back(text);
            
            QGroupBox* box=new QGroupBox(QString("Output #%1").arg(i));
            QVBoxLayout* layout=new QVBoxLayout;
            layout->addWidget(new QLabel("Description"));
            layout->addWidget(text);
            box->setLayout(layout);
            outputLayout->addWidget(box);
            outputBoxes.push_back(box);
        }
        nameEditor->setText(IOitem->getName());
    }
}


void IOContextWidget::updateName()
{
    IOitem->setName(nameEditor->text());
    
    VInputItem *in = dynamic_cast<VInputItem*>(IOitem);
    VOutputItem *out = dynamic_cast<VOutputItem*>(IOitem);
    
    if(in && in->hasGui())
    {
        in->getGUI()->setTitle(nameEditor->text());
    }
    else if(out && out->hasGui())
    {
        out->getGUI()->setTitle(nameEditor->text());
    }
}


void IOContextWidget::updateOutputTexts()
{
    if(outputDescriptions.size()!=IOitem->outputs.size()) return;
    
    for(int i=0; i<IOitem->outputs.size(); ++i)
    {
        IOitem->outputs[i]->setDescription(outputDescriptions[i]->toPlainText());
    }
}


void IOContextWidget::updateInputTexts()
{
    if(inputDescriptions.size()!=IOitem->inputs.size()) return;
    
    for(int i=0; i<IOitem->inputs.size(); ++i)
    {
        IOitem->inputs[i]->setDescription(inputDescriptions[i]->toPlainText());
    }
    
}


// --------------------------------------------------

PluginContextWidget::PluginContextWidget(QWidget* parent):
    QTabWidget(parent)
{
//     QTabWidget* tabWidget=new QTabWidget(this);

    ioWidget=new QTextEdit; ioWidget->setReadOnly(true);
    helpWidget=new QTextEdit; helpWidget->setReadOnly(true);
    docWidget=new QTextBrowser;

    /*tabWidget->*/addTab(ioWidget, "Plugin I/O");
    /*tabWidget->*/addTab(helpWidget, "Help");
    /*tabWidget->*/addTab(docWidget, "Documentation");
}

void PluginContextWidget::setPluginInfo(VOperatorItem* plugin)
{
    ioWidget->clear();
    helpWidget->clear();
    docWidget->clear();
    if(plugin->isPlugin())
    {
        QString io, help, doc;
        plugin->getPluginInfo(io, help, doc);

        ioWidget->setHtml(io);
        helpWidget->setText(help);
//         docWidget->setText(doc);

        FILE *fp;
        fp = fopen(doc.toLatin1().data(), "r");
        if ( fp != NULL )
        {
            fclose (fp);
            docWidget->setSource(doc);
        }
    }
}

JoinContextWidget::JoinContextWidget(QWidget* parent, CalcView * view):
	QWidget(parent)
{
	_view = view;
	
	QGridLayout * join_layout=new QGridLayout(this);
	join_layout->setSpacing(4);
	
	QLabel* algorithm_label = new QLabel("Join Algorithm ");
	join_layout->addWidget( algorithm_label, 0, 0, Qt::AlignRight );
		
	_algorithm = new QComboBox(this);
	_algorithm->addItem(DV_LINEAR);
    _algorithm->addItem(DV_BOXCAR);
    _algorithm->addItem(DV_NN);
	join_layout->addWidget( _algorithm, 0, 1, Qt::AlignLeft );
	_algorithm->setMinimumWidth(150);
				
	QLabel* width_label = new QLabel("Width ",this);
	join_layout->addWidget( width_label, 0, 2, Qt::AlignRight );
			
	_width_field = new QguWLinput( this, "width_field", 8 );
	_width_field->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	_width_field->setEnabled(false);

	QLabel* width_labelSec = new QLabel("(sec)",this);
	join_layout->addWidget( width_labelSec, 0, 4, Qt::AlignLeft );
	
	_width_field->setToolTip("Width of the boxcar (Time is in seconds)");
	join_layout->addWidget( _width_field, 0, 3 );
			
	
			
    QLabel* gap_label = new QLabel("Gap Handling ",this);
    join_layout->addWidget( gap_label, 1, 0, Qt::AlignRight );
    
    _gap = new QComboBox(this);
    _gap->addItem(DV_LINEAR);
    _gap->addItem(DV_ZERO_FILL);
    _gap->addItem(DV_FILL);
    _gap->addItem(DV_NN);
    _gap->addItem(DV_REMOVE);
    _gap->setMinimumWidth(150);
    join_layout->addWidget( _gap, 1, 1, Qt::AlignLeft );
    
    QLabel* end_label = new QLabel("End Handling ",this);
    join_layout->addWidget( end_label, 2, 0, Qt::AlignRight );
    
    _end = new QComboBox(this);
    _end->addItem(DV_LINEAR);
    _end->addItem(DV_ZERO_FILL);
    _end->addItem(DV_FILL);
    _end->addItem(DV_NN);
    _end->addItem(DV_REMOVE);
    _end->setMinimumWidth(150);
    _end->setCurrentIndex(3);
    join_layout->addWidget( _end, 2, 1, Qt::AlignLeft );
    
    QLabel* min_box_label = new QLabel("Min Boxcar Num ",this);
    join_layout->addWidget( min_box_label, 2, 2, Qt::AlignRight );
    
    _box_min_field = new QguWLinput( this, "min_box_field", 8 );
    _box_min_field->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
    _box_min_field->setEnabled(false);
    _box_min_field->setText("3");
    
    _box_min_field->setToolTip("minimum number of points in the boxcar");
    join_layout->addWidget( _box_min_field, 2, 3 );
    
    
    
	QLabel* gapSize_label = new QLabel(" Gap threshold ",this);
	join_layout->addWidget( gapSize_label, 1, 2, Qt::AlignRight );
			
	_gap_field = new QguWLinput( this, "gap_field", 8 );
	_gap_field->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));

	_gap_field->setToolTip("minimum interval without data to be treated as a gap (time is in seconds)");
	join_layout->addWidget( _gap_field, 1, 3 );

	QLabel* gap_labelSec = new QLabel("(sec)",this);
	join_layout->addWidget( gap_labelSec, 1, 4, Qt::AlignLeft );
	
	warningLabel=new QLabel("Multiple items selected. Values are from ONE, but changes affect all selected items");
	warningLabel->setWordWrap(true);
	join_layout->addWidget(warningLabel,2,0,1,5);

	connect(_algorithm, SIGNAL(activated(int)), this, SLOT(setAlgorithm()));
	connect(_width_field, SIGNAL(edited_OK()), this, SLOT(setWidthField()));
    connect(_gap, SIGNAL(activated(int)), this, SLOT(setGap()));
    connect(_end, SIGNAL(activated(int)), this, SLOT(setEnd()));
    connect(_gap_field, SIGNAL(edited_OK()), this, SLOT(setGapField()));
    connect(_box_min_field, SIGNAL(edited_OK()), this, SLOT(setMinBoxField()));
	
	
	QLabel* time_label = new QLabel("  Timeline   (applies to whole page)");
	join_layout->addWidget( time_label, 4, 0, 1, 4, Qt::AlignBottom|Qt::AlignLeft );
			
	// Timeline stuff (applies to whole page)
	
	tlBox=new QComboBox(this);
	tlBox->setMinimumWidth(250);
	join_layout->addWidget(tlBox, 5, 0, 1, 3, Qt::AlignLeft);
	
	saveAsButton=new QPushButton("Save as...");
	connect(saveAsButton, SIGNAL(clicked()), this, SLOT(saveAs()));
	join_layout->addWidget(saveAsButton, 5, 3, Qt::AlignLeft);

	deleteButton=new QPushButton("Delete");
	connect(deleteButton, SIGNAL(clicked()), this, SLOT(deleteSelected()));
	join_layout->addWidget(deleteButton, 5, 4, Qt::AlignLeft);
	deleteButton->setEnabled(false);
	
	
	tlBox->addItem("Automatic");
	QList<QString> names=timeLineHandler.getNames();
	foreach(QString name, names)
	{
		tlBox->addItem(name);
	}
	connect(tlBox, SIGNAL(currentIndexChanged(const QString&)), this, SLOT(newSelection(const QString&)));	

	connect(&timeLineHandler, SIGNAL(addedTimeLine(const QString&)), this, SLOT(addedTimeLine(const QString&)));
	connect(&timeLineHandler, SIGNAL(deletedTimeLine(const QString&)), this, SLOT(deletedTimeLine(const QString&)));
	connect(&timeLineHandler, SIGNAL(timeLineChanged(const QString&)), this, SLOT(setTimeLineTab(const QString&)));
	
    QLabel* TimeInterval_label = new QLabel("Interval ",this);
    join_layout->addWidget( TimeInterval_label, 6, 0, Qt::AlignRight );
        
    _TimeIvlSelect = new QComboBox(this);
    _TimeIvlSelect->insertItem(0, "from Data");
    _TimeIvlSelect->insertItem(1, "from Object");
	_TimeIvlSelect->setContentsMargins(0,0,0,0);
	_TimeIvlSelect->setMinimumWidth(150);
    join_layout->addWidget( _TimeIvlSelect, 6, 1, Qt::AlignLeft );

        
    IvlSource_label = new QLabel("  ",this);
    join_layout->addWidget( IvlSource_label, 6, 2, Qt::AlignRight );
        
    _IvlSource = new QguWLinput( this, "IvlSource", 23 );
	_IvlSource->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));
	_IvlSource->addAcceptable(Qcondition(QS_NARROW, QS_ANY, QS_ANY, QS_SEQ, QS_ANY));
    _IvlSource->active = false;
	_IvlSource->setVisible(false);
        
    _IvlSource->setToolTip("not active");
    join_layout->addWidget( _IvlSource, 6, 3, 1, 2, Qt::AlignLeft );
    
    QLabel* TimeTags_label = new QLabel("Target Tags ",this);
    join_layout->addWidget( TimeTags_label, 7, 0, Qt::AlignRight );
        
    _TimeTagSelect = new QComboBox(this);
    _TimeTagSelect->insertItem(0, "Create Regular");
    _TimeTagSelect->insertItem(1, "from Object");
	_TimeTagSelect->setContentsMargins(1,1,1,1);
	_TimeTagSelect->setMinimumWidth(150);
    join_layout->addWidget( _TimeTagSelect, 7, 1, Qt::AlignLeft );
    
        
    TagSource_label = new QLabel(" Tag Object ",this);
    join_layout->addWidget( TagSource_label, 7, 2, Qt::AlignRight );
        
    _TagSource = new QguWLinput( this, "TagSource", 23 );
	_TagSource->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_SEQ, QS_ANY));
    _TagSource->addAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_SEQ, QS_ANY));
    
    _TagSource->setToolTip("drag and drop object here");
    join_layout->addWidget( _TagSource, 7, 3, 1, 2, Qt::AlignLeft );

	connect(_TimeIvlSelect, SIGNAL(currentIndexChanged(int)), this, SLOT(setTTivl(int)));
	connect(_IvlSource, SIGNAL(edited_OK()), this, SLOT(saveChanges()));

	connect(_TimeTagSelect, SIGNAL(currentIndexChanged(int)), this, SLOT(setTTsource(int)));
	connect(_TagSource, SIGNAL(edited_OK()), this, SLOT(saveChanges()));
		
	setLayout(join_layout);
	
	newSelection("Automatic");

}

QList<VInputItem*> JoinContextWidget::listInputs(){

	QList<QGraphicsItem*> items = _view->scene()->selectedItems(); // otherwise empty list

	QList<VInputItem*> inputs;
	foreach(QGraphicsItem* item, items)
	{
		InputItem* tempItem=qgraphicsitem_cast<InputItem*>(item);
		if(tempItem!=NULL)
		{
			inputs.push_back(tempItem->parentItem);
		}
	}
	return inputs;

}

QList<VJoinItem*> JoinContextWidget::listJoins(){

	QList<QGraphicsItem*> items = _view->scene()->selectedItems(); // otherwise empty list

	QList<VJoinItem*> joins;
	foreach(QGraphicsItem* item, items)
	{
		JoinItem* tempItem=qgraphicsitem_cast<JoinItem*>(item);
		if(tempItem!=NULL)
		{
			joins.push_back(tempItem->parentItem);
		}
	}
	return joins;	
}

void JoinContextWidget::setJoinParameters(bool withItems)
{
	_algorithm->blockSignals(true);
	_width_field->blockSignals(true);
	_gap->blockSignals(true);
    _gap_field->blockSignals(true);
    _end->blockSignals(true);
	
	if(withItems){
        QList<VInputItem*> inputItems = listInputs();
        QList<VJoinItem*>  joinItems  = listJoins();
		
        warningLabel->setVisible(inputItems.size()+joinItems.size()>1);
	
        if(!inputItems.isEmpty())
        {
            _algorithm->setCurrentIndex(inputItems[0]->getAlgorithm());

            if(tlBox->currentText()!="Automatic") _width_field->setText(QString("%1").arg(inputItems[0]->getWidth()));
            else _width_field->setText(QString(" "));

            _gap->setCurrentIndex(inputItems[0]->getGap());
        
            _end->setCurrentIndex(inputItems[0]->getEnd());
        
            _box_min_field->setText(QString("%1").arg(inputItems[0]->getMinBoxSize()));
        
            if(tlBox->currentText()!="Automatic") _gap_field->setText(QString("%1").arg(inputItems[0]->getGapSize()));

        }
        else if(!joinItems.isEmpty())
        {
            _algorithm->setCurrentIndex(joinItems[0]->getAlgorithm());
        
            if(tlBox->currentText()!="Automatic") _width_field->setText(QString("%1").arg(joinItems[0]->getWidth()));
            else _width_field->setText(QString(" "));
        
            _gap->setCurrentIndex(joinItems[0]->getGap());

            _end->setCurrentIndex(joinItems[0]->getEnd());
        
            _box_min_field->setText(QString("%1").arg(joinItems[0]->getMinBoxSize()));
        
            if(tlBox->currentText()!="Automatic") _gap_field->setText(QString("%1").arg(joinItems[0]->getGapSize()));
            else _gap_field->setText(QString(" "));
 		
        }
    }
	_algorithm->blockSignals(false);
	_width_field->blockSignals(false);
	_gap->blockSignals(false);
    _gap_field->blockSignals(false);
    _end->blockSignals(false);

}

void JoinContextWidget::setJoinStatus(bool status)
{
	
	// join stuff
	if(_algorithm->currentText() == DV_BOXCAR) _gap_field->setToolTip("gap is when there are too few points in boxcar");
	else _gap_field->setToolTip("minimum interval without data to be treated as a gap (time is in seconds)");

	_width_field->setEnabled(status && _algorithm->currentText() == DV_BOXCAR);
	_gap->setEnabled(status);
    _gap_field->setEnabled(status && _algorithm->currentText() != DV_BOXCAR);
    _box_min_field->setEnabled(status && _algorithm->currentText() == DV_BOXCAR);
	_algorithm->setEnabled(status);
    _end->setEnabled(status);
    
	// Timeline stuff
	_TimeIvlSelect->setEnabled(status);
	_TimeTagSelect->setEnabled(status);
	_TagSource->setEnabled(status);
	_IvlSource->setEnabled(status);
	saveAsButton->setEnabled(status);


	setJoinParameters(true); // updates field contents

}


void JoinContextWidget::setAlgorithm()
{
	QList<VInputItem*> inputItems = listInputs();
	QList<VJoinItem*>  joinItems  = listJoins();

    foreach(VInputItem* input, inputItems)
	{
		input->setAlgorithm(_algorithm->currentIndex());
	}
	foreach(VJoinItem* join, joinItems)
	{
		join->setAlgorithm(_algorithm->currentIndex());
	}
	setJoinStatus(tlBox->currentText() != "Automatic");
	
//    _width_field->setEnabled(_algorithm->currentText() == DV_BOXCAR && tlBox->currentText() != "Automatic");
 //   _box_min_field->setEnabled(_algorithm->currentText() == DV_BOXCAR && tlBox->currentText() != "Automatic");
}

void JoinContextWidget::setWidthField()
{
	QList<VInputItem*> inputItems = listInputs();
	QList<VJoinItem*>  joinItems  = listJoins();

    foreach(VInputItem* input, inputItems)
	{
		input->setWidth(_width_field->text().toDouble());
	}
	foreach(VJoinItem* join, joinItems)
	{
		join->setWidth(_width_field->text().toDouble());
	}

}

void JoinContextWidget::setGap()
{
    QList<VInputItem*> inputItems = listInputs();
    QList<VJoinItem*>  joinItems  = listJoins();
    
    foreach(VInputItem* input, inputItems)
    {
        input->setGap(_gap->currentIndex());
    }
    foreach(VJoinItem* join, joinItems)
    {
        join->setGap(_gap->currentIndex());
    }
}

void JoinContextWidget::setEnd()
{
    QList<VInputItem*> inputItems = listInputs();
    QList<VJoinItem*>  joinItems  = listJoins();
    
    foreach(VInputItem* input, inputItems)
    {
        input->setEnd(_end->currentIndex());
    }
    foreach(VJoinItem* join, joinItems)
    {
        join->setEnd(_end->currentIndex());
    }
}

void JoinContextWidget::setGapField()
{
    QList<VInputItem*> inputItems = listInputs();
    QList<VJoinItem*>  joinItems  = listJoins();
    
    foreach(VInputItem* input, inputItems)
    {
        input->setGapSize(_gap_field->text().toDouble());
    }
    foreach(VJoinItem* join, joinItems)
    {
        join->setGapSize(_gap_field->text().toDouble());
    }
}

void JoinContextWidget::setMinBoxField()
{
    QList<VInputItem*> inputItems = listInputs();
    QList<VJoinItem*>  joinItems  = listJoins();
    
    foreach(VInputItem* input, inputItems)
    {
        input->setMinBoxSize(_box_min_field->text().toInt());
    }
    foreach(VJoinItem* join, joinItems)
    {
        join->setMinBoxSize(_box_min_field->text().toInt());
    }
}

void JoinContextWidget::addedTimeLine(const QString& newName)
{
	tlBox->blockSignals(true);
	//clear
	tlBox->clear();
	tlBox->addItem("Automatic");
	//fill
	QList<QString> names=timeLineHandler.getNames();
	foreach(QString name, names)
	{
		tlBox->addItem(name);
	}

	tlBox->blockSignals(false);
	
	int bkpIndex=tlBox->findText(newName);
	if(bkpIndex>=0)
		tlBox->setCurrentIndex(bkpIndex);
	

}


void JoinContextWidget::deletedTimeLine(const QString&)
{
	QString save=tlBox->currentText();
	tlBox->blockSignals(true);
	//clear
	tlBox->clear();
	tlBox->addItem("Automatic");
	//fill
	QList<QString> names=timeLineHandler.getNames();
	foreach(QString name, names)
	{
		tlBox->addItem(name);
	}
	tlBox->blockSignals(false);
	int bkpIndex=tlBox->findText(save);
	if(bkpIndex>=0)
		tlBox->setCurrentIndex(bkpIndex);
}


void JoinContextWidget::setTimeLineTab(const QString name){

	// just show timeline in use
	int bkpIndex=tlBox->findText(name);
	if(bkpIndex>=0) tlBox->setCurrentIndex(bkpIndex);

	newSelection(name);
	
}



void JoinContextWidget::newSelection(const QString& name)
{
	if(name=="Target" || name =="Automatic") deleteButton->setEnabled(false);
	else deleteButton->setEnabled(true);
	
	_TimeIvlSelect->blockSignals(true);
	_TimeTagSelect->blockSignals(true);
	_TagSource->blockSignals(true);
	_IvlSource->blockSignals(true);

	setJoinStatus( (name!="Automatic") ) ;

	TimeLine* tl=timeLineHandler.getTimeLine(name);
	if(tl!=NULL)
	{
		_TimeIvlSelect->setCurrentIndex((tl->fromData?0:1));
		setTTivlTab((tl->fromData?0:1));
		_TimeTagSelect->setCurrentIndex((tl->regularTT?0:1));
		setTTsourceTab((tl->regularTT?0:1));
		_IvlSource->setText(tl->fromObjString);
		_TagSource->setText(tl->TTString);

	}
	
	// tell page new name (also sets noJoin flag in page input items)
	((CalcScene*)_view->scene())->setTimeLine(name);
	
	_TimeIvlSelect->blockSignals(false);
	_TimeTagSelect->blockSignals(false);
	_TagSource->blockSignals(false);
	_IvlSource->blockSignals(false);

}


void JoinContextWidget::deleteSelected()
{
	QString name=tlBox->currentText();
	if(name!="Target")
	{
		timeLineHandler.deleteTimeLine(name);
	}
}



void JoinContextWidget::saveChanges()
{
	QString name=tlBox->currentText();

	TimeLine* tl=timeLineHandler.getTimeLine(name);
	if(tl!=NULL)
	{
		tl->fromData=(_TimeIvlSelect->currentIndex()==0);
		tl->regularTT=(_TimeTagSelect->currentIndex()==0);
		tl->fromObjString=_IvlSource->text();
		tl->TTString=_TagSource->text();
	}
}

void JoinContextWidget::saveAs()
{
	bool ok;
	QString text = QInputDialog::getText(this, tr("Enter name for Time/Scalar Dependency"),
                                          tr("New name:"), QLineEdit::Normal,
                                          "New_Name", &ok);
	text=text.trimmed();
	if (ok && !text.isEmpty())
	{
        QList<QString> names=timeLineHandler.getNames();
		while(names.contains(text))
		{
			text.append("A");
		}
		timeLineHandler.addTimeLine(text, (_TimeIvlSelect->currentIndex()==0),
									(_TimeTagSelect->currentIndex()==0),
									 _IvlSource->text(), _TagSource->text());
									 
		qApp->processEvents();
		
		int bkpIndex=tlBox->findText(text);
		if(bkpIndex>=0)
			tlBox->setCurrentIndex(bkpIndex);
	}
}


void JoinContextWidget::setTTivl(int i)
{
	setTTivlTab(i);
	saveChanges();
}

void JoinContextWidget::setTTivlTab(int i)
{
	if(i == 0)
	{
		// from data
		IvlSource_label->setText(" ");
		_IvlSource->setToolTip("not active");
		_IvlSource->active = false;
		_IvlSource->setText(" "); // force signal so that colour updates
		_IvlSource->setVisible(false);
	}
	else if(i == 1)
	{
		IvlSource_label->setText("Interval Obj");
		_IvlSource->setToolTip("drag and drop object here");
		_IvlSource->active = true;
		_IvlSource->setVisible(true);
		_IvlSource->setText("");
		_IvlSource->reValidate(true);
	}
}

void JoinContextWidget::setTTsource(int i)
{
	setTTsourceTab(i);
	
	_TagSource->reValidate(true); // updates T display if valid
	saveChanges();
}

void JoinContextWidget::setTTsourceTab(int i)
{
	if(i == 0)
	{
		// create regular
		TagSource_label->setText(" Spacing");
		_TagSource->setToolTip("Spacing between regular tags (time is in seconds)");
		_TagSource->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	}
	else if(i == 1)
	{
		TagSource_label->setText(" Tag Object");
		_TagSource->setToolTip("Uses DEPEND_0 of this object as target time or scalar sequence to join onto");
		_TagSource->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_SEQ, QS_ANY));
	    _TagSource->addAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_SEQ, QS_ANY));
	}
	
}

// ------------------------------------------------

DefaultContextWidget::DefaultContextWidget(QWidget* parent, CalcView* view):
    QWidget(parent)
{
	this->view=view;
    QVBoxLayout * mainLayout=new QVBoxLayout;
	
	QGroupBox * addBox=new QGroupBox("Add item");
	QGridLayout * add_layout=new QGridLayout(addBox);
    add_layout->setSpacing(20);
    add_layout->setColumnStretch(1, 10);
	mainLayout->addWidget(addBox);
	QPushButton * inputButton=new QPushButton("Input");
	inputButton->setMinimumWidth(150);
	connect(inputButton, SIGNAL(clicked()), this, SLOT(addInput()));
	add_layout->addWidget(inputButton, 0, 1, Qt::AlignLeft );
	QPushButton * outputButton=new QPushButton("Output");
	outputButton->setMinimumWidth(150);
	connect(outputButton, SIGNAL(clicked()), this, SLOT(addOutput()));
	add_layout->addWidget(outputButton, 0, 2, Qt::AlignLeft );
	QPushButton * extractorButton=new QPushButton("Extractor");
	extractorButton->setMinimumWidth(150);
	connect(extractorButton, SIGNAL(clicked()), this, SLOT(addExtractor()));
	add_layout->addWidget(extractorButton, 1, 1, Qt::AlignLeft );
	QPushButton * joinButton=new QPushButton("Join/Resample");
	joinButton->setMinimumWidth(150);
	connect(joinButton, SIGNAL(clicked()), this, SLOT(addJoin()));
	add_layout->addWidget(joinButton, 1, 2, Qt::AlignLeft );
	
	QLabel *blank = new QLabel(" ");
	add_layout->addWidget(blank, 2, 0, 1, 2, Qt::AlignLeft );

	QLabel *blank2 = new QLabel("                 ");
	blank2->setMinimumWidth(200);
	add_layout->addWidget(blank2, 0, 0, Qt::AlignLeft );

	QPushButton* operatorButton=new QPushButton("Operator");
	operatorButton->setMinimumWidth(150);
	add_layout->addWidget(operatorButton, 3, 1, Qt::AlignLeft );
	connect(operatorButton, SIGNAL(pressed()), view, SLOT(autoSelectPosition()));
	QMenu* operatorMenu=new QMenu(operatorButton);
	operatorButton->setMenu(operatorMenu);
	QMenu* binOp=operatorMenu->addMenu("Arith f(A,B)");
	binOp->addAction("A+B", view, SLOT(addOpAdd()));
	binOp->addAction("A-B", view, SLOT(addOpSub()));
	binOp->addAction("A*B", view, SLOT(addOpMul()));
	binOp->addAction("A/B", view, SLOT(addOpDiv()));
	binOp->addAction("A^B", view, SLOT(addOpPow()));
	binOp->addAction("A\%B", view, SLOT(addOpRemainder()));
	QMenu* unOp=operatorMenu->addMenu("Arith f(A)");
	unOp->addAction("abs(A)", view, SLOT(addOpAbs()));
	unOp->addAction("ln(A)", view, SLOT(addOpLog()));
	unOp->addAction("log(A)", view, SLOT(addOpLog10()));
	unOp->addAction("1/A", view, SLOT(addOpInverse()));
	unOp->addAction("-A", view, SLOT(addOpOpp()));
	unOp->addAction("sqrt(A)", view, SLOT(addOpSqrt()));
	unOp->addAction("exp(A)", view, SLOT(addOpExp()));
	QMenu* trigOp=operatorMenu->addMenu("Trig");
	trigOp->addAction("cos(A)", view, SLOT(addOpCos()));
	trigOp->addAction("sin(A)", view, SLOT(addOpSin()));
	trigOp->addAction("tan(A)", view, SLOT(addOpTan()));
	trigOp->addAction("cot(A)", view, SLOT(addOpCot()));
	trigOp->addAction("acos(A)", view, SLOT(addOpAcos()));
	trigOp->addAction("asin(A)", view, SLOT(addOpAsin()));
	trigOp->addAction("atan(A)", view, SLOT(addOpAtan()));
	trigOp->addAction("atan2(A,B)", view, SLOT(addOpAtan2()));
	trigOp->addAction("acot(A)", view, SLOT(addOpAcot()));
	trigOp->addAction("cosh(A)", view, SLOT(addOpCosh()));
	trigOp->addAction("sinh(A)", view, SLOT(addOpSinh()));
	trigOp->addAction("tanh(A)", view, SLOT(addOpTanh()));
	trigOp->addAction("coth(A)", view, SLOT(addOpCoth()));
	trigOp->addAction("acosh(A)", view, SLOT(addOpAcosh()));
	trigOp->addAction("asinh(A)", view, SLOT(addOpAsinh()));
	trigOp->addAction("atanh(A)", view, SLOT(addOpAtanh()));
	trigOp->addAction("acoth(A)", view, SLOT(addOpAcoth()));
	trigOp->addAction("(-180,180) to (0,360)", view, SLOT(addOpAngle360()));
	trigOp->addAction("(0,360) to (-180,180)", view, SLOT(addOpAngle180()));
	trigOp->addAction("To Radians", view, SLOT(addOpToRad()));
	trigOp->addAction("To Degrees", view, SLOT(addOpToDeg()));
    QMenu* vectOp=operatorMenu->addMenu("Vector Ops");
    vectOp->addAction("A.B", view, SLOT(addOpDotProd()));
    vectOp->addAction("AxB", view, SLOT(addOpVectProd()));
    vectOp->addAction("Normalize", view, SLOT(addOpNormalize()));
    vectOp->addAction("outer product", view, SLOT(addOpOuter()));
    view->addPluginMenu(vectOp, "Vectors", true);
    QMenu* MatOp=operatorMenu->addMenu("Array Ops");
    MatOp->addAction("Matrix Inverse", view, SLOT(addOpMatInv()));
    MatOp->addAction("trace(A)", view, SLOT(addOpTrace()));
    MatOp->addAction("inner product", view, SLOT(addOpInner()));
    MatOp->addAction("outer product", view, SLOT(addOpOuter()));
    MatOp->addAction("transpose(A)", view, SLOT(addOpTranspose()));
    MatOp->addAction("determinant(A)", view, SLOT(addOpDeterminant()));
    MatOp->addAction("adjoint(A)", view, SLOT(addOpAdjoint()));
    MatOp->addAction("element(i)", view, SLOT(addOpElement_i()));
    MatOp->addAction("element(i,j,k...)", view, SLOT(addOpElement_ijk()));
    MatOp->addAction("remove(i)", view, SLOT(addOpRemove_i()));
    MatOp->addAction("A(i,j) * B(i,j) -> C(i,j)", view, SLOT(addOpPiecewise()));
    MatOp->addAction("A(i,j) / B(i,j) -> C(i,j)", view, SLOT(addOpPwDiv()));
    QMenu* derivOp=operatorMenu->addMenu("Derivative");
    derivOp->addAction("3pt derivative", view, SLOT(addOpDiff3()));
    derivOp->addAction("5pt derivative", view, SLOT(addOpDiff5()));
	QMenu* integOp=operatorMenu->addMenu("Integral");
	integOp->addAction("Definite Integral", view, SLOT(addOpIntDef()));
	integOp->addAction("Indefinite Integral", view, SLOT(addOpIntIndef()));
	QMenu* modOp=operatorMenu->addMenu("Filtering");
    view->addPluginMenu(modOp, "Filtering", true);
	QMenu* unitOp=operatorMenu->addMenu("Change Units");
    view->addPluginMenu(unitOp, "UnitOperations", true);
	QMenu* timeOp=operatorMenu->addMenu("Time Operations");
    view->addPluginMenu(timeOp, "TimeOperations", true);
    timeOp->addAction("Timetags to seconds from offset", view, SLOT(addOpTTtoS()));
    timeOp->addAction("Seconds from offset to Timetags", view, SLOT(addOpStoTT()));
	timeOp->addAction("Time Interval to seconds", view, SLOT(addOpTIVLtoS()));
	timeOp->addAction("Time2 - Time1 in sec", view, SLOT(addOpT2_T1()));
	timeOp->addAction("Timetags2 - Timetags1 in seconds", view, SLOT(addOpTT2_TT1()));
    QMenu* createRegOp=operatorMenu->addMenu("Create Regular");
    createRegOp->addAction("Time", view, SLOT(addOpCreateRegTime()));
    createRegOp->addAction("Double", view, SLOT(addOpCreateRegDbl()));
    createRegOp->addAction("Int", view, SLOT(addOpCreateRegInt()));
	QMenu* statsOp=operatorMenu->addMenu("Statistics");
    view->addPluginMenu(statsOp, "Statistics", true);
	statsOp->addAction("Linear Regression", view, SLOT(addOpLinReg()));
	QMenu* eventsOp=operatorMenu->addMenu("Event Tables");
    view->addPluginMenu(eventsOp, "EventTables", true);
    QMenu* plugOp=operatorMenu->addMenu("Plug-ins");
    QMenu* geoOp=plugOp->addMenu("Geophysics");
    view->addPluginMenu(geoOp, "Geophysics", true);
    QMenu* anaOp=plugOp->addMenu("Analysis");
    view->addPluginMenu(anaOp, "Analysis", true);
	plugOp->addSeparator();
	plugOp->addAction("Browse...", view, SLOT(addPlugInBrowse()));
	
	bbOp=operatorMenu->addMenu("Black Box from library");
	view->addBlackBoxMenu(bbOp);
	connect(bbOp, SIGNAL(triggered(QAction*)), view, SLOT(addBB(QAction*)));
	
	connect(view, SIGNAL(updateBlackBoxMenu()), this, SLOT(updateBlackBoxMenu()));

	
	QPushButton* templateButton=new QPushButton("Template");
	templateButton->setMinimumWidth(150);
	add_layout->addWidget(templateButton, 3, 2, Qt::AlignLeft );
	connect(templateButton, SIGNAL(pressed()), view, SLOT(autoSelectPosition()));
	QMenu* templateMenu=new QMenu(templateButton);
	templateButton->setMenu(templateMenu);
	templateMenu->addAction("A+B", view, SLOT(addTemplateAdd()));
	templateMenu->addAction("A-B", view, SLOT(addTemplateSub()));
	templateMenu->addAction("A*B", view, SLOT(addTemplateMul()));
	templateMenu->addAction("A/B", view, SLOT(addTemplateDiv()));
	templateMenu->addAction("-A", view, SLOT(addTemplateOpp()));
    
	mainLayout->addStretch();

    setLayout(mainLayout);
}

void DefaultContextWidget::updateBlackBoxMenu()
{
	bbOp->clear();
	view->addBlackBoxMenu(bbOp);
}

void DefaultContextWidget::addInput()
{
	view->autoSelectPosition();
	view->addInputItem();
}

void DefaultContextWidget::addOutput()
{
	view->autoSelectPosition();
	view->addOutputItem();
}

void DefaultContextWidget::addExtractor()
{
	view->autoSelectPosition();
	view->addExtractionItem();
}

void DefaultContextWidget::addJoin()
{
	view->autoSelectPosition();
	view->addJoinItem();
}



SelectionContextWidget::SelectionContextWidget(QWidget* parent, CalcView* view):
    QWidget(parent)
{
	this->view=view;
    QVBoxLayout * mainLayout=new QVBoxLayout;
	
	QGroupBox * bbBox=new QGroupBox("Black box");
	QGridLayout * bb_layout=new QGridLayout(bbBox);
	bb_layout->setSpacing(4);
    bb_layout->setColumnStretch(1, 10);
	mainLayout->addWidget(bbBox);
	
	QPushButton * bbButton=new QPushButton("Set as black box");
	connect(bbButton, SIGNAL(clicked()), view, SLOT(changeForBlackBox()));
	bb_layout->addWidget(bbButton, 0, 0, Qt::AlignLeft );
	
	QGroupBox * aspectBox=new QGroupBox("Aspect");
	QGridLayout * aspect_layout=new QGridLayout(aspectBox);
	aspect_layout->setSpacing(4);
	mainLayout->addWidget(aspectBox);
	
	QPushButton * resetButton=new QPushButton("Reset");
	connect(resetButton, SIGNAL(clicked()), view, SLOT(resetAspect()));
	aspect_layout->addWidget(resetButton, 3, 4, Qt::AlignRight );
	
    QLabel *orientation = new QLabel("Orientation");
    aspect_layout->addWidget(orientation, 0, 0, Qt::AlignRight );

	QPushButton * hButton=new QPushButton("Horizontal");
	connect(hButton, SIGNAL(clicked()), view, SLOT(horizontalAspect()));
	aspect_layout->addWidget(hButton, 0, 1, Qt::AlignCenter );
	QPushButton * vButton=new QPushButton("Vertical");
	connect(vButton, SIGNAL(clicked()), view, SLOT(verticalAspect()));
	aspect_layout->addWidget(vButton, 0, 3, Qt::AlignCenter );
	
    QLabel *size = new QLabel("Size");
    aspect_layout->addWidget(size, 1, 0, Qt::AlignRight );
    
	QPushButton * hSizeButton=new QPushButton("1/2x");
	connect(hSizeButton, SIGNAL(clicked()), view, SLOT(halfSize()));
	aspect_layout->addWidget(hSizeButton, 1, 1, Qt::AlignCenter );
	QPushButton * nSizeButton=new QPushButton("Normal size");
	connect(nSizeButton, SIGNAL(clicked()), view, SLOT(normalSize()));
	aspect_layout->addWidget(nSizeButton, 1, 2, Qt::AlignCenter );
	QPushButton * dSizeButton=new QPushButton("2x");
	connect(dSizeButton, SIGNAL(clicked()), view, SLOT(doubleSize()));
	aspect_layout->addWidget(dSizeButton, 1, 3, Qt::AlignCenter );
	

	mainLayout->addStretch();

    setLayout(mainLayout);
}





ExtractorContextWidget::ExtractorContextWidget(ContextWidget* parent):
	QScrollArea((QWidget*)parent)
{
	QWidget* mainWidget=new QWidget;
    QVBoxLayout * mainLayout=new QVBoxLayout;
	
	ctxtWidget = parent;
	
	warningLabel=new QLabel("Multiple items selected. Values are from ONE item, changes affect all selected items");
	warningLabel->setWordWrap(true);
	mainLayout->addWidget(warningLabel);
	
	QGroupBox * outGBox=new QGroupBox("Output options");
	QGridLayout * outLayout=new QGridLayout(outGBox);
	
	target=new QComboBox();
	target->addItem("Scalar");
	target->addItem("1D Array");
	target->addItem("2D Array");
	target->addItem("3D Array");
	target->addItem("Text");
	target->addItem("Time");
	target->addItem("Time Interval");
	target->addItem("Time Tags");
	target->addItem("Metadata");
	connect(target, SIGNAL(currentIndexChanged(int)), this, SLOT(setTarget(int)) );
	
	QLabel * targetLabel = new QLabel("Output choice: ");
	outLayout->addWidget(targetLabel, 0, 0, Qt::AlignRight);
	outLayout->addWidget(target, 0, 1, Qt::AlignLeft);

	outGBox->setLayout(outLayout);
	mainLayout->addWidget(outGBox);
	
	compCheckBox=new QCheckBox("Ask user ");
	outLayout->addWidget(compCheckBox, 1, 0);
	ComponentButton=new QPushButton("Component");
	EventButton=new QPushButton("Event");
	MetaButton=new QPushButton("Metadata");
	ComponentButton->setEnabled(true);
	EventButton->setEnabled(true);
	MetaButton->setEnabled(true);
	
	talkBack = new QLabel("Calculation will pause at unresolved Extractors");
	outLayout->addWidget(talkBack, 1, 1, 1, 3);


	QAction* action;
	
	/* QMenu* */ComponentMenu0=new QMenu;
	action=ComponentMenu0->addAction("Default"); action->setData(QVariant(""));

	action=ComponentMenu0->addSeparator();
	// vectors
	action=ComponentMenu0->addAction("r (mag)"); action->setData(QVariant("r (mag)"));
	action=ComponentMenu0->addAction("x"); action->setData(QVariant("x"));
	action=ComponentMenu0->addAction("y"); action->setData(QVariant("y"));
	action=ComponentMenu0->addAction("z"); action->setData(QVariant("z"));
	action=ComponentMenu0->addAction("Lat (deg)"); action->setData(QVariant("Lat (deg)"));
	action=ComponentMenu0->addAction("theta (deg)"); action->setData(QVariant("theta (deg)"));
	action=ComponentMenu0->addAction("phi (deg)"); action->setData(QVariant("phi (deg)"));
	action=ComponentMenu0->addAction("Lat (rad)"); action->setData(QVariant("Lat (rad)"));
	action=ComponentMenu0->addAction("theta (rad)"); action->setData(QVariant("theta (rad)"));
	action=ComponentMenu0->addAction("phi (rad)"); action->setData(QVariant("phi (rad)"));
	action=ComponentMenu0->addSeparator();
	// Array
	action=ComponentMenu0->addAction("Syntax for 3D -> 0D"); action->setData(QVariant("Sum(i,j) Sum(i,j) Sum(i,j)"));
	action=ComponentMenu0->addAction("Syntax for 2D -> 0D"); action->setData(QVariant("Sum(i,j) Sum(i,j)"));
	action=ComponentMenu0->addAction("Syntax for 1D -> 0D"); action->setData(QVariant("Sum(i,j)"));
	action=ComponentMenu0->addSeparator();
	action=ComponentMenu0->addAction("Syntax for Metadata"); action->setData(QVariant("=> META_NAME"));
	ComponentButton->setMenu(ComponentMenu0);

	/* QMenu* */ComponentMenu1=new QMenu;
	action=ComponentMenu1->addAction("Default"); action->setData(QVariant(""));
	action=ComponentMenu1->addSeparator();
	// Array
	action=ComponentMenu1->addAction("Syntax for 3D -> 1D"); action->setData(QVariant("Sum(i,j) Sum(i,j) (i,j)"));
	action=ComponentMenu1->addAction("Syntax for 2D -> 1D"); action->setData(QVariant("Sum(i,j) (i,j)"));
	action=ComponentMenu1->addAction("Syntax for 1D range"); action->setData(QVariant("(i,j)"));
	action=ComponentMenu1->addSeparator();
	action=ComponentMenu1->addAction("Syntax for Metadata"); action->setData(QVariant("=> META_NAME"));

	/* QMenu* */ComponentMenu2=new QMenu;
	action=ComponentMenu2->addAction("Default"); action->setData(QVariant(""));
	action=ComponentMenu2->addSeparator();
	// Array
	action=ComponentMenu2->addAction("Syntax for 3D -> 2D"); action->setData(QVariant("Sum(i,j) (i,j) (i,j)"));
	action=ComponentMenu2->addAction("Syntax for 2D range"); action->setData(QVariant("(i,j) (i,j)"));
	action=ComponentMenu2->addSeparator();
	action=ComponentMenu2->addAction("Syntax for Metadata"); action->setData(QVariant("=> META_NAME"));

	/* QMenu* */ComponentMenu3=new QMenu;
	action=ComponentMenu3->addAction("Default"); action->setData(QVariant(""));
	action=ComponentMenu3->addSeparator();
	// Array
	action=ComponentMenu3->addAction("Syntax for 3D range"); action->setData(QVariant("(i,j) (i,j) (i,j)"));
	action=ComponentMenu3->addSeparator();
	action=ComponentMenu3->addAction("Syntax for Metadata"); action->setData(QVariant("=> META_NAME"));

	/* QMenu* */ComponentMenuText=new QMenu;
	action=ComponentMenuText->addAction("Default"); action->setData(QVariant(""));
	action=ComponentMenuText->addSeparator();
	// Array
	action=ComponentMenuText->addAction("Syntax for 1D -> 0D"); action->setData(QVariant("(i)"));
	action=ComponentMenuText->addAction("Syntax for 1D range"); action->setData(QVariant("(i,j)"));
	action=ComponentMenuText->addSeparator();
	action=ComponentMenuText->addAction("Syntax for Metadata"); action->setData(QVariant("=> META_NAME"));

	/* QMenu* */ComponentMenuTime=new QMenu;
	
	// Time
	action=ComponentMenuTime->addAction("Start Time"); action->setData(QVariant("Start Time"));
	action=ComponentMenuTime->addAction("Centre Time"); action->setData(QVariant("Centre Time"));
	action=ComponentMenuTime->addAction("End Time"); action->setData(QVariant("End Time"));
	action=ComponentMenuTime->addSeparator();
	action=ComponentMenuTime->addAction("Time at record"); action->setData(QVariant("i"));
	action=ComponentMenuTime->addSeparator();
	action=ComponentMenuTime->addAction("Syntax for Metadata"); action->setData(QVariant("=> META_NAME"));

	/* QMenu* */ComponentMenuTimeIvl=new QMenu;
	
	// Time
	action=ComponentMenuTimeIvl->addAction("Object Timetag Range"); action->setData(QVariant(""));
	action=ComponentMenuTimeIvl->addAction("Interval between records"); action->setData(QVariant("i,j"));
	action=ComponentMenuTimeIvl->addSeparator();
	action=ComponentMenuTimeIvl->addAction("Event at entry"); action->setData(QVariant("i"));
	action=ComponentMenuTimeIvl->addSeparator();
	action=ComponentMenuTimeIvl->addAction("Syntax for Metadata"); action->setData(QVariant("=> META_NAME"));

	/* QMenu* */ComponentMenuMeta=new QMenu;
	action=ComponentMenuMeta->addAction("Time Tags"); action->setData(QVariant("=> DEPEND_0"));
	action=ComponentMenuMeta->addAction("DEPEND_1"); action->setData(QVariant("=> DEPEND_1"));
	action=ComponentMenuMeta->addAction("DEPEND_2"); action->setData(QVariant("=> DEPEND_2"));
	action=ComponentMenuMeta->addAction("DEPEND_3"); action->setData(QVariant("=> DEPEND_3"));
	action=ComponentMenuMeta->addAction("SI_CONVERSION"); action->setData(QVariant("=> DEPEND_3"));
	action=ComponentMenuMeta->addAction("COORDINATE_SYSTEM"); action->setData(QVariant("=> COORDINATE_SYSTEM"));
	action=ComponentMenuMeta->addAction("FRAME"); action->setData(QVariant("=> FRAME"));
	action=ComponentMenuMeta->addSeparator();
	action=ComponentMenuMeta->addAction("Syntax for Metadata"); action->setData(QVariant("=> META_NAME"));

	outLayout->addWidget(ComponentButton, 2, 0); // these never co-exist

	outLayout->addWidget(EventButton, 2, 0);
	EventButton->hide();
	SuffixComponent=new QLineEdit;
	SuffixEvent=new QLineEdit;
	outLayout->addWidget(MetaButton, 2, 0);
	MetaButton->hide();
	SuffixMeta=new QLineEdit;
	SuffixComponent->setMinimumWidth(350);
	SuffixEvent->setMinimumWidth(350);
	SuffixMeta->setMinimumWidth(350);

//	QRegExp vectSuffixRegexp("r\\(mag\\)|x|y|z|Lat\\s\\(deg\\)|Lat\\s\\(rad\\)|theta\\s\\(deg\\)|theta\\s\\(rad\\)|phi\\s\\(deg\\)|phi\\s\\(rad\\)|\\([123],[123]\\)|Sum\\([123],[123]\\)|Ave\\([123],[123]\\)|\\([123]\\)|=>\\s\\w+");
//	val=new QRegExpValidator(vectSuffixRegexp, vectSuffixComponent);
//	vectSuffixComponent->setValidator(val);
	outLayout->addWidget(SuffixComponent, 2, 1);// these never co-exist
	outLayout->addWidget(SuffixEvent, 2, 1);
	outLayout->addWidget(SuffixMeta, 2, 1);
	SuffixEvent->hide();
	SuffixMeta->hide();
	
	TimeButton=new QPushButton("Time Option");
	TimeButton->setEnabled(true);

	/*QMenu* */TimeMenu=new QMenu;
	action=TimeMenu->addAction("Default"); action->setData(QVariant(""));
	action=TimeMenu->addAction("Syntax for Subset time from record i to j"); action->setData(QVariant("i,j"));
	action=TimeMenu->addAction("Syntax for Average time between records i and j"); action->setData(QVariant("Ave i,j"));
	action=TimeMenu->addAction("Syntax for time at record i"); action->setData(QVariant("j"));
	TimeButton->setMenu(TimeMenu);
	outLayout->addWidget(TimeButton, 3, 0);

	/*QMenu* */TimetagMenu=new QMenu;
	action=TimetagMenu->addAction("Default"); action->setData(QVariant(""));
	action=TimetagMenu->addAction("Syntax for Subset time from record i to j"); action->setData(QVariant("i,j"));

	SuffixTime=new QLineEdit;
	SuffixTime->setMinimumWidth(350);
//	QRegExp SuffixTimeRegexp("\\d+|\\d+,\\d+|Ave\\s\\d+,\\d+");
//	val=new QRegExpValidator(SuffixTimeRegexp, SuffixTime);
//	SuffixTime->setValidator(val);
	outLayout->addWidget(SuffixTime, 3, 1);
	Suffix=new QLabel;
	outLayout->addWidget(Suffix, 4, 0, 1, 2);

		
	

//	QRegExp arraySuffixRegexp("(((Sum|Ave|)\\(\\d+,\\d+\\))|(\\(\\d+\\)))\\s(((Sum|Ave|)\\(\\d+,\\d+\\))|(\\(\\d+\\)))\\s(((Sum|Ave|)\\(\\d+,\\d+\\))|(\\(\\d+\\)))|=>\\s\\w+");
//	val=new QRegExpValidator(arraySuffixRegexp, arraySuffixComponent);
//	arraySuffixComponent->setValidator(val);
	
	
//	mainLayout->addStretch();
	mainWidget->setLayout(mainLayout);
	setWidget(mainWidget);
	
	// for choice setting
//	QRegExp scalarSuffixRegexp("=>\\s\\w+");
//	QRegExpValidator* val=new QRegExpValidator(scalarSuffixRegexp, scalarSuffixComponent);
//	SuffixComponent->setValidator(val);

	connect(compCheckBox, SIGNAL(clicked(bool)), this, SLOT(CheckChanged(bool)));
	connect(ComponentMenu0, SIGNAL(triggered(QAction*)), this, SLOT(selectedComponent(QAction*)));
	connect(ComponentMenu1, SIGNAL(triggered(QAction*)), this, SLOT(selectedComponent(QAction*)));
	connect(ComponentMenu2, SIGNAL(triggered(QAction*)), this, SLOT(selectedComponent(QAction*)));
	connect(ComponentMenu3, SIGNAL(triggered(QAction*)), this, SLOT(selectedComponent(QAction*)));
	connect(ComponentMenuText, SIGNAL(triggered(QAction*)), this, SLOT(selectedComponent(QAction*)));
	connect(ComponentMenuTime, SIGNAL(triggered(QAction*)), this, SLOT(selectedComponent(QAction*)));
	connect(ComponentMenuTimeIvl, SIGNAL(triggered(QAction*)), this, SLOT(selectedEvent(QAction*)));
	connect(ComponentMenuMeta, SIGNAL(triggered(QAction*)), this, SLOT(selectedMeta(QAction*)));
	connect(SuffixComponent, SIGNAL(textEdited(const QString&)), this, SLOT(SuffixChanged(const QString&)));
	connect(SuffixEvent, SIGNAL(textEdited(const QString&)), this, SLOT(EventChanged(const QString&)));
	connect(SuffixMeta, SIGNAL(textEdited(const QString&)), this, SLOT(MetaChanged(const QString&)));
	connect(TimeMenu, SIGNAL(triggered(QAction*)), this, SLOT(selectedTime(QAction*)));
	connect(TimetagMenu, SIGNAL(triggered(QAction*)), this, SLOT(selectedTime(QAction*)));
	connect(SuffixTime, SIGNAL(textEdited(const QString&)), this, SLOT(SuffixChanged(const QString&)));

}

void ExtractorContextWidget::setTarget(int targetType){
	
	foreach(VExtractionItem* item, items)
	{
		ExtractionItem *gui = item->getGUI();	
		if( !gui) return;		
	
		// Set what object is acceptable and change component and time menu options as appropriate
		
		if(targetType == 0){
			ComponentButton->setMenu(ComponentMenu0);
			ComponentButton->show();
			EventButton->hide();
			SuffixEvent->hide();
			MetaButton->hide();
			SuffixMeta->hide();
			SuffixComponent->show();
			TimeButton->show();
			SuffixTime->show();
			TimeButton->setMenu(TimeMenu);
			gui->edit->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_ANY, QS_ANY));
		}
		else if(targetType == 1){
			ComponentButton->setMenu(ComponentMenu1);
			ComponentButton->show();
			EventButton->hide();
			SuffixEvent->hide();
			MetaButton->hide();
			SuffixMeta->hide();
			SuffixComponent->show();
			TimeButton->show();
			SuffixTime->show();
			TimeButton->setMenu(TimeMenu);
			gui->edit->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_ANY, QS_ANY));
		}
		else if(targetType == 2){
			ComponentButton->setMenu(ComponentMenu2);
			ComponentButton->show();
			EventButton->hide();
			SuffixEvent->hide();
			MetaButton->hide();
			SuffixMeta->hide();
			SuffixComponent->show();
			TimeButton->show();
			SuffixTime->show();
			TimeButton->setMenu(TimeMenu);
			gui->edit->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_2D, QS_ANY, QS_ANY));
		}
		else if(targetType == 3){
			ComponentButton->setMenu(ComponentMenu3);
			ComponentButton->show();
			EventButton->hide();
			SuffixEvent->hide();
			MetaButton->hide();
			SuffixMeta->hide();
			SuffixComponent->show();
			TimeButton->show();
			SuffixTime->show();
			TimeButton->setMenu(TimeMenu);
			gui->edit->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_2D, QS_ANY, QS_ANY));
		}
		else if(targetType == 4){
			ComponentButton->setMenu(ComponentMenuText);
			ComponentButton->show();
			EventButton->hide();
			SuffixEvent->hide();
			MetaButton->hide();
			SuffixMeta->hide();
			SuffixComponent->show();
			TimeButton->show();
			SuffixTime->show();
			TimeButton->setMenu(TimeMenu);
			gui->edit->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_ANY, QS_ANY, QS_ANY));
		}
		else if(targetType == 5){
			ComponentButton->setMenu(ComponentMenuTime);
			ComponentButton->show();
			EventButton->hide();
			SuffixEvent->hide();
			MetaButton->hide();
			SuffixMeta->hide();
			SuffixComponent->show();
			TimeButton->hide();
			SuffixTime->hide();
			gui->edit->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
		}
		else if(targetType == 6){
			EventButton->setMenu(ComponentMenuTimeIvl);
			EventButton->show();
			SuffixEvent->show();
			MetaButton->hide();
			SuffixMeta->hide();
			ComponentButton->hide();
			SuffixComponent->hide();
			TimeButton->hide();
			SuffixTime->hide();
			gui->edit->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));
		}
		else if(targetType == 7){
			ComponentButton->hide();
			SuffixComponent->hide();
			EventButton->hide();
			SuffixEvent->hide();
			MetaButton->hide();
			SuffixMeta->hide();
			TimeButton->show();
			SuffixTime->show();
			gui->edit->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_SEQ, QS_TT));
			TimeButton->setMenu(TimetagMenu);
		}
		else if(targetType == 8){
			ComponentButton->hide();
			SuffixComponent->hide();
			EventButton->hide();
			SuffixEvent->hide();
			MetaButton->setMenu(ComponentMenuMeta);
			MetaButton->show();
			SuffixMeta->show();
			TimeButton->hide();
			SuffixTime->hide();
			gui->edit->setAcceptable(Qcondition(QS_NARROW, QS_ANY, QS_ANY, QS_ANY, QS_ANY));
		}
		
	}
	
}


void ExtractorContextWidget::setExtractorParameters(QList<VExtractionItem*>& items)
{
	SuffixComponent->blockSignals(true);
	SuffixTime->blockSignals(true);
	
	bool ask = items[0]->askUser;
	compCheckBox->setChecked(ask);

	this->items=items;
	warningLabel->setVisible(items.size()>1);
	
	int i1, i2;
	
	i1=items[0]->Suffix.indexOf("[");
	if(i1>0)
	{
		i2=items[0]->Suffix.indexOf("]");
		QString text=items[0]->Suffix.mid(i1+1, i2-i1-1);
		SuffixComponent->setText(text.trimmed());
	}
	else
	{
		SuffixComponent->setText("");
	}

	i1=items[0]->Suffix.indexOf("<");
	if(i1>0)
	{
		i2=items[0]->Suffix.indexOf(">");
		QString text=items[0]->Suffix.mid(i1+1, i2-i1-1);
		SuffixTime->setText(text.trimmed());
	}
	else
	{
		SuffixTime->setText("");
	}

	SuffixComponent->blockSignals(false);
	SuffixTime->blockSignals(false);

}

void ExtractorContextWidget::selectedComponent(QAction* action)
{
	SuffixComponent->setText(action->data().toString());
	if(action->data().toString().contains("Sum")){
		talkBack->setText("Edit (i,j), Sum(i,j), Ave(i,j) or (i)) for each dimension");
	}
	else if(action->data().toString().contains("META_NAME")){
		talkBack->setText("Edit META_NAME to required attribute ");
	}
	else if(action->data().toString().contains("i,j")){
		talkBack->setText("Edit indices i,j for required range");
	}
	else if(action->data().toString().contains("i")){
		talkBack->setText("Edit index i for required slice");
	}
	SuffixChanged(SuffixComponent->text());
}

void ExtractorContextWidget::selectedEvent(QAction* action)
{
	SuffixEvent->setText(action->data().toString());
	if(action->data().toString().contains("META_NAME")){
		talkBack->setText("Edit META_NAME to required attribute ");
	}
	else if(action->data().toString().contains("i,j")){
		talkBack->setText("Edit indices i,j for required range");
	}
	else if(action->data().toString().contains("i")){
		talkBack->setText("Edit index i for required event");
	}
	EventChanged(SuffixEvent->text());
}

void ExtractorContextWidget::selectedMeta(QAction* action)
{
	SuffixMeta->setText(action->data().toString());
	if(action->data().toString().contains("META_NAME")){
		talkBack->setText("Edit META_NAME to required attribute ");
	}
	else {
		talkBack->setText("Edit index i for required event");
	}
	MetaChanged(SuffixMeta->text());
}

void ExtractorContextWidget::selectedTime(QAction* action)
{
	SuffixTime->setText(action->data().toString());
	SuffixChanged(SuffixTime->text());

}


void ExtractorContextWidget::SuffixChanged(const QString&)
{
	compCheckBox->setChecked(false);

	QString suffix=" ";
	if(!SuffixComponent->text().isEmpty())
	{
		if(SuffixComponent->text().startsWith("=>"))
		{
			suffix.append("[");
		}
		else
		{
			suffix.append("[ ");
		}
		suffix.append(SuffixComponent->text().trimmed());
		suffix.append(" ] ");
	}
	if(!SuffixTime->text().isEmpty())
	{
		suffix.append("< ");
		suffix.append(SuffixTime->text().trimmed());
		suffix.append(" >");
	}
	
	Suffix->setText(QString("Suffix: %1").arg(suffix));

	foreach(VExtractionItem* item, items)
	{
		item->Suffix=suffix;
		item->setComp();
		item->askUser=false;
		ExtractionItem *gui = item->getGUI();
		if(gui) {
			gui->edit->lineBox->setEnabled(false);
		}

	}
}

void ExtractorContextWidget::EventChanged(const QString&)
{
	compCheckBox->setChecked(false);
	
	QString suffix=" ";
	if(!SuffixEvent->text().isEmpty())
	{
		if(SuffixEvent->text().startsWith("=>"))
		{
			suffix.append("[");
			suffix.append(SuffixEvent->text().trimmed());
			suffix.append(" ] ");
		}
		else
		{
			suffix.append("< ");
			suffix.append(SuffixEvent->text().trimmed());
			suffix.append(" > ");
		}
	}
	Suffix->setText(QString("Suffix: %1").arg(suffix));
	
	foreach(VExtractionItem* item, items)
	{
		item->Suffix=suffix;
		item->setComp();
		item->askUser=false;
		ExtractionItem *gui = item->getGUI();
		if(gui) {
			gui->edit->lineBox->setEnabled(false);
		}

	}
}

void ExtractorContextWidget::MetaChanged(const QString&)
{
	compCheckBox->setChecked(false);
	
	QString suffix=" ";
	if(!SuffixMeta->text().isEmpty())
	{
		if(SuffixMeta->text().startsWith("=>"))
		{
			suffix.append("[");
			suffix.append(SuffixMeta->text().trimmed());
			suffix.append(" ] ");
		}
	}
	Suffix->setText(QString("Suffix: %1").arg(suffix));
	
	foreach(VExtractionItem* item, items)
	{
		item->Suffix=suffix;
		item->setComp();
		item->askUser=false;
		ExtractionItem *gui = item->getGUI();
		if(gui) {
			gui->edit->lineBox->setEnabled(false);
		}

	}
}

void ExtractorContextWidget::CheckChanged(bool checked)
{
	if(checked){
		talkBack->setText("Calculation will pause at unresolved Extractors");
		ComponentButton->setEnabled(false);
		EventButton->setEnabled(false);
		MetaButton->setEnabled(false);
		TimeButton->setEnabled(false);
		SuffixComponent->blockSignals(true);
		SuffixTime->blockSignals(true);
			SuffixComponent->setText("");
			SuffixTime->setText("");
		SuffixComponent->blockSignals(false);
		SuffixTime->blockSignals(false);
		Suffix->setText("");
	
	}
	else{
		talkBack->setText(" ");
		ComponentButton->setEnabled(true);
		EventButton->setEnabled(true);
		MetaButton->setEnabled(true);
		TimeButton->setEnabled(true);
	}
	
	foreach(VExtractionItem* item, items)
	{
		item->askUser=checked;
		ExtractionItem *gui = item->getGUI();
		if(gui) gui->edit->lineBox->setEnabled(checked);
	}
	

}

void ExtractorContextWidget::userSetComponent(const char *comp, const char *type){
	QString suffix=comp;
	Suffix->setText(QString("Suffix: %1").arg(suffix));
	int i1, i2;
	i1 = suffix.indexOf("[");
	if(i1>0)
	{
		i2 = suffix.indexOf("]");
		QString text=suffix.mid(i1+1, i2-i1-1);
		SuffixComponent->setText(text.trimmed());
	}
	else
	{
		SuffixComponent->setText("");
	}
	
	i1 = suffix.indexOf("<");
	if(i1>0)
	{
		i2 = suffix.indexOf(">");
		QString text = suffix.mid(i1+1, i2-i1-1);
		SuffixTime->setText(text.trimmed());
	}
	else
	{
		SuffixTime->setText("");
	}
	

}


