#include "qcalcEvaluator.h"

#ifdef WINDOWS
/* SJS attempt to undefine X and Y 6Aug2014 */
#undef X
#undef Y
#include <windows.h>
#else
#include <dlfcn.h>     // dynamic-load stuff
#endif

extern qwuiWindow* qsasWindow;

QEvaluatorPlugin::QEvaluatorPlugin(const QString& pluginName):
    QEvaluator()
{
	if(pluginName.endsWith(".qtpl"))
	{
		this->pluginName=pluginName;
		qtplFile=new QtplParserFile(pluginName);
	}
	else
	{
		this->pluginName=pluginName;
		qtplFile=new QtplParserFile(pluginName+".qtpl");
	}

    // Location of the so/dll/dylib file, directly adapted from qtpl.cc
    
    FILE *fp;
    const char * str_NULL = "";
 
  // find path to loaded .qtpl file as a possible location for .so
    QString qtplPath = QString((pluginName+(".qtpl")).toLatin1().data()); // deep copy
  
    int posn = qtplPath.lastIndexOf('/');
    if(posn >= 0)  qtplPath.truncate(posn);
    else           qtplPath = "./";
  
  
    QString file_stem = QString(qtplFile->object_file.toLatin1().data());
    posn = file_stem.lastIndexOf('.');
    if(posn >= 0)  file_stem.truncate(posn);

    QString user_lib = QSAS_profiles->current()->UserLibPath;
  
    QString search_path = QString("");
    if( (qtplFile->object_file)[0] != '/')
        search_path = QString("./");

    search_path += qtplFile->object_file;
    QString windowsPath = QString(qtplFile->object_file.toLatin1().data());
    int ptr = windowsPath.indexOf(".so");
    windowsPath.replace(ptr, 3, ".dll");

    fp = fopen(search_path.toLatin1().data(), "r");
    if ( fp == NULL ){
        search_path = qtplPath;
        search_path += "/";
        search_path += qtplFile->object_file;
        fp = fopen(search_path.toLatin1().data(), "r");
    }
    if ( fp == NULL && strcmp(user_lib.toLatin1().data(), str_NULL) != 0){
        search_path = user_lib;
        search_path += "/";
        search_path += qtplFile->object_file;
        fp = fopen(search_path.toLatin1().data(), "r");
    }
    if ( fp == NULL){
        search_path = QString(qsasWindow->qsas_home);
        search_path += "/lib/";
        search_path += qtplFile->object_file;
        fp = fopen(search_path.toLatin1().data(), "r");
    }
  // try for Windows .dl file
    if ( fp == NULL ){
        search_path = qtplPath;
        search_path += "/";
        search_path += windowsPath;
        fp = fopen(search_path.toLatin1().data(), "r");
    }
    if ( fp == NULL && strcmp(user_lib.toLatin1().data(), str_NULL) != 0){
        search_path = user_lib;
        search_path += "/";
        search_path += windowsPath;
        fp = fopen(search_path.toLatin1().data(), "r");
    }
    if ( fp == NULL){
        search_path = QString(qsasWindow->qsas_home);
        search_path += "/lib/";
        search_path += windowsPath;
        fp = fopen(search_path.toLatin1().data(), "r");
    }

    if ( fp == NULL)  {
         QplugAppendTextDisplay(qtplFile->object_file.toLatin1().data());
         QplugAppendTextDisplay(" < cannot locate dynamic library  \n");
     
        try_entry_name = "";
        return;
    }
      
  // We have the correct path if we get here. 

  // keep full path to .so for dynamic load

    PathAndNameSO = QString(search_path.toLatin1().data());  // deep copy

  
  // Close file and use system call to find entry point
    
    fclose(fp);
  
    try_entry_name = QString(qtplFile->entry_name.toLatin1().data());

}

void QEvaluatorPlugin::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	
	// DVOS aware plugins
	
    ok=true;

	QEvaluator::evaluate(in, out, ok, message);
	if(!ok) return;

    vector <DvObject_var> inputs;
    vector <DvObject_var> outputs;

#ifdef WINDOWS
    HINSTANCE handle;
    typedef QplugReturnStatus (__cdecl *MYPROC)(vector <DvObject_var>&, vector <DvObject_var>&);
    MYPROC fptr;
#else
    void *handle = NULL;                      // for dynamic load
    QplugReturnStatus (*fptr)(vector <DvObject_var>&, vector <DvObject_var>&)=NULL;
#endif
    QplugReturnStatus  f_status;

	//  Check there is a template/plug-in loaded to run,
	//  and return promptly if there is not.

    if (this->qtplFile == NULL || !(this->qtplFile->openedOK))  {
        message="ERROR: no plug-in template loaded.";
        ok=false;
        return;
    }

	// start with a clean empty list
	// create and init a null ptr for each plug-in-argument

    inputs.clear();
    outputs.clear();
	
    for(int i=0; i<num_in; i++)
    {
        DvObject_var val;
        if(in[i]->connectedSlot()!=NULL)
        {
            val=in[i]->connectedSlot()->getValue(ok);
        }
        else
        {
            val=new DvObject();
        }
        if(!ok)
        {
            message=QString("Input: ")+in[i]->getName()+QString(" invalid");
            ok=false;
            return;
        }
        inputs.push_back(val);
    }

      //  built input arg list, attempt plug-in load

    handle = NULL;

#ifdef WINDOWS
    handle=LoadLibrary(PathAndNameSO.toLatin1().data());
    if( !handle) {
      
        message="Dynamic load ERROR: LoadLibrary returned null, abandoning dynamic-load for ";
        message+=qtplFile->object_file.toLatin1().data();
        message+=" with path ";
        message+=PathAndNameSO.toLatin1().data();
        message+="\n";
        message+="LoadLibrary error... ";
        ok=false;
        return;
    }
        
    if ( (fptr = (MYPROC) GetProcAddress(handle, try_entry_name.toLatin1().data())) == NULL )  {
      
        message="plugin entry point ERROR: GetProcAddress lookup failed, dlerror contents are:\n  ";
        message+="\n";
        if( !FreeLibrary(handle)) message+="Error freeing the library";
        ok=false;
        return;
    }
#else
    handle = dlopen(PathAndNameSO.toLatin1().data(), RTLD_LAZY);
    if( handle == NULL ) {
        message="Dynamic load ERROR: dlopen returned null, abandoning dynamic-load for\a\n";
        message+=qtplFile->object_file.toLatin1().data();
        message+=" with path ";
        message+=PathAndNameSO.toLatin1().data();
        message+="\n";
        message+="dlopen error... ";
        message+=dlerror();
        ok=false;
        return;
    }
        
    if ( (fptr = (QplugReturnStatus_e (*)(vector <DvObject_var>&, vector <DvObject_var>&)) dlsym(handle, try_entry_name.toLatin1().data())) == NULL )  {
      
        message="plugin entry point ERROR: dlsym lookup failed, dlerror contents are:\n  ";
        message+=dlerror();
        message+="\n";
        if( dlclose(handle) != 0 ) message+=dlerror();
        ok=false;
        return;
    }
#endif

    f_status = (*fptr)(inputs, outputs);  // ATTEMPT DYNAMIC FUNCTION CALL
    
    message="";
    switch(f_status)
    {
        case QPLUG_WARNING:  // print warning, then fall thru QPLUG_SUCCESS case to put output args on working list
        {
            message+="Qplug> WARNING: plug-in returned warning\n";
            message+="      use the output data objects at your own risk!\n";
        }
        case QPLUG_SUCCESS:
        {
            for(int i=0; i<outputs.size(); i++)
            {
                if(outputs[i].is_ok() )
                {
                    out[i]->setValue(outputs[i], true);
                }
                else
                {
					DvObject_var nilObj = new DvObject();
                    out[i]->setValue(nilObj, false);
                }
            }
            break;
        }
        case QPLUG_FAILURE:  // print error, don't touch output args
        {
            message+="Qplug> ERROR: plug-in returned failure.\n";
            ok=false;
            break;

        }
        default:  // print panic error and don't touch output args!
        {
            message+="Qplug> PANIC: plug-in returned unrecognised status code.\n";
            message+="      Check that plug-in returns QplugReturnStatus_e\n";
            message+="      No output data objects generated.\n";
            ok=false;
            break;
        }
    }
#ifdef WINDOWS
  	// FreeLibrary(handle); // crashes
#else
    if( dlclose(handle) != 0 )
    {
        ok=false;
        message+=dlerror();
    }
#endif

}
