#include "qcalcEvaluator.h"

const double PI_  = 4.0 * atan(1.0);


typedef DvObject_var ( *calc2ObjectFnPtr )( DvObject_var operand1, QString* pstrErrorMsg);

DvObject_var AcosLocalCalc2( DvObject_var  operand1, QString* pstrErrorMsg )
{

	DvObject_var result = operand1->acos();
	
	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute acos of this object" );
	}

	return result ;
}

DvObject_var AsinLocalCalc2( DvObject_var  operand1, QString* pstrErrorMsg )
{

	DvObject_var result = operand1->asin();
	
	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute asin of this object" );
	}

	return result ;
}

DvObject_var AtanLocalCalc2( DvObject_var  operand1, QString* pstrErrorMsg )
{
	DvObject_var  result =  operand1->atan() ;
	
	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute atan of this object" );
	}
	
	return result;
}


DvObject_var AcotLocalCalc2( DvObject_var operand1, QString* pstrErrorMsg )
{
	DvObject_var result = operand1->acot();
	
	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute acot of this object" );
	}
	
	return result ;
}

DvObject_var AcoshLocalCalc2( DvObject_var operand1, QString* pstrErrorMsg )
{
	DvObject_var result = operand1->acosh();
	
	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute acosh of this object" );
	}
	
	return result;
}

DvObject_var AsinhLocalCalc2( DvObject_var operand1, QString* pstrErrorMsg )
{
	DvObject_var result = operand1->asinh();
	
	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute asinh of this object" );
	}
	
	return result;
}

DvObject_var AtanhLocalCalc2( DvObject_var operand1, QString* pstrErrorMsg )
{
	DvObject_var result = operand1->atanh();
	
	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute atanh of this object" );
	}
	
	return result;
}
  
DvObject_var AcothLocalCalc2( DvObject_var operand1, QString* pstrErrorMsg )
{
	DvObject_var result = operand1->acoth();
	
	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute acoth of this object" );
	}
	
	return result;
}

DvObject_var TangentCalc2( DvObject_var operand1, QString* pstrErrorMsg )
{
	DvObject_var result =  operand1->tan();
	
	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute tan of this object" );
	}
	
	return result;
}


DvObject_var CTangentCalc2(  DvObject_var operand1, QString* pstrErrorMsg )
{
	DvObject_var result =  operand1->cot();
	
	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute cot of this object" );
	}
	
	return result;
}


DvObject_var TangentHCalc2( DvObject_var operand1, QString* pstrErrorMsg )
{
	DvObject_var result = operand1->tanh();

	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute tanh of this object" );
	}
	
	return result;
}


DvObject_var CTangentHCalc2( DvObject_var operand1, QString* pstrErrorMsg )
{
	DvObject_var result =  operand1->coth();

	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute coth of this object" );
	}
	
	return result;
}


DvObject_var cosLocalCalc2( DvObject_var operand1, QString* pstrErrorMsg )
{
	DvObject_var result =  operand1->cos();

	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute cos of this object" );
	}
	
	return result;
}


DvObject_var sinLocalCalc2( DvObject_var operand1, QString* pstrErrorMsg )
{
	DvObject_var result =  operand1->sin();

	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute sin of this object" );
	}
	
	return result;
}

DvObject_var coshLocalCalc2( DvObject_var operand1, QString* pstrErrorMsg )
{
	DvObject_var result =  operand1->cosh();

	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute cosh of this object" );
	}
	
	return result;
}

DvObject_var sinhLocalCalc2( DvObject_var operand1, QString* pstrErrorMsg )
{
	DvObject_var result =  operand1->sinh();

	if ( result.is_nil() )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute sinh of this object" );
	}
	
	return result;
}



DvObject_var TrigFunction( calc2ObjectFnPtr pFn,
			      			DvObject_var operand1,
  							QString&     rqstrErrorMsg,
  							bool         bHyperbolic=false)
{
	DvObject_var result;

	if ( operand1.is_nil() )
	{
        // The object will be empty - this should never happen
	    // The qdResult object is NULL at this point
		return result;
	}

	if ( pFn == NULL )
	{
		return result;
	}

	if ( operand1.is_nil() )
	{
		rqstrErrorMsg="Error>> 1st input empty.";
		return result;
	}


	//-----check and read what's in the SI_conversion ----------

	
	if ( !bHyperbolic )
	{
		// ensure input is in rad if we can
		operand1 = operand1->toRad();
	}

	// DO TRIG OPERATION


	result = ( *pFn )( operand1, &rqstrErrorMsg );
		
	if ( result.is_nil() ) 
	{
		rqstrErrorMsg =  QString( "Error>> invalid input to the trigonometry function. Empty object returned." );
	}

	return result;
}

void QEvaluatorTrig::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	DvObject_var result;
	
	if(in[0]->connectedSlot()==NULL)
	{
		ok=false;
		message=QString("Missing input 1");
		return;
	}
	
	if(op==QCALC_ATAN2 && in[1]->connectedSlot()==NULL)
	{
		ok=false;
		message=QString("Missing input 2");
		return;
	}
	
	DvObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	switch(op)
	{
		case QCALC_SIN:
			result=TrigFunction( &sinLocalCalc2, val1, message);
			break;
		case QCALC_TAN:
			result=TrigFunction( &TangentCalc2, val1, message);
			break;
		case QCALC_COT:
			result=TrigFunction( &CTangentCalc2, val1, message);
			break;
		case QCALC_COSH:
			result=TrigFunction( &coshLocalCalc2, val1, message, true);
			break;
		case QCALC_SINH:
			result=TrigFunction( &sinhLocalCalc2, val1, message, true);
			break;
		case QCALC_TANH:
			result=TrigFunction( &TangentHCalc2, val1, message, true);
			break;
		case QCALC_COTH:
			result=TrigFunction( &CTangentHCalc2, val1, message, true);
			break;
		case QCALC_ACOS:
			result=TrigFunction( &AcosLocalCalc2, val1, message);
			break;
		case QCALC_ASIN:
			result=TrigFunction( &AsinLocalCalc2, val1, message);
			break;
		case QCALC_ATAN:
			result=TrigFunction( &AtanLocalCalc2, val1, message);
			break;
		case QCALC_ATAN2:
			{		
				DvObject_var val2=in[1]->connectedSlot()->getValue(ok);
				if(!ok)
				{
					message=QString("Input ")+in[1]->getName()+QString(" invalid");
					ok=false;
					return;
				}
				if( !val1.is_nil() && !val2.is_nil()) result = val1->atan2(val2); // atan2(val1/val2)

				if ( result.is_nil() )
				{
					message =  QString( "Error>> invalid input to the trigonometry function. Empty object returned." );
					ok=false;
					return;
				}
			}
			break;
		case QCALC_ACOT:
			result=TrigFunction( &AcotLocalCalc2, val1, message);
			break;
		case QCALC_ACOSH:
			result=TrigFunction( &AcoshLocalCalc2, val1, message, true);
			break;
		case QCALC_ASINH:
			result=TrigFunction( &AsinhLocalCalc2, val1, message, true);
			break;
		case QCALC_ATANH:
			result=TrigFunction( &AtanhLocalCalc2, val1, message, true);
			break;
		case QCALC_ACOTH:
			result=TrigFunction( &AcothLocalCalc2, val1, message, true);
			break;
		case QCALC_COS:
		default:
			result=TrigFunction( &cosLocalCalc2, val1, message);
			break;
	}
	
	if(!message.isEmpty() && result.is_nil())
	{
		ok=false;
		return;
	}

	out[0]->setValue(result, true);
}

// --------------------------------

void QEvaluatorAngle360::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	DvObject_var result;

 	QEvaluator::evaluate(in, out, ok, message);
 	if(!ok)
 		return;
	
	if(in[0]->connectedSlot()==NULL)
	{
		ok=false;
		message=QString("Missing input 1");
		return;
	}
	
	
	DvObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	  
	// copy object

	result = new DvObject(val1);
	
	int rank = val1->nDims();
		
	// find each DEPEND_i
	for(int i=1; i <= rank; i++){
		DvString dep = "DEPEND_";
		dep += i;
		if( !result->xref_exists(dep) ) continue;
		DvObject_var xref = result->get_xref(dep);
			
		// If angle DEPEND modify
		if( !xref->isAngle() ) continue;
		DvString angle = xref->angleUnitStr();
			
		DvObject_var newXref = xref->angleMod(360, angle);
		result->change_xref(dep, newXref);
	}

	if(result.is_nil())
	{
		ok=false;
		return;
	}
	
	out[0]->setValue(result, true);
}
// --------------------------------

void QEvaluatorAngle180::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	DvObject_var result;

 	QEvaluator::evaluate(in, out, ok, message);
 	if(!ok)
 		return;
	
	if(in[0]->connectedSlot()==NULL)
	{
		ok=false;
		message=QString("Missing input 1");
		return;
	}
	
	
	DvObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	  
	// copy object

	result = new DvObject(val1);
	
	int rank = val1->nDims();
		
	// find each DEPEND_i
	for(int i=1; i <= rank; i++){
		DvString dep = "DEPEND_";
		dep += i;
		if( !result->xref_exists(dep) ) continue;
		DvObject_var xref = result->get_xref(dep);
			
		// If angle DEPEND modify
		if( !xref->isAngle() ) continue;
		DvString angle = xref->angleUnitStr();
			
		DvObject_var newXref = xref->angleMod(180, angle);
		result->change_xref(dep, newXref);
	}

	if(result.is_nil())
	{
		ok=false;
		return;
	}
	
	out[0]->setValue(result, true);
}

// --------------------------------

void QEvaluatorToRad::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	DvObject_var result;

 	QEvaluator::evaluate(in, out, ok, message);
 	if(!ok)
 		return;
	
	if(in[0]->connectedSlot()==NULL)
	{
		ok=false;
		message=QString("Missing input 1");
		return;
	}
	
	
	DvObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	

	result = val1->toRad();
	
	if(result.is_nil())
	{
		ok=false;
		return;
	}
	
	out[0]->setValue(result, true);
}

// --------------------------------

void QEvaluatorToDeg::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	DvObject_var result;

 	QEvaluator::evaluate(in, out, ok, message);
 	if(!ok)
 		return;
	
	if(in[0]->connectedSlot()==NULL)
	{
		ok=false;
		message=QString("Missing input 1");
		return;
	}
	
	
	DvObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	

	result = val1->toDeg();
	
	if(result.is_nil())
	{
		ok=false;
		return;
	}
	
	out[0]->setValue(result, true);
}

