#include "qcalcJoinItem.h"
#include "qcalcSlots.h"
#include "qcalcInputItem.h"
#include "qcalcOutputItem.h"

using namespace QSAS;


VJoinItem::VJoinItem():
		VCalcItem()
{
	join_id=counter++;
	gui=NULL;
	VInSlot* slot=new VInSlot(this);
	inputs.push_back(slot);
	VOutSlot* oslot=new VOutSlot(this);
	outputs.push_back(oslot);

	joinSettings=new JoinSettings;
    joinSettings->noJoin=false;
	joinSettings->algorithm=0;
	joinSettings->width=60.;
	joinSettings->gap=0;
	joinSettings->gapSize=6.;
    joinSettings->minBoxSize=3;
	
	name="Join";
}

VJoinItem::~VJoinItem()
{
	
	if(gui!=NULL)
	{
		QGraphicsScene * scene=gui->scene();
		if(scene!=NULL)
		{
			scene->removeItem(gui);
		}
		gui->parentItem=NULL; // thus gui knows it is being deleted

		delete gui;
	}
	
	delete joinSettings;
}

QDomElement VJoinItem::toXMLElement(QDomDocument& document) const
{
	QDomElement tag=document.createElement("Join");
    if(gui!=NULL)
    {
		tag.appendChild(gui->toXMLElement(document));
    }

	tag.appendChild(createTextXMLNode(document, "XPosnInBlackBox", positionInBlackBox.x()));
	tag.appendChild(createTextXMLNode(document, "YPosnInBlackBox", positionInBlackBox.y()));
	tag.appendChild(createTextXMLNode(document, "ID", calc_id));
	tag.appendChild(createTextXMLNode(document, "Page", page));
	tag.appendChild(createTextXMLNode(document, "Join_No_Join", joinSettings->noJoin));
	tag.appendChild(createTextXMLNode(document, "Join_Algorithm", joinSettings->algorithm));
	tag.appendChild(createTextXMLNode(document, "Join_Width", joinSettings->width));
	tag.appendChild(createTextXMLNode(document, "Join_Gap", joinSettings->gap));
    tag.appendChild(createTextXMLNode(document, "Join_Gap_Size", joinSettings->gapSize));
    tag.appendChild(createTextXMLNode(document, "Join_End", joinSettings->end));
    tag.appendChild(createTextXMLNode(document, "Join_Min_Box", joinSettings->minBoxSize));
	
		QDomElement inputSlots=document.createElement("Input_Slots");
	tag.appendChild(inputSlots);
	for(int i=0; i<inputs.size(); ++i)
	{
		QDomElement s=document.createElement("Inslot");
		s.setAttribute("Index", i);
		s.setAttribute("Name", inputs[i]->getName());
		s.setAttribute("Description", inputs[i]->getDescription());
		inputSlots.appendChild(s);
	}
	QDomElement outputSlots=document.createElement("Output_Slots");
	tag.appendChild(outputSlots);
	for(int i=0; i<outputs.size(); ++i)
	{
		QDomElement s=document.createElement("Outslot");
		s.setAttribute("Index", i);
		s.setAttribute("Name", outputs[i]->getName());
		s.setAttribute("Description", outputs[i]->getDescription());
		s.setAttribute("open", (int) outputs[i]->openSlot);
		outputSlots.appendChild(s);
	}

    return tag;
}

JoinItem* VJoinItem::getGUI()
{
	if(gui==NULL)
	{
		gui=new JoinItem(this, inputs[0], outputs[0]);
	}
	return gui;
}


void VJoinItem::setTags(DvObject_var tt)
{
	tags = tt;
}

void VJoinItem::removeGUI()
{
	if(gui!=NULL)
	{
		gui->scene()->removeItem(gui);
		delete gui;
		gui=NULL;
	}
}

void VJoinItem::setErrMessage(bool err, const QString& message)
{
	if(gui) gui->setErrMessage(err, message);
}

void VJoinItem::reevaluateOutSlots()
{

	bool ok=true;
	DvObject_var value;

	if(inputs[0]->connectedSlot() == NULL) {
		setErrMessage(true, "Input missing");
		outputs[0]->setValue(value, false);
		return;
	}


	DvObject_var not_joined_value=inputs[0]->connectedSlot()->getNotJoinedValue(ok); // for input slots
	if(!ok) not_joined_value=inputs[0]->connectedSlot()->getValue(ok); // for everything else
    
	if(!ok)
	{
		setErrMessage(true, "Input invalid");
		outputs[0]->setValue(value, false);
		return;
	}
	else setErrMessage(false, "");

	
	if(joinSettings->noJoin)
    {
        value=not_joined_value;
    }
    else
    {
        if(not_joined_value->seqSize() <= 1 || joinSettings->noJoin)
        {
            value=not_joined_value;
        }
        else
        {
 
            switch(joinSettings->gap)
            {
                case 1:
                {
                    DvString gap_opt_ptr(DV_ZERO_FILL);
                    not_joined_value->change_xref(DV_GAP_OPTION, gap_opt_ptr);
                    break;
                }
                case 2:
                {
                    DvString gap_opt_ptr(DV_FILL);
                    not_joined_value->change_xref(DV_GAP_OPTION, gap_opt_ptr);
                    break;
                }
                case 3:
                {
                    DvString gap_opt_ptr(DV_NN);
                    not_joined_value->change_xref(DV_GAP_OPTION, gap_opt_ptr);
                    break;
                }
                case 4:
                {
                    DvString gap_opt_ptr(DV_REMOVE);
                    not_joined_value->change_xref(DV_GAP_OPTION, gap_opt_ptr);
                    break;
                }
                case 0:
                default:
                {
                    DvString gap_opt_ptr(DV_LINEAR);
                    not_joined_value->change_xref(DV_GAP_OPTION, gap_opt_ptr);
                    break;
                }
            }
            
            switch(joinSettings->end)
            {
                case 1:
                {
                    DvString end_opt_ptr(DV_ZERO_FILL);
                    not_joined_value->change_xref(DV_END_OPTION, end_opt_ptr);
                    break;
                }
                case 2:
                {
                    DvString end_opt_ptr(DV_FILL);
                    not_joined_value->change_xref(DV_END_OPTION, end_opt_ptr);
                    break;
                }
                case 3:
                {
                    DvString end_opt_ptr(DV_NN);
                    not_joined_value->change_xref(DV_END_OPTION, end_opt_ptr);
                    break;
                }
                case 4:
                {
                    DvString end_opt_ptr(DV_REMOVE);
                    not_joined_value->change_xref(DV_END_OPTION, end_opt_ptr);
                    break;
                }
                case 0:
                default:
                {
                    DvString end_opt_ptr(DV_LINEAR);
                    not_joined_value->change_xref(DV_END_OPTION, end_opt_ptr);
                    break;
                }
            }
							
			if(joinSettings->algorithm == 0){
				double gap=joinSettings->gapSize;
				if(gap > 1.e-20) not_joined_value->change_xref(DV_GAP_WIDTH, gap);

				value = not_joined_value->linearJoin(*tags);
			}
			else {
				double boxcar_width=joinSettings->width;
                not_joined_value->change_xref(DV_BOX_WIDTH, boxcar_width);
                int min_box = joinSettings->minBoxSize;
                not_joined_value->change_xref(DV_MIN_BOXCAR, min_box);
				value = not_joined_value->boxcarJoin(*tags);
			}				
								
            if(value.is_nil() || value->seqSize() == 0) {
                QplugAppendTextDisplay( "Input failed on join: Check Timeline is valid");
                if(value->nError() > 0){
                    for(size_t n=0; n<value->nError(); n++) QplugAppendTextDisplay(value->getError(n).c_str());
                }
                outputs[0]->setValue(value, false);
                return;
            }
        }
    }
    
	outputs[0]->setValue(value, true);
	return;
	
}

bool VJoinItem::hasGui()
{
	return gui!=NULL;
}


void VJoinItem::fromXMLElement(QDomElement& el, QGraphicsScene* scene)
{
	VCalcItem::fromXMLElement(el, scene);

	QDomElement guiElement=el.firstChildElement("GUI");
	QString val;
	
	if(getXMLNodeText(el, "XPosnInBlackBox", val))
	{
		positionInBlackBox.setX(val.toDouble());
	}
	if(getXMLNodeText(el, "YPosnInBlackBox", val))
	{
		positionInBlackBox.setY(val.toDouble());
	}

	if(getXMLNodeText(el, "Page", val))
	{
		page=val.toInt();
	}
	if(getXMLNodeText(el, "Join_No_Join", val))
	{
		joinSettings->noJoin=val.toInt();
	}
	if(getXMLNodeText(el, "Join_Algorithm", val))
	{
		joinSettings->algorithm=val.toInt();
	}
	if(getXMLNodeText(el, "Join_Width", val))
	{
		joinSettings->width=val.toDouble();
	}
	if(getXMLNodeText(el, "Join_Gap", val))
	{
		joinSettings->gap=val.toInt();
    }
    if(getXMLNodeText(el, "Join_End", val))
    {
        joinSettings->end=val.toInt();
    }
    if(getXMLNodeText(el, "Join_Min_Box", val))
    {
        joinSettings->minBoxSize=val.toInt();
    }
	if(getXMLNodeText(el, "Join_Gap_Size", val))
	{
		joinSettings->gapSize=val.toDouble();
	}
	if(!guiElement.isNull())
	{
		JoinItem* item=this->getGUI();
        scene->addItem(item);
        item->fromXMLElement(guiElement);
	}

	QDomElement inputsElement=el.firstChildElement("Input_Slots");
	if(!inputsElement.isNull())
	{
		QDomElement inputSlot=inputsElement.firstChildElement("Inslot");
		while(!inputSlot.isNull())
		{
			int index=inputSlot.attribute("Index", "-1").toInt();
			QString name=inputSlot.attribute("Name", "");
			QString description=inputSlot.attribute("Description", "");
			if(index>=0 && index< inputs.size())
			{
				inputs[index]->setName(name);
				inputs[index]->setDescription(description);
			}
			inputSlot=inputSlot.nextSiblingElement("Inslot");
		}
	}
	
	QDomElement outputsElement=el.firstChildElement("Output_Slots");
	if(!outputsElement.isNull())
	{
		QDomElement outputSlot=outputsElement.firstChildElement("Outslot");
		while(!outputSlot.isNull())
		{
			int index=outputSlot.attribute("Index", "-1").toInt();
			QString name=outputSlot.attribute("Name", "");
			QString description=outputSlot.attribute("Description", "");
			int isOpenSlot=outputSlot.attribute("open", "0").toInt();
			if(index>=0 && index< outputs.size())
			{
				outputs[index]->setName(name);
				outputs[index]->setDescription(description);
				outputs[index]->openSlot=(bool)isOpenSlot;
			}
			outputSlot=outputSlot.nextSiblingElement("Outslot");
		}
	}

}


JoinItem::JoinItem(VJoinItem* parentItem, VInSlot* input, VOutSlot* output, QGraphicsItem* parent):
		CalcItem(parent)
{
	this->parentItem=parentItem;
	
	outlineColor=QColor(50, 50, 70);
	color1=QColor(220, 220, 255);
	color2=QColor(200, 200, 240);

	
	title=parentItem->getName();
	
	width=BASE_WIDTH;
	height=BASE_HEIGHT;
	setRect(-width/2., -height/2., width, height);
	
	InSlot* islot=new InSlot(this, input);
	input->setGUI(islot);
	input->resetToolTips();
	islot->setPos(0., -height/2.);
	inSlots.push_back(islot);

	OutSlot* oslot=new OutSlot(this, output);
	output->setGUI(oslot);
	output->resetToolTips();
	oslot->setPos(0., height/2.);
	outSlots.push_back(oslot);

	rzButton->setPos(-width/2.+1.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);
	closeButton->setPos(width/2.-HEADER_HEIGHT/2., -height/2.+HEADER_HEIGHT/2.);
	errButton->setPos(width/2.-1.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);
}

JoinItem::~JoinItem()
{
	foreach(InSlot* slot, inSlots)
	{
		delete slot;
	}
	foreach(OutSlot* slot, outSlots)
	{
		delete slot;
	}
}

QVariant JoinItem::itemChange(GraphicsItemChange change, const QVariant& value)
{
	if (change == ItemPositionChange && scene()) {
		QPointF newPos = value.toPointF();
		
		updateConnectorsPositions();
		
		return newPos;
	}
	else if(change==ItemSelectedChange && scene())
	{
		if(value.toBool())
		{
			setZValue(1.);
		}
		else setZValue(0.);
		parentItem->setSelected(value.toBool());
	}
	return QGraphicsItem::itemChange(change, value);
}

void JoinItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
	CalcItem::paint(painter, option, widget);
}


QDomElement JoinItem::toXMLElement(QDomDocument& doc) const
{
	QDomElement el=CalcItem::toXMLElement(doc);
	return el;
}

void JoinItem::fromXMLElement(QDomElement& el)
{
	CalcItem::fromXMLElement(el);

}

