#include "triangulation.h"
#include <sys/time.h>
#include <time.h>
using namespace std;


void triangulation(std::list<Point2D>& in, std::list<Element2D>& out)
{
    Facet3D::currentStatus=1;
    list<Point3D*> pointsToProcess;
    in.sort();
    double add=1.;
    for(list<Point2D>::iterator iter=in.begin(); iter!=in.end(); iter++)
    {
        Point3D*p=new Point3D(*iter, (iter->x*iter->x+iter->y*iter->y)*(1.+add*DBL_EPSILON));
        add+=100.;
        pointsToProcess.push_front(p);
    }
    
    quickHull(pointsToProcess, out);
    
    for(list<Point3D*>::iterator iter=pointsToProcess.begin(); iter!=pointsToProcess.end(); iter++)
    {
        delete(*iter);
    }
}

// "in" should be sorted
double getMinDist(std::list<Point3D>& in)
{
    double min=-1.;
    for(list<Point3D>::iterator iter=in.begin(); iter!=in.end(); iter++)
    {
        list<Point3D>::iterator temp=iter;
        temp++;
        for( ; temp!=in.end(); temp++)
        {
            if(min<0.)
            {
                min=iter->distance2(*temp);
            }
            else
            {
                double dist=iter->distance2(*temp);
                if(dist<min) min=dist;
            }
        }
    }
    return sqrt(min);
}


void triangulation(std::list<Point3D>& in, std::list<Element3D>& out)
{   
    Facet4D::currentStatus=1;
	
    list<Point4D*> pointsToProcess;
    in.sort();
    double scale=1;//getMinDist(in);
    double add=1.;

    for(list<Point3D>::iterator iter=in.begin(); iter!=in.end(); iter++)
    {
        double temp_x=iter->x/scale*(1.+((double)rand())/RAND_MAX*.01-.005);
        double temp_y=iter->y/scale*(1.+((double)rand())/RAND_MAX*.01-.005);
        double temp_z=iter->z/scale*(1.+((double)rand())/RAND_MAX*.01-.005);
        Point4D* p=new Point4D(temp_x, temp_y, temp_z, (temp_x*temp_x*(1.+add*DBL_EPSILON)+temp_y*temp_y*(1.+add*DBL_EPSILON)+temp_z*temp_z*(1.+add*DBL_EPSILON)), iter->val, iter->area);
        p->orig_x=iter->x;
        p->orig_y=iter->y;
        p->orig_z=iter->z;
        add+=100.;
        pointsToProcess.push_front(p);
    }

    quickHull(pointsToProcess, out);

    for(list<Point4D*>::iterator iter=pointsToProcess.begin(); iter!=pointsToProcess.end(); iter++)
    {
        delete(*iter); // delete Point4D objects
    }
	pointsToProcess.clear();// delete pointers
}

void interpolation(list<Point2D>& points, list<Element2D>& delaunay)
{
    points.sort();
    delaunay.sort();
    
    bool found=false;
    double coeff1, coeff2, coeff3;
    for(list<Point2D>::iterator iter=points.begin(); iter!=points.end(); iter++)
    {
        iter->val=-1;
        found=false;
        list<Element2D>::iterator temp_i;
        // Look for the element containing the point;
        for(list<Element2D>::iterator i=delaunay.begin(); i!=delaunay.end() && !found; i++)
        {
            if(i->distToOrigin<iter->norm2)
            {
                temp_i=i;
                temp_i++;
                delaunay.erase(i);
                i=temp_i;
                i--;
            }
            else
            {
                if(i!=delaunay.end())
                {
                    found=i->findBarycentricCoordinates(*iter, coeff1, coeff2, coeff3);
                    if(found)
                    {
                        iter->val=coeff1*i->val[0]+coeff2*i->val[1]+coeff3*i->val[2];
                    }

                }
            }
        }// if(!found) cout << "not found" << endl;
    }
}

void interpolation(list<Point3D>& points, list<Element3D>& delaunay)
{
    points.sort();
    delaunay.sort();
    
	list<Element3D>::iterator low_i=delaunay.begin();

    bool found=false;
    double coeff1, coeff2, coeff3, coeff4;
    for(list<Point3D>::iterator iter=points.begin(); iter!=points.end(); iter++)
    {
        found=false;
        // Look for the element containing the point;
        for(list<Element3D>::iterator i=low_i; i!=delaunay.end() && !found; i++)
        {
            if(i->distToOrigin < iter->norm2)
            {
                low_i=i;
                low_i++;
            }
            else
            {
                if(i!=delaunay.end())
                {
                    found=i->findBarycentricCoordinates(*iter, coeff1, coeff2, coeff3, coeff4);
                    if(found)
                    {
                        iter->val=coeff1*i->val[0]+coeff2*i->val[1]+coeff3*i->val[2]+coeff4*i->val[3];
                    }

                }
            }
        } //if(!found) iter->val=-1.;
    }
}

