/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/
#include <stdlib.h>

#include <qtooltip.h> 
#include <qfiledialog.h> 
#include <qlayout.h> 
#include <qglobal.h>
#include <qsize.h>
#include <qtextedit.h>
#include <qsplitter.h>

#include "qtpl.h"
#include "qwui.h"
#include "qdutil.h"
#include "qplug_if.h"

using namespace QSAS; 

QTextEdit * activeqplugArea;
extern qwuiWindow* qsasWindow;

PlotID *plotWin_id;
PlugPlotWidgets *plotWidgets;

// qtplMenu Class
// --------------------

qtplMenu::qtplMenu(QWidget *parent, const char *name)
    :  QMenu(QString(name), parent)
{  
}

// -----------------------

void qtplMenu::qtplOpen(QAction* act){
 if( act->text().contains("&Join") == 1 || act->text().isEmpty()) return;
 
  QString name = QString(menuPath.toLatin1().data()); // force deep copy
          name += "/";           
          name += act->text();
	  name += ".qtpl";

  QtplOpenByName(name, this->parentWidget());
  
}

// -----------------------

void qtplMenu::addFolderToMenu(const char *folder, bool first)
{
  this->menuPath = QString(qsasWindow->qsas_home); // force deep copy
  this->menuPath += "/qtpl/";
  this->menuPath += folder;
  QDir d(this->menuPath);

  if( !d.exists() ) {
    QuMsgBar("Cannot find qtpl folder", this->menuPath.toLatin1().data(), true);
    return;
  }
  
  for(int i=0; i<d.count(); i++){
    QString name = QString(d[i].toLatin1().data());
    int posn = name.lastIndexOf(".qtpl");
    if(posn <0 ) continue;
    name.truncate(posn);
    this->addAction(name) ;
  }
  
  
  //Same thing for subfolders
  QStringList subFolders=d.entryList(QDir::Dirs/* | QDir::NoDotAndDotDot QT4 only*/);
  for(int i=0; i<subFolders.count(); i++)
  {
      if(!subFolders[i].startsWith("."))
      {
        QDir d2(QString(menuPath+"/"+subFolders[i]));
        QStringList filters;
		filters.clear();
		filters.push_back(QString("*.qtpl"));
        QStringList subFolders2=d2.entryList(filters, QDir::Files);
        
        if(subFolders2.count()>0)
        {
            qtplMenu * temp=new qtplMenu(this, subFolders[i].toLatin1().data());
            temp->menuPath=menuPath+"/"+subFolders[i];
			connect(temp, SIGNAL(triggered(QAction *)),temp,  SLOT(qtplOpen(QAction *)) );
            this->addMenu(temp);
          
            for(int j=0; j<subFolders2.count(); j++)
            {
                QString name = QString(subFolders2[j].toLatin1().data());
                int posn = name.lastIndexOf(".qtpl");
                if(posn <0 ) continue;
                name.truncate(posn);
                temp->addAction(name);
            }
        }
      }
 }
  

}

  // qplugWindow  class
  // -----------------------

 qplugWindow::qplugWindow(QWidget *parent, const char *name, QtplParserFile *qtplFile ) 
  : QMainWindow(0)
{
   
    qtplf = qtplFile;
    handle = NULL;

    if(plotWin_id) {
        delete plotWin_id;
        plotWin_id=NULL;
    }
    if(plotWidgets) {
        plotWidgets=NULL;
    }

    setAcceptDrops(true);
    setFocusPolicy(Qt::WheelFocus);

   setAttribute(Qt::WA_DeleteOnClose);
  
   menuBar()->clear();

   QMenu* file_menu = new QMenu(this);


   menuBar()->addSeparator();

   QMenu * help = new QMenu( this );

   menuBar()->addMenu(help );
   help->setTitle(" Help");
   help->addAction("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
   help->addSeparator();
   help->addAction("...Plugins", this, SLOT( launchManPlugins() ));
   help->addAction("...This Plugin Help", this, SLOT( launchPluginHelp() ));
   help->addAction("...This Plugin Documentation", this, SLOT( launchDocPlugin() ));
   help->addSeparator();
   help->addAction("Module Version", this, SLOT( launchVersion() ));
      
   QString space;
   QString smallSpace;
	// Space buttons out unless compact GUI is set
	if( QSAS_profiles->current()->GUImargins <= 1) 
	{
		// compact
		space ="";
		smallSpace ="";
	}
	else
	{
		space = "                       ";
		smallSpace = "   ";
	}
   
   QSplitter *plugVB = new QSplitter( Qt::Vertical, this);
   setCentralWidget( plugVB );
   

   scrollW = new QScrollArea(plugVB);
   scrollW->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
   scrollW->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);


   ioWidget=new QWidget(scrollW->viewport());
   ioHLayout=new QHBoxLayout(ioWidget);
  
   inputWidget=new QWidget(ioWidget);
   inputVLayout=new QVBoxLayout(inputWidget);
   outputWidget=new QWidget(ioWidget);
   outputVLayout=new QVBoxLayout(outputWidget);

   QplugInit(); //  initialize arguments interface
   
   inputWidget->setMinimumSize(inputWidget->sizeHint());
   outputWidget->setMinimumSize(outputWidget->sizeHint());

   ioHLayout->addWidget(inputWidget);
   ioHLayout->addWidget(outputWidget);

   ioWidget->setMinimumSize(ioWidget->sizeHint());

   scrollW->setWidget(ioWidget); 

   QWidget *vp=scrollW->viewport();
   vp->setMinimumWidth(QTPL_MINWIDTH);
   vp->adjustSize();
   scrollW->setMinimumSize(min(QTPL_MINWIDTH,vp->width()+5),\
		   min(QTPL_MINHEIGHT,vp->height()+5));

   scrollW->adjustSize();

   textArea = new QTextEdit(plugVB);
   textArea->setReadOnly(true);

   QWidget * buttons=new QWidget(plugVB);
   
   QGridLayout * buttons_layout=new QGridLayout(buttons);

   buttons_layout->setSpacing(QSAS_profiles->current()->GUImargins);
   buttons_layout->setMargin(QSAS_profiles->current()->GUImargins);

	buttons->setLayout(buttons_layout);
   
   
	runBtn = new QPushButton( " >>       Run       >> ", buttons);
	runBtn->setMinimumSize(runBtn->sizeHint());
	runBtn->setDefault(true);
	connect( runBtn, SIGNAL(clicked()), this, SLOT(runPlugin()) );
	buttons_layout->addWidget(runBtn, 0, 0);

	QLabel *spacer1 = new QLabel(space);
	buttons_layout->addWidget(spacer1, 0, 1);

	defaultBtn = new QPushButton( "   Default All   ", buttons );
	defaultBtn->setMinimumSize(defaultBtn->sizeHint());
	connect( defaultBtn, SIGNAL(clicked()), this, SLOT(defaultAll()) );
   
	buttons_layout->addWidget(defaultBtn, 0, 2);

	saveBtn = new QPushButton( " Save text area...", buttons );
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(saveLog()) );

	clearBtn = new QPushButton( " Clear text area", buttons );
	clearBtn->setMinimumSize(clearBtn->sizeHint());
	connect( clearBtn, SIGNAL(clicked()), this, SLOT(clearText()) );

	QLabel *smallSpacer = new QLabel(smallSpace);
	buttons_layout->addWidget(smallSpacer, 0, 3);
	   
	dismissBtn = new QPushButton( "     Dismiss     ", buttons );
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );

	buttons_layout->addWidget(saveBtn, 0, 4);
	buttons_layout->addWidget(clearBtn, 0, 5);
	buttons_layout->addWidget(dismissBtn, 0, 6);
   
	this->adjustSize();
 
  
}
void qplugWindow::keyPressEvent(QKeyEvent * e )
{  
  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if(   runBtn->isDefault()) {
		 runBtn->setDefault(false);
		 defaultBtn->setDefault(true);
	  }
	  else if(defaultBtn->isDefault()) {
		 defaultBtn->setDefault(false);
		 dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault())  {
		 dismissBtn->setDefault(false);
		 runBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(runBtn->isDefault()) {
        runPlugin();
	 }
	 else if (defaultBtn->isDefault()) defaultAll();
	 else if (dismissBtn->isDefault()) dismiss();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void qplugWindow::launchManual(){ 

 if(QuSlotDebug())
 {
	printf("qplugWindow::launchManual \n");
	fflush(stdout);
 }

   qsasWindow->help_window->manEntry("index.html");
}

void qplugWindow::launchManPlugins(){

 if(QuSlotDebug())
 {
	printf("qplugWindow::launchManPlugins \n");
	fflush(stdout);
 }

   qsasWindow->help_window->manEntry("QSAS_analysis.html");
}

void qplugWindow::launchPluginHelp(){

 if(QuSlotDebug())
 {
	printf("qplugWindow::launchPluginHelp \n");
	fflush(stdout);
 }

  QString text;
  text  = QString(qtplf->description);
  if( text == "" || text == " " ) text = QString("Description Not Available");

// set up scrolling text window
  QMainWindow *db = new QMainWindow( 0);
  QTextEdit *textArea = new QTextEdit(db);
  textArea->setReadOnly(true);
  textArea->setPlainText(text);
  
  db->setCentralWidget(textArea);
  db->resize(500,300);  
  
  db->show();


}

void qplugWindow::launchDocPlugin(){

 if(QuSlotDebug())
 {
	printf("qplugWindow::launchDocPlugin \n");
	fflush(stdout);
 }

  // Find html doc for this plugin
  QString search_path = QString(qtplName.toLatin1().data());// force deep copy

  int posn = search_path.lastIndexOf("/");
  search_path.remove(0, posn+1);
  search_path.prepend("PlugIns/");
  posn = search_path.lastIndexOf(".qtpl");
  search_path.truncate(posn);
  search_path += QString(".html");

    qsasWindow->help_window->manEntry(search_path.toLatin1().data());

}


void qplugWindow::launchVersion(){

 if(QuSlotDebug())
 {
	printf("qplugWindow::launchVersion \n");
	fflush(stdout);
 }


  QString text;
  text  = QString("Version Numbers...\n");
  text += QString(QSAS_VERSION_ID);
  text += "\n Plugin Template Parser ";
  text += QString(QTPL_VERSION);
  QMessageBox::information( this, "QSAS information", text.toLatin1().data(), "OK" );

}


void qplugWindow::dismiss(){

  activeqplugArea = NULL;
  close();
}

void qplugWindow::defaultAll(){
  activeqplugArea = this->textArea;
  QplugSetDefaults();  
 }


void qplugWindow::saveLog(){
	if(QuSlotDebug())
	{
		printf("qplugWindow::saveLog \n");
		fflush(stdout);
	}

	QString s = QFileDialog::getSaveFileName(this,  tr("Choose a filename to save under") , "", "Text (*.txt)" );
	
	if(!s.isEmpty())
	{
		ofstream file;
		file.open(s.toLatin1().data());
		if(!file.fail())
		{
			file << textArea->toPlainText().toLatin1().data();
			file.close();
		}
	}
}

void qplugWindow::clearText(){
	if(QuSlotDebug())
	{
		printf("qplugWindow::clearText \n");
		fflush(stdout);
	}

    textArea->clear();
		
}


// ----------------------------------------------------

void qplugWindow::runPlugin(){

    errorSet = 0;
    
	activeqplugArea = this->textArea;
      
	QplugAppendTextDisplay ("\n Running... ");
	
	vector<DvObject_var> inputs;
	vector<DvObject_var> outputs;
	inputs.clear();
	outputs.clear();

#ifdef WINDOWS
	typedef QplugReturnStatus (__cdecl *MYPROC)(vector<DvObject_var>&, vector<DvObject_var>& );
	MYPROC fptr;
#else
    QplugReturnStatus (*fptr)(vector<DvObject_var>&, vector<DvObject_var>&)=NULL;
#endif
	QplugReturnStatus f_status;	
	
	//  Check there is a template/plug-in loaded to run,
	//  and return promptly if there is not.

	if (this->qtplf == NULL || !(this->qtplf->openedOK))  {
    	QplugAppendTextDisplay("\nERROR: no plug-in template loaded.\n");
        errorSet = 1;
    	return;
	}
	
	// work through input args, type-checking or creating-from-value 
	QplugBuildInputArgsList(inputs);
	
	if(inputs.size() == 0) {
		QplugAppendTextDisplay ("\nQplug> ERROR: Not attempting plug-in load because of arg error.\n");
        errorSet = 2;
		return;
	}

	// Added by AJA for Arithmetic

	for (int o_count = 0; o_count < oSlots.size(); o_count++) {
       SetSlotText(o_count, oSlots[o_count]->ioSlot->text().toLatin1().data());
       oSlots[o_count]->argIndex  =  iSlots.size() + o_count;                         
	}  
	
	// Try to find entry point and the full path to .so used
	QplugGetEntryName();     

	// dynamic load of plugin
    
	QplugAppendTextDisplay ("Qplug> using entry name... ");     
	QplugAppendTextDisplay (try_entry_name.toLatin1().data());
	QplugAppendTextDisplay ("\n");     
      
#ifdef WINDOWS
	handle=LoadLibrary(PathAndNameSO.toLatin1().data());
	if( !handle) {
      
          QplugAppendTextDisplay ("DVOS load: LoadLibrary returned null, abandoning dynamic-load for\n");
          QplugAppendTextDisplay(qtplf->object_file.toLatin1().data());
          QplugAppendTextDisplay(" with path ");
          QplugAppendTextDisplay(PathAndNameSO.toLatin1().data());
          QplugAppendTextDisplay("\n");
          return;
	}
		
	if ( (fptr = (MYPROC) GetProcAddress(handle, try_entry_name.toLatin1().data())/*(QplugReturnStatus_e (*)(QplugArgList *)) GetProcAddress(handle, try_entry_name.toLatin1().data())*/) == NULL )  {
      
          QplugAppendTextDisplay ("plugin entry point ERROR: GetProcAddress lookup failed, dlerror contents are:\n  ");
          QplugAppendTextDisplay("\n");
		  if( !FreeLibrary(handle)) QplugAppendTextDisplay("Error freeing the library");
          return;
	}
#else
    handle = dlopen(PathAndNameSO.toLatin1().data(), RTLD_LAZY);
	if( handle == NULL ) {
          QplugAppendTextDisplay ("DVOS load: dlopen returned null, abandoning dynamic-load for\a\n");
          QplugAppendTextDisplay(qtplf->object_file.toLatin1().data());
          QplugAppendTextDisplay(" with path ");
          QplugAppendTextDisplay(PathAndNameSO.toLatin1().data());
          QplugAppendTextDisplay("\n");
          QplugAppendTextDisplay(dlerror());
          return;
	}
		
	if ( (fptr = (QplugReturnStatus_e (*)(vector<DvObject_var>&, vector<DvObject_var>&)) dlsym(handle, try_entry_name.toLatin1().data())) == NULL )  {
      
          QplugAppendTextDisplay ("plugin entry point ERROR: dlsym lookup failed, dlerror contents are:\n  ");
          QplugAppendTextDisplay(dlerror());
          QplugAppendTextDisplay("\n");
          if( dlclose(handle) != 0 ) QplugAppendTextDisplay(dlerror());
          handle = NULL;
          return;
	}
#endif


    f_status = (*fptr)(inputs, outputs);  // ATTEMPT DYNAMIC FUNCTION CALL 
	

	switch (f_status)  {

        case QPLUG_WARNING:  // print warning, then fall thru QPLUG_SUCCESS case to put output args on working list
		{
              QplugAppendTextDisplay  ("Qplug> WARNING: plug-in returned warning\n");
              QplugAppendTextDisplay  ("      use the output data objects at your own risk!\n");
		}
        case QPLUG_SUCCESS:
		{
			// put output data-objects on working list
			int nReturned=0;
			for (int o_count = 0; o_count < oSlots.size(); o_count++) {
	      
				// don't crash if plugin doesn't put output on list.
				if ( o_count < outputs.size() ) {
					nReturned++;
					DvString usrStr = GetSlotText(o_count);  // returned from plug-in
                    usrStr.clean();
                    if( !usrStr.empty() ) QwuiPutOnWL(outputs[o_count], usrStr.c_str());
					
				}  // END of test for non-null output

			}  // END of for loop for output args
			
			if(nReturned == 1) QplugAppendTextDisplay  ("Qplug> Placing output data object on working list...");
			else if(nReturned > 1)  QplugAppendTextDisplay  ("Qplug> Placing output data objects on working list...");

			QplugAppendTextDisplay("finished.\n");

			break;

		}
        case QPLUG_FAILURE:  // print error, don't touch output args
		{
            QplugAppendTextDisplay  ("Qplug> ERROR: plug-in returned failure.\n");
            errorSet = 3;
            break;

		}
        default:  // print panic error and don't touch output args!
		{
            errorSet = 3;
            QplugAppendTextDisplay ("Qplug> PANIC: plug-in returned unrecognised status code.\n");
            QplugAppendTextDisplay ("      Check that plug-in returns QplugReturnStatus_e\n");
            QplugAppendTextDisplay ("      No output data objects generated.\n");

            break;
		}
	}  // END of switch on f_status
    
qApp->processEvents();

    
	return;

}

// ----------------------------------------------------

void qplugWindow::QplugBuildInputArgsList (vector <DvObject_var> &inputs){
  
  // Work through Qplug input args, type-checking or creating-from-value 

  for (int i = 0; i < iSlots.size(); i++) {
    // keep track of it's index in the arg list
    iSlots[i]->argIndex = i;

    if (iSlots[i]->type == "STRING")  // copy the string value from the widget
    {
         DvObject_var ptr =  new DvObject(iSlots[i]->ioSlot->text().toLatin1().data());
         inputs.push_back(ptr);
    }
    else{

		// ask slot for object
		DvObject_var slotObj = iSlots[i]->ioSlot->getObject();
		inputs.push_back( slotObj);
    }


  }  // END of for loop through input_args


  return;

}


void qplugWindow::QplugInit (){

  if(!qtplf || !(qtplf->openedOK))
    return;

  //  Work through the function argument list and create the slots  
  int nOutputs=0;
  for (int i=0; i<qtplf->qtpl_vars.size(); i++) {

    QtplParserVar *pVar = qtplf->qtpl_vars[i];
    if(pVar == NULL) continue;
    
    QString title = pVar->public_name;
    QString text  = pVar->description;

      if(pVar->ioType == "INPUT") 
      {
        QplugSlot *next = new QplugSlot(title, text, pVar->var_type, inputWidget);
	    if ( !(pVar->default_value.isEmpty()) )
	      next->defaultValue = QString(pVar->default_value.toLatin1().data());
	    else
	      next->defaultValue = QString("");

	    inputVLayout->addWidget(next);
	  
	    iSlots.push_back(next);
      }
      else if(pVar->ioType == "OUTPUT"){
		nOutputs++;
        QplugSlot *next = new QplugSlot(title, text, QString("NAME"), outputWidget);
	    if ( !(pVar->default_value.isEmpty()) )
	      next->defaultValue = QString(pVar->default_value.toLatin1().data());
	    else
	      next->defaultValue = QString("");

	    outputVLayout->addWidget(next);

	    oSlots.push_back(next);
      }
      else{
        QplugAppendTextDisplay("ERROR: unknown argument type for: ");
        QplugAppendTextDisplay(pVar->public_name.toLatin1().data());
        QplugAppendTextDisplay("\n");
      }

  }  // END of loop in qtplf args

if(nOutputs == 0)
{
	QplugSlot *dummy = new QplugSlot("No output objects", " See text window \n New display and/or attaches results as attributes", QString("NAME"), outputWidget);
	dummy->defaultValue = QString("");
	outputVLayout->addWidget(dummy);
	dummy->setDisabled(true);

}
   /* Set default values for args that have them */

  QplugSetDefaults();

}


void qplugWindow::QplugSetDefaults (){
  
  //  Set all arguments to their default values.  

  for (int i=0; i <iSlots.size() ; i++){
     iSlots[i]->ioSlot->setText(iSlots[i]->defaultValue);
     iSlots[i]->ioSlot->reValidate();
  }
  
  for (int i=0; i <oSlots.size() ; i++){
     oSlots[i]->ioSlot->setText(oSlots[i]->defaultValue);
     oSlots[i]->ioSlot->reValidate();
  }
  
  return;
}

void qplugWindow::QplugGetEntryName (){

  FILE *fp;
  static const char * str_NULL = "";
 
  // find path to loaded .qtpl file as a possible location for .so
  QString qtplPath = QString(qtplName.toLatin1().data()); // deep copy
  
  int posn = qtplPath.lastIndexOf('/');
  if(posn >= 0)  qtplPath.truncate(posn);
  else           qtplPath = "./";
  
  
  QString file_stem = QString(qtplf->object_file.toLatin1().data());
  posn = file_stem.lastIndexOf('.');
  if(posn >= 0)  file_stem.truncate(posn);

  QString user_lib = QSAS_profiles->current()->UserLibPath;
  
  QString search_path = QString("");		
  if( (qtplf->object_file)[0] != '/') 
      search_path = QString("./");		

  search_path += qtplf->object_file;
  QString windowsPath = QString(qtplf->object_file.toLatin1().data());
  int ptr = windowsPath.indexOf(".so");
  windowsPath.replace(ptr, 3, ".dll");

  fp = fopen(search_path.toLatin1().data(), "r");
  if ( fp == NULL ){
       search_path = qtplPath; 
       search_path += "/"; 
       search_path += qtplf->object_file;
       fp = fopen(search_path.toLatin1().data(), "r");
  }
  if ( fp == NULL && strcmp(user_lib.toLatin1().data(), str_NULL) != 0){
       search_path = user_lib; 
       search_path += "/";
       search_path += qtplf->object_file;
       fp = fopen(search_path.toLatin1().data(), "r");
  }
  if ( fp == NULL){
       search_path = QString(qsasWindow->qsas_home); 
       search_path += "/lib/";
       search_path += qtplf->object_file;
       fp = fopen(search_path.toLatin1().data(), "r");
  }
  // try for Windows .dl file
  if ( fp == NULL ){
       search_path = qtplPath; 
       search_path += "/"; 
       search_path += windowsPath;
       fp = fopen(search_path.toLatin1().data(), "r");
  }
  if ( fp == NULL && strcmp(user_lib.toLatin1().data(), str_NULL) != 0){
       search_path = user_lib; 
       search_path += "/";
       search_path += windowsPath;
       fp = fopen(search_path.toLatin1().data(), "r");
  }
  if ( fp == NULL){
       search_path = QString(qsasWindow->qsas_home); 
       search_path += "/lib/";
       search_path += windowsPath;
       fp = fopen(search_path.toLatin1().data(), "r");
  }

  if ( fp == NULL)  {
    QplugAppendTextDisplay(qtplf->object_file.toLatin1().data());
    QplugAppendTextDisplay(" < cannot locate dynamic library  \n");
    
    try_entry_name = "";
    return; 
  }
      
  // We have the correct path if we get here. 

  // keep full path to .so for dynamic load

  PathAndNameSO = QString(search_path.toLatin1().data());  // deep copy

  
  // Close file and use system call to find entry point
    
  fclose(fp);
  
  try_entry_name = QString(qtplf->entry_name.toLatin1().data());
  
  return;
}



// QplugSlot class
//-----------------

 QplugSlot::QplugSlot(const QString &title, const QString &text, 
                      QString varType,
                      QWidget *parent, const char *name)
    :  QGroupBox(title, parent)
{ 

  type = varType;
  
  ioSlot = new QguWLinput(this, "ioSlot");
  ioSlot->setEnabled(true);
  ioSlot->setMinimumWidth( 300 );

  QVBoxLayout * mainLayout=new QVBoxLayout;
  
  mainLayout->addWidget(ioSlot);


  QLabel *label = new QLabel(text, this);
  label->setAlignment( Qt::AlignTop | Qt::AlignLeft );

  mainLayout->addWidget(label);
  
  setLayout(mainLayout);

         if(type == "NAME"){
            toolTipText = "NAME OF OUTPUT OBJECT FOR WORKING LIST";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
         }
         else if(type == "ANY_DATA_OBJECT"){
            toolTipText = "ANY DATA OBJECT";
 	        ioSlot->setAcceptable(Qcondition());
 	        ioSlot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
 	        ioSlot->addAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
 	        ioSlot->addAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
            // no constraints
         }
         else if(type == "INTEGER"){
            toolTipText = "INTEGER";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));			
         }
         else if(type == "FLOAT"){
            toolTipText = "FLOAT";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));	
         }
         else if(type == "VECTOR"){
            toolTipText = "VECTOR";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_VECTOR, QS_NOT_SEQ, QS_ANY));
         }
         else if(type == "MATRIX"){
            toolTipText = "MATRIX";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ARRAY, QS_NOT_SEQ, QS_ANY));	
         }
         else if(type == "STRING"){
            toolTipText = "STRING";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
         }
         else if(type == "TIME"){
            toolTipText = "Single time value (can extract from interval or series)";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
         }
         else if(type == "TIMETAGS"){
            toolTipText = "TIMETAGS";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_SEQ, QS_ANY));
         }
         else if(type == "TIME_INTERVAL"){
            toolTipText = "TIME INTERVAL";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));
 	        ioSlot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
         }
         else if(type == "SCALAR_SERIES"){
            toolTipText = "SCALAR DATA SERIES";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_ANY));
         }
         else if(type == "VECTOR_SERIES"){
            toolTipText = "VECTOR DATA SERIES";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_VECTOR, QS_SEQ, QS_ANY));
         }
         else if(type == "MATRIX_SERIES"){
            toolTipText = "MATRIX DATA SERIES";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ARRAY, QS_SEQ, QS_ANY));
         }
         else if(type == "ANY_TIME_SERIES"){
            toolTipText = "ANY TIME SERIES";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_SEQ, QS_TT));
         }
         else if(type == "SCALAR_TIME_SERIES"){
            toolTipText = "SCALAR TIME SERIES";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_TT));
         }
         else if(type == "VECTOR_TIME_SERIES"){
            toolTipText = "VECTOR TIME SERIES";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_VECTOR, QS_SEQ, QS_TT));
		 }
         else if(type == "MATRIX_TIME_SERIES"){
            toolTipText = "MATRIX TIME SERIES";
 	        ioSlot->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ARRAY, QS_SEQ, QS_TT));
         }
         else
            toolTipText = "?? unknown data type ??";
        
	this->setToolTip(toolTipText);
}



// functions
// -----------


qplugWindow * QtplOpenByName(QString qtplName, QWidget * parent, bool show){
  
  qplugWindow *qplug_widget=0;
  QtplParserFile *qtplFile;
  QString title;
  
  activeqplugArea = NULL;
  
  if (qtplName.isEmpty() ){
    QuMsgBar("no template specified", false);
    return qplug_widget;
  }
  
   // attempt to open and parse template file... 
     
  qtplFile = new QtplParserFile(qtplName);
  
  if (qtplFile->openedOK) {
    
   // create empty qplugWindow widget
    qplug_widget = new qplugWindow(parent, "pluginWindow", qtplFile);
    activeqplugArea = qplug_widget->textArea;
    
    qplug_widget->qtplName = QString(qtplName.toLatin1().data());

    // set title bar
    
    title = QString(qtplFile->public_name.toLatin1().data());
    qplug_widget->setWindowTitle(title);
    
    qplug_widget->move(qsasGUISize().width(),30);
	qplug_widget->resize(qplug_widget->sizeHint());
    if(show) qplug_widget->show();
    else qplug_widget->hide();
  }

    return qplug_widget;
}


static DvString slot_text[N_MAX_QPLUG_OUTPUT_ARGS];

const char * GetSlotText(int n_slot){

    return slot_text[n_slot].c_str();
}

void SetSlotText(int n_slot, const char * text){ 

      slot_text[n_slot] = text;
    
      return;
}


void QplugSetTextDisplay (QTextEdit * textArea)
{
	activeqplugArea = textArea;
}

void QplugAppendTextDisplay (DvString txt){
	QplugAppendTextDisplay(txt.c_str());
}

void QplugAppendTextDisplay (const char *text)
{
    if(activeqplugArea == NULL) {
      cout <<  text ;
      QuMsgBar("see console... ", false);
      return;
    }
    QString t(text);
    if(t.endsWith("\n")) t=t.left(t.length()-1); // because append() adds a \n
    activeqplugArea->append(t);

	qApp->processEvents(); // do it now
    return;
}


// ----------------------------------------------------

int runBatchPlugin(DvString name, QWidget *parent, vector <DvString> &in, vector <DvString> &out){
    
    activeqplugArea = NULL;
    
    qplugWindow *qplug_widget = QtplOpenByName(name.c_str(), parent, false); // create but don't show
    
    if(qplug_widget == 0) return -1;
    
    for(int i=0; i<qplug_widget->iSlots.size(); i++){
        if(in.size() > i && !(in[i].empty()) ) qplug_widget->iSlots[i]->ioSlot->setText(in[i].c_str());
    }
    
    for(int i=0; i<qplug_widget->oSlots.size(); i++){
        if(out.size() > i && !(out[i].empty()) ) qplug_widget->oSlots[i]->ioSlot->setText(out[i].c_str());
    }

    qplug_widget->runPlugin();
    
    qApp->processEvents();
    
    int err = qplug_widget->errorSet;
    
    if(err > 0) qplug_widget->show();
    else delete qplug_widget;
    
    return err;
}

