/* makeArrow.cc Jeremy Mitchell 06/08/2009 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "qplug_if.h"
#include "qdutil.h"
#include "DvObj.h"
#include "Xrefs.h"

extern "C" QplugReturnStatus DvmakeArrow(vector <DvObject_var>&, vector <DvObject_var>& );

using namespace QSAS;

QplugReturnStatus DvmakeArrow(vector <DvObject_var>& inputs, vector <DvObject_var>& outputs) {
  // Unpack input data
  
  DvObject_var pos1_in    = inputs[0];
  DvObject_var deltax_in  = inputs[1];
  DvObject_var scale_in   = inputs[2];
  DvObject_var head_in    = inputs[3];
  DvObject_var hscale_in  = inputs[4];
  
  //*********************************************************
  
  //***************
  // Test Inputs:
  //***************
  
  
  if (!pos1_in->isThreeVector()) {
    QplugAppendTextDisplay("makeArrow error: Position 1 is not a Vector\n");
    return QPLUG_FAILURE;
  }
  
  if (!deltax_in->isThreeVector()) {
    QplugAppendTextDisplay("makeArrow error:delta x is not a Vector\n");
    return QPLUG_FAILURE;
  }
  
  if (scale_in.is_nil() || scale_in->arraySize() > 1) {
    QplugAppendTextDisplay("makeArrow error: Scale is not a Scalar\n");
    return QPLUG_FAILURE;
  }
  
  if (head_in.is_nil() || head_in->arraySize() > 1) {
    QplugAppendTextDisplay("makeArrow error: Draw Head is not a Scalar\n");
    return QPLUG_FAILURE;
  }
  
  int head = head_in->asInt();
  if ((head < 0) || (head > 3)) {
    QplugAppendTextDisplay("makeArrow error: Draw Head must be between 0 and 3\n");
    return QPLUG_FAILURE;
  }
  
  if (hscale_in.is_nil() || hscale_in->arraySize() > 1) {
    QplugAppendTextDisplay("makeArrow error: Width is not a Scalar\n");
    return QPLUG_FAILURE;
  }
  
  float hScale = (float) hscale_in->asDouble();
  if (hScale < 0) {
    QplugAppendTextDisplay("makeArrow WARNING: Width is less than 0\n\tArrow may point wrong way!");
  }
  
 //******************** calculate *************************
  
  DvObject_var arrow;
  DvObject_var dep0;

  if (head == 0)
    arrow = new DvObject(0.0, DIM3, 2);
  else
    arrow = new DvObject(0.0, DIM3, 5);
    
  arrow->dbl(0,0) = pos1_in->dbl(0);
  arrow->dbl(0,1) = pos1_in->dbl(1);
  arrow->dbl(0,2) = pos1_in->dbl(2);
  
  double delx = deltax_in->dbl(0) * scale_in->dbl(0);
  double dely = deltax_in->dbl(1) * scale_in->dbl(0);
  double delz = deltax_in->dbl(2) * scale_in->dbl(0);
  arrow->dbl(1,0) = pos1_in->dbl(0) + delx;
  arrow->dbl(1,1) = pos1_in->dbl(1) + dely;
  arrow->dbl(1,2) = pos1_in->dbl(2) + delz;
  
  if (head != 0) {
    double phi, xarr, yarr, zarr, width;
    
    switch (head) {
      case 1:
        width = hScale*sqrt(delx*delx + dely*dely);
        phi = atan2(dely, delx);
        xarr = -sin(phi)*width;
        yarr = cos(phi)*width;
	zarr = 0;
	break;
      case 2:
        width = hScale*sqrt(delx*delx + delz*delz);
        phi = atan2(delz, delx);
        xarr = -sin(phi)*width;
        yarr = 0;
	zarr = cos(phi)*width;
	break;
      case 3:
        width = hScale*sqrt(dely*dely + delz*delz);
        phi = atan2(dely, delz);
        xarr = 0;
        yarr = cos(phi)*width;
	zarr = -sin(phi)*width;
	break;
    }
    
    double onemw = 1. - hScale;
    arrow->dbl(2,0) = pos1_in->dbl(0) + onemw*delx + xarr;
    arrow->dbl(2,1) = pos1_in->dbl(1) + onemw*dely + yarr;
    arrow->dbl(2,2) = pos1_in->dbl(2) + onemw*delz + zarr;
    
    arrow->dbl(3,0) = arrow->dbl(1,0);
    arrow->dbl(3,1) = arrow->dbl(1,1);
    arrow->dbl(3,2) = arrow->dbl(1,2);
    
    arrow->dbl(4,0) = pos1_in->dbl(0) + onemw*delx - xarr;
    arrow->dbl(4,1) = pos1_in->dbl(1) + onemw*dely - yarr;
    arrow->dbl(4,2) = pos1_in->dbl(2) + onemw*delz - zarr;
    
    dep0 = new DvObject(0.0, 5);
    
    size_t d0=0;
    if (head == 3)
      d0 = 1;
    for (int i=0; i<5; i++) dep0->dbl(i) = arrow->dbl(i,d0);
    
  } 
  else {
    dep0 = new DvObject(0.0, 2);
    dep0->dbl(0) = arrow->dbl(0,0); 
    dep0->dbl(1) = arrow->dbl(1,0);
  }
  
  // Add attributes:
  
  arrow->copy_xrefs_from(pos1_in);
  dep0->copy_xrefs_from(pos1_in);
  
  // Narrow type to a scalar
  if (dep0->xref_exists(DEPEND_0)) {
    dep0->delete_xref(DEPEND_0);
    QplugAppendTextDisplay("Deleting arrow DEPEND_0\n");
  }
  if (dep0->xref_exists(DEPEND_1)) {
    dep0->delete_xref(DEPEND_1);
    arrow->delete_xref(DEPEND_1);
    QplugAppendTextDisplay("Deleting arrow DEPEND_1\n");
  }
  
  dep0->change_xref(FRAME, "scalar>na");
  arrow->change_xref(DEPEND_0, dep0);
    
    
  //***********************************************************
  
  // Assign data to output:
  
  outputs.push_back( arrow );
  
  QplugAppendTextDisplay ("makeArrow>> finished...\n");
  
  return QPLUG_SUCCESS;
}
