/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

/**********************************************************************
*
*  QSAS Plugin to shift timetags of a time series
*
*
*  Inputs:  Time Series
*           dt          time (secs) to ADD to each timetag
*
*  Output:  Cloned Time series with shifted tags and all xrefs
*
*  History:  sjs September 2001
***********************************************************************/

#include "qdutil.h"
#include "qplug_if.h"
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "DvObj.h"

using namespace QSAS;

// local prototype
extern "C" QplugReturnStatus DvMakeMonotonic (vector <DvObject_var>&, vector <DvObject_var>& );

QplugReturnStatus DvMakeMonotonic(vector <DvObject_var>& inputs, vector <DvObject_var>& outputs)
{

// call-list arguments

DvObject_var tsin_obj;
DvObject_var ts_obj;
DvObject_var tol_ptr;
double tolerance;

//	Unpack the input data objects
//
//	..first input item is  time series

//  set pointer to objects and check objects

tsin_obj = inputs[0];

if (tsin_obj.is_nil()){
   QplugAppendTextDisplay("TimeShift> Input TS is nil\n");
   return QPLUG_FAILURE;
}

// re-order time tags
ts_obj = tsin_obj->makeMonotonic();

tol_ptr = inputs[1];

if (tol_ptr.is_nil()){
   QplugAppendTextDisplay("Tolerence not set: Keeping all time tags\n");
}
else
{
	// remove duplicate time tags (closer than tolerance)
	tolerance = tol_ptr->asDouble();
	
	ts_obj->stripDuplicates(tolerance);
	
}

outputs.push_back(ts_obj);

QplugAppendTextDisplay ("MakeMonotonic: Done\n");
  
return QPLUG_SUCCESS;

}

