#include <stdio.h>
#include <math.h>

#ifdef HAVE_IEEEFP_H
 #include <ieeefp.h>
#endif

#include <float.h>
#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"
#include <QString>
#include "qdutil.h"
#include "DvObj.h"


using namespace QSAS;

// local prototypes
extern "C" QplugReturnStatus Dvstats (vector<DvObject_var>&, vector<DvObject_var>&);

// -------------------------------------------------

QplugReturnStatus Dvstats(vector<DvObject_var>& inputs, vector<DvObject_var>& outputs)
{
	 
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data objects

	DvObject_var dobj = inputs[0];
   
	if(dobj.is_nil()  ){
		QplugAppendTextDisplay( "Missing input object in stats filter\n" );
		return QPLUG_FAILURE; 
	}	

	// is sequence a matrix sequence
	if(dobj->is_dbl())
	{
		
		double fillval = -10.e30;
		DvObject_var 	m_fill = new DvObject(fillval, dobj->Dims()); // default if matrix not found

        DvObject_var fillObj = dobj->get_xref(FILL_VALUE);
        if( fillObj.is_ok()) 
		{
			if(fillObj->arraySize() == 1) 
			{
				m_fill = new DvObject(fillObj->asDouble(), dobj->Dims());
			}
			else
            {
                if(fillObj->arraySize() == dobj->arraySize())
                {
                    m_fill = fillObj;
                }
                else m_fill = new DvObject(fillObj->asDouble(), dobj->Dims());
                    
            }
		}
		
		// initialise matrix output sequence and reserve max space
		DvObject_var m_mean = new DvObject(0.0, dobj->Dims());
		DvObject_var m_rms = new DvObject(0.0, dobj->Dims());
		DvObject_var m_min = new DvObject(DBL_MAX, dobj->Dims() );
		DvObject_var m_max = new DvObject(-DBL_MAX, dobj->Dims());
		DvObject_var n_mseq = new DvObject((int)0, dobj->Dims());

		int nseq = 0;
		QString str_ans = QString("seq size: %1\n").arg(dobj->seqSize());
		QplugAppendTextDisplay(str_ans.toLatin1().data());
		
		for(size_t i=0; i<dobj->seqSize(); i++)
		{
			// get underlying data for matrix element
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				if( isFill(dobj->dbl(i,j), m_fill->dbl(j)) ) continue;

				m_mean->dbl(j) += dobj->dbl(i,j);
				if( m_min->dbl(j) > dobj->dbl(i,j)) m_min->dbl(j) = dobj->dbl(i,j);
				if( m_max->dbl(j) < dobj->dbl(i,j)) m_max->dbl(j) = dobj->dbl(i,j);
				n_mseq->itg(j) += 1;

			}
		}

		for(size_t j=0; j<m_mean->arraySize(); j++)
		{
			if(n_mseq->itg(j) >= 1) 
			{
				m_mean->dbl(j) /= (double) n_mseq->itg(j);
			}
		}

		// now we have mean, can get rms
		for(size_t i=0; i<dobj->seqSize(); i++)
		{
			// get underlying data for matrix element
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				if(isFill(dobj->dbl(i,j), m_fill->dbl(j)) ) continue;

				double diff = m_mean->dbl(j) -  dobj->dbl(i,j);
				diff *= diff;
				m_rms->dbl(j) += diff;
			}

		}
		for(size_t j=0; j<m_mean->arraySize(); j++)
		{
			if(n_mseq->itg(j) >= 1) 
			{
				m_rms->dbl(j) /= (double)n_mseq->itg(j);
				m_rms->dbl(j) = sqrt(m_rms->dbl(j));
			}
		}
		
		dobj->change_xref("Mean", m_mean);
		dobj->change_xref("Standard Deviation", m_rms);
		dobj->change_xref("Minumum", m_min);
		dobj->change_xref("Maximum", m_max);
		dobj->change_xref("NumberInMean", n_mseq);

		if(dobj->xref_exists(SI_CONVERSION))
		{
			DvObject_var SI_xref = dobj->get_xref(SI_CONVERSION);
			m_mean->change_xref(SI_CONVERSION, SI_xref);
			m_rms->change_xref(SI_CONVERSION, SI_xref);
			m_min->change_xref(SI_CONVERSION, SI_xref);
			m_max->change_xref(SI_CONVERSION, SI_xref);
		}
		if(dobj->xref_exists(DEPEND_1))
		{
			DvObject_var dep1_xref = dobj->get_xref(DEPEND_1);
			m_mean->change_xref(DEPEND_1, dep1_xref);
			m_rms->change_xref(DEPEND_1, dep1_xref);
			m_min->change_xref(DEPEND_1, dep1_xref);
			m_max->change_xref(DEPEND_1, dep1_xref);
		}
		if(dobj->xref_exists(DEPEND_2))
		{
			DvObject_var dep2_xref = dobj->get_xref(DEPEND_2);
			m_mean->change_xref(DEPEND_2, dep2_xref);
			m_rms->change_xref(DEPEND_2, dep2_xref);
			m_min->change_xref(DEPEND_2, dep2_xref);
			m_max->change_xref(DEPEND_2, dep2_xref);
		}
		if(dobj->xref_exists(DEPEND_3))
		{
			DvObject_var dep3_xref = dobj->get_xref(DEPEND_3);
			m_mean->change_xref(DEPEND_3, dep3_xref);
			m_rms->change_xref(DEPEND_3, dep3_xref);
			m_min->change_xref(DEPEND_3, dep3_xref);
			m_max->change_xref(DEPEND_3, dep3_xref);
		}
		if(dobj->isThreeVector())
		{
			if(dobj->xref_exists(REPRESENTATION_1))
			{
				DvObject_var rep_xref = dobj->get_xref(REPRESENTATION_1);
				m_mean->change_xref(REPRESENTATION_1, rep_xref);
				m_rms->change_xref(REPRESENTATION_1, rep_xref);
				m_min->change_xref(REPRESENTATION_1, rep_xref);
				m_max->change_xref(REPRESENTATION_1, rep_xref);
			}
			if(dobj->xref_exists(COORDINATE_SYSTEM))
			{
				DvObject_var coord_xref = dobj->get_xref(COORDINATE_SYSTEM);
				m_mean->change_xref(COORDINATE_SYSTEM, coord_xref);
				m_rms->change_xref(COORDINATE_SYSTEM, coord_xref);
				m_min->change_xref(COORDINATE_SYSTEM, coord_xref);
				m_max->change_xref(COORDINATE_SYSTEM, coord_xref);
			}
			if(dobj->xref_exists(TENSOR_ORDER))
			{
				DvObject_var order_xref = dobj->get_xref(TENSOR_ORDER);
				m_mean->change_xref(TENSOR_ORDER, order_xref);
				m_rms->change_xref(TENSOR_ORDER, order_xref);
				m_min->change_xref(TENSOR_ORDER, order_xref);
				m_max->change_xref(TENSOR_ORDER, order_xref);
			}
			if(dobj->xref_exists(FRAME))
			{
				DvObject_var frame_xref = dobj->get_xref(FRAME);
				m_mean->change_xref(FRAME, frame_xref);
				m_rms->change_xref(FRAME, frame_xref);
				m_min->change_xref(FRAME, frame_xref);
				m_max->change_xref(FRAME, frame_xref);
			}
			
		}
		
        DvString tmp;
        int prec = tmp.getDblPrecision();
        tmp.setDblPrecision(8);
        
		QplugAppendTextDisplay(" Min: \n");
        QplugAppendTextDisplay(m_min->stringArray());
        
        QplugAppendTextDisplay("\n Max: \n");
        QplugAppendTextDisplay(m_max->stringArray());

        tmp.setDblPrecision(6);
       
        QplugAppendTextDisplay("\n Mean: \n");
        QplugAppendTextDisplay(m_mean->stringArray());
       
        QplugAppendTextDisplay("\n Standard Deviation: \n");
        QplugAppendTextDisplay(m_rms->stringArray());

		QplugAppendTextDisplay ("\nMin, Max, Mean & Standard Deviation added as Attributes to input object \n");
		QplugAppendTextDisplay ("(or cached component if applicable). \n");

        // reset
        tmp.setDblPrecision(prec);

        
		DvObject_var tsout_obj = new DvObject(dobj);
		outputs.push_back(tsout_obj);
		
		return QPLUG_SUCCESS;

	}




   // string and other input types not cleaned - return input object

   QplugAppendTextDisplay( " Stats filter does not handle this data type\n" );
   return QPLUG_FAILURE;

}

