/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

/**********************************************************************
*
*  QSAS Plugin to shift timetags of a time series
*
*
*  Inputs:  Time Series
*           dt          time (secs) to ADD to each timetag
*
*  Output:  Cloned Time series with shifted tags and all xrefs
*
*  History:  sjs September 2001
***********************************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"
#include "qdutil.h"
#include "DvObj.h"

using namespace QSAS;


extern "C" QplugReturnStatus DvTimeShift( vector<DvObject_var>& inputs, vector<DvObject_var>& outputs )
{

// call-list arguments

DvObject_var tsin_obj;
DvObject_var dt_obj;
DvObject_var tsout_obj;


//	..first input item is  time series

tsin_obj = inputs[0];
dt_obj   = inputs[1];

if (tsin_obj.is_nil()){
   QplugAppendTextDisplay("TimeShift> Input TS is nil\n");
   return QPLUG_FAILURE;
}
if (dt_obj.is_nil()){
   QplugAppendTextDisplay("TimeShift> Input dt is nil\n");
   return QPLUG_FAILURE;
}

// get timetags  and get ready
DvObject_var tagsin = tsin_obj->getTimeTags();
if (tagsin.is_nil()){
   QplugAppendTextDisplay("TimeShift> Input TS not a time series!\n");
   return QPLUG_FAILURE;
}


// copy input tags, object, and xrefs
    DvObject_var tagsout = new DvObject(tagsin);
    if(dt_obj->is_event()) {
        DvObject_var duration = new DvObject(0.0, dt_obj->seqSize());
        duration->change_xref(SI_CONVERSION, "1>s");
        for(size_t i=0; i<dt_obj->seqSize(); i++) duration->dbl(i) = dt_obj->event(i).duration();
        tagsout += duration;
    }
    else  {
        tagsout += dt_obj;
    }
    
tsout_obj = new DvObject(tsin_obj);
tsout_obj->change_xref(DEPEND_0, tagsout);

tsout_obj->change_xref("Modified", "Time Tags shifted");

outputs.push_back(tsout_obj);

QplugAppendTextDisplay ("TimeShift: Done\n");
  
return QPLUG_SUCCESS;

}

