/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

/**********************************************************************
*
*  QSAS Plugin to stretch timetags of a time series; also displaces
*
*
*  Inputs:  Time Series
*           Original Time Interval
*           Time Interval to Stretch To
*
*  Output:  Cloned Time series with displaced & stretched tags and all xrefs
*
*  History:  sjs September 2001
***********************************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"
#include "qdutil.h"
#include "DvObj.h"

using namespace QSAS;

extern "C" QplugReturnStatus DvTimeStretch  ( vector<DvObject_var>& inputs, vector<DvObject_var>& outputs )
{

// call-list arguments

DvObject_var tsin_obj;
DvEvent tint_in;
DvEvent tint_out;
DvObject_var tsout_obj;

QplugAppendTextDisplay("\n\n");

//	Unpack the input data objects

tsin_obj = inputs[0];
tint_in   = inputs[1]->getTimeRange();
tint_out  = inputs[2]->getTimeRange();

if (tsin_obj.is_nil() || tsin_obj->seqSize() < 2){
   QplugAppendTextDisplay("TimeStretch> Input TS is nil or not sequence\n");
   return QPLUG_FAILURE;
}

// sequence input series to tint_in

tsout_obj = tsin_obj->subSequence(tint_in);

// get time tags of subsequence (hence tsout_obj)

DvObject_var tt_in = tsout_obj->getTimeTags();
if (tt_in.is_nil()){
   QplugAppendTextDisplay("TimeShift> Input TS not a time series!\n");
   return QPLUG_FAILURE;
}

// create new tags and stretch from old ones
DvObject_var tt_out = new DvObject(tt_in);

double stretch_factor = tint_out.duration() / tint_in.duration();

DvTime tout_start = tint_out.start();
DvTime tin_start = tint_in.start();

for(size_t i=0; i<tt_out->seqSize(); i++){
  tt_out->time(i) = tout_start + (tt_out->time(i) - tin_start) * stretch_factor;
}

tsout_obj->change_xref(DEPEND_0, tt_out);

tsout_obj->change_xref("Original Time Interval", tint_in.getISOstring().c_str());
tsout_obj->change_xref("New Stretched Time Interval", tint_out.getISOstring().c_str());

QplugAppendTextDisplay("Old Interval = ");
QplugAppendTextDisplay(tint_in.getISOstring().c_str());QplugAppendTextDisplay("\n");
QplugAppendTextDisplay("Stretched to New Interval = ");
QplugAppendTextDisplay(tint_out.getISOstring().c_str());QplugAppendTextDisplay("\n");

outputs.push_back(tsout_obj);

QplugAppendTextDisplay ("TimeStretch: Done\n");
  
return QPLUG_SUCCESS;

}

