#ifndef INTERVAL_WIDGETS_H
#define INTERVAL_WIDGETS_H

#include "QbTimeWidgets.h"
#include "IntervalSliders.h"

class TimeIntervalWidget: public QWidget
{
	Q_OBJECT

	public:
		
		TimeIntervalWidget(QWidget* parent=NULL, int expand=0);

		~TimeIntervalWidget();

		DvTime getStart() const;
		DvTime getEnd() const;
		double getDuration() const;
		DvEvent getInterval() const;

		QString dobjPath;
		void setStart( DvTime start);
		void setEnd( DvTime end);
		void setEvent(DvEvent interval);
		void setInterval(DvTime start, DvTime end, bool setViewRangeToo=true);
		void setInterval(DvEvent interval, bool setViewRangeToo=true);
		void setInterval(DvObject_var& obj, bool setViewRangeToo=true);

		void clearEventTable();
		void setEventTable(DvObject_var et);
		DvObject_var getEventTable();
		void setAvailabilityRange(DvTime start, DvTime end);
		void setAvailabilityRange(DvEvent interval);

        void setViewRange(DvEvent tintv);
        DvEvent getViewRange() const;

        void setBarString(const QString& s)
        {
            slider->setBarString(s);
        }
        
		void resetView();

		void expand();
		void collapse();
		void showEvents(bool show);

		void setAltPalette(bool alt);

		bool hasFocus() const;
		
		DvObject_var events;
		DvEvent currentInterval;
		QLineEdit * nameFld;

		
	signals:
		void intervalUpdated();

	public slots:

		void expand_collapse();
        void expand_collapse_fields();
		
		void updateStartEnd();
		void updateStartCentre();
		void updateStartDuration();
		void updateEndCentre();
		void updateEndDuration();
		void updateCentreDuration();
		void updateSelection();
		void updateFields();
		void startFocus();
		void endFocus();
		void centreFocus();
		void durationFocus();

		void dropEvent(QDropEvent *event);
		void dragEnterEvent(QDragEnterEvent *event);

		void updateAllFields();
		void updateEvent(int);

		void saveIntervalToList();
		void appendIntervalSlot();
		void changeEventSlot();

		void saveInterval();
		void saveIntervalAs();
		void saveEventTable();
		void saveNewEventTable();
		void resetWidget();
		void resetInterval();
		void undoDelete();

		void nextEvent();
		void previousEvent();
		void deleteEvent();
		void sortEvents();
		void setEvent(const QString&);
		void setEventName(const QString&);
		
	protected:

		void updateDisplay();

		int current, previous;
		int lastEventNum;
		
		QbDateTimeWidget* fromWidget;
		QbDateTimeWidget* toWidget;
		QLabel* centreLabel;
		QbDateTimeWidget* centreWidget;
		QLabel* durationLabel;
		QbDurationWidget* durationWidget;
		QHBoxLayout* extrasLayout;
		QGroupBox * intervalBox;
		QLabel* sliderTitle;
		TimeSlider* slider;
		QLineEdit * eventNum;
		QLabel *eventLab;
		QLabel *nameLab;
		
		DvEvent deletedEvent;

		QPushButton* previousEventButton;
		QPushButton* nextEventButton;
		QPushButton* sortButton;
		QPushButton* delButton;
		QPushButton* resetButton;
		QPushButton* expandButton;
		QPushButton* saveButton;
		QPushButton *viewButton;
		QPushButton *saveEventsButton;
		
		DvTime InitialStart;
		DvTime InitialEnd;
};

class ScalarIntervalWidget: public QWidget
{
	Q_OBJECT

	public:
		ScalarIntervalWidget(QWidget* parent=NULL);

		~ScalarIntervalWidget();

		double getStart() const;
		double getEnd() const;
		double getLength() const;

		void setStart(const double start);
		void setEnd(const double end);
		void setRangeName(QString Name);
		void setInterval(double start, double end, bool setViewRangeToo=true);

//        void setViewRange(double start, double end);
        
		void setAvailabilityRange(double start, double end);

		void resetView(){}

		void expand();
		void collapse();

		bool hasFocus() const;

		void setAltPalette(bool alt);

        void setBarString(const QString& s)
        {
            slider->setBarString(s);
        }

	signals:
		void intervalUpdated();

	public slots:

		void expand_collapse();
		
		void updateStartEnd();
		void updateStartCentre();
		void updateStartLength();
		void updateEndCentre();
		void updateEndLength();
		void updateCentreLength();
		void updateSelection();
		void updateFields();
		void startFocus();
		void endFocus();
		void centreFocus();
		void lengthFocus();
		void updateAllFields();
		void saveInterval();
		void saveNewInterval();
		void dropEvent(QDropEvent *event);
		void dragEnterEvent(QDragEnterEvent *event);
		
	protected:

		void updateDisplay();

		int current, previous;
		
		QguWLinput* fromWidget;
		QguWLinput* toWidget;
		QLabel* centreLabel;
		QguWLinput* centreWidget;
		QLabel* lengthLabel;
		QguWLinput* lengthWidget;
		
		QPushButton* expandButton;
		QPushButton* saveButton;


		double currentStart;
		double currentEnd;

		ScalarIntervalSlider* slider;

		QString rangeName;
		bool altPalette;
};

#endif
