#ifndef QSAS_BROWSE_H_
#define QSAS_BROWSE_H_

#include <QObject>
#include <QListWidget>
#include <QListWidgetItem>
#include <QItemDelegate>
#include <QHeaderView>
#include <QApplication>
#include "QbTimeWidgets.h"
#include "QbTimeEditors.h"
#include "QbEditors.h"
#include "WLItem.h"
#define QBU_VERSION  "QBU V_1.3 [19 Jul7 2011]"
#define QB_DOUBLE 0
#define QB_STRING 1

class QbObjViewer;
class DataBrowseWindow;

class XrefListView;

class XrefNameEditor: public QItemDelegate
{
	Q_OBJECT
	
	public:
		XrefNameEditor(QObject *parent = 0);

		QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;

		void setEditorData(QWidget *editor, const QModelIndex &index) const;
		
		void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;

		void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const;
		
		QbObjViewer *viewer;
		
};

class XrefListItem: public QListWidgetItem
{
	public:
		XrefListItem(XrefListView* parent, const QString& text);

		virtual ~XrefListItem(){;}

	protected:
};

class XrefListView: public QListWidget
{
	public:
		XrefListView(QWidget* parent);

		virtual ~XrefListView(){}

		QbObjViewer *viewer;

	protected:

		virtual void dragEnterEvent(QDragEnterEvent* event);

		virtual void dropEvent(QDropEvent* event);
		
		virtual QMimeData* mimeData( const QList<QListWidgetItem *> items ) const ;
		
		QStringList mimeTypes() const;
		
		XrefNameEditor* delegate;
		

};

class arrDialog : public QDialog
{
    Q_OBJECT

    public slots:
        void editArray();

    public:
        arrDialog(QWidget *parent=0, const char *name=0, QbObjViewer* browser=0, bool modal=false);
        QguWLinput *dimSlot;
        QbObjViewer *parentViewer;
        int arrType;
	QString folderPath;

};

class QbObjViewer : public QGroupBox {

    Q_OBJECT

    public slots:
		void xrefEdit ( QListWidgetItem *item );
        void newXref();
        void addTimeTags();
        void attachXref();
        void deleteXref();
        void refreshList();
        void createText();
        void createTextArr();
        void createTime();
        void createTInterval();
        void createArray();
        void createVConstant();
        void createConstant();
        void openParentFile();
        void renameAttribute();
		void copyXrefToWL(const QString &attrName);

    signals:
        void launchQDA_Window(const char *file);
 
    public:
        QbObjViewer( DataBrowseWindow * pWindow, DvObject_var dobj, QWidget *parent=0, QString name=0);
        DvObject_var getUpdate();
        const char * objName();
		XrefListView* XrefList;
        QbTimeIntervalEditor* TI_window;
        QbEventTableEditor* TIS_window;
        QbEditor* activeEditor;
        QDialog *objTypeSelector;
        arrDialog *arrTypeSelector;
        QPushButton *renameBtn;
        QMenu *fileList;
        DataBrowseWindow *parentBrowser;
        QWidget *Attribute_box;
    
    protected:
  
};


class DataBrowseWindow : public QMainWindow {

    Q_OBJECT

            QMenuBar *_menu_bar;
    QStatusBar *msgBar;

    public slots:
        void saveEdits();
        void incRec();
        void decRec();
        void lastRec();
        void firstRec();
        void inputRec();
        void adjustRec(int);
        void launchVersion();
        void launchManBrowser();
        void launchManual();
        void dismiss();
        void extract();
        void clearErrors();

    signals:
        void objChanged(const QString&);
  
    public:
    
        DataBrowseWindow(DvObject_var &dobj, QString WLcontext, QWidget *parent=0, const char *name=0 );
				   
        ~DataBrowseWindow();
        void setObject(DvObject_var &dataObject, QString WLcontext);
        void allowSave(bool allow);
    	void emitObjChanged(const QString& name){emit objChanged( name);}
        QbObjViewer * browser;
        QString editObjectName;
        QString originalName;
        DvObject_var parentObject;
        DataBrowseWindow* parentBrowser;
		QWidget *browserVB;
        QGroupBox *buttons_groupbox;
        QGroupBox *recSlider;
        QguWLinput * recField;
        QSlider *recBar;
        QGroupBox *name_panel;
        QguWLinput *name_field;
        QPushButton *saveEditsBtn;
        QPushButton *dismissBtn;
        QPushButton *extractBtn;
		QLabel *pad;
        QPushButton *parentsBtn;
        QGroupBox *error_panel;
        QPushButton *clearBtn;
        QVBoxLayout * browserVBLayout;
		
        bool disallowSaves;
        QMenu * editMenu;
		WLItem* item;

    protected:
        void keyPressEvent(QKeyEvent * e );
        virtual void dragEnterEvent(QDragEnterEvent *event);
};


class QbuAttachWindow : public QDialog
{

    Q_OBJECT

    public:
        QbuAttachWindow(QbObjViewer *parent=0, const char *name=0);
        DvObject_var dobj;
        QguWLinput *name_field;
        QguWLinput *object_field;
        QbObjViewer * thisViewer;
  
    public slots:
        void updateName();
        void attachObject();
   
};
   


 DataBrowseWindow * QbuLaunchBrowse( DvObject_var &source, 
                                     QWidget *parent, const char * objectName, 
                                     DvObject_var &dobj, QString WLcontext);

# endif // QSAS_BROWSE_H_
