#include "ColourTableList.h"

void ColourTablePreview::paintEvent(QPaintEvent* e)
{
    if(regenerate)
    {
        QPainter tempPainter(&pix);
        table.drawColourBar(&tempPainter, width(), height());

        QPen pen(Qt::black);
        QBrush brush(Qt::black);
        tempPainter.setPen(pen);
        tempPainter.setBrush(brush);

        for(int i=0; i<table.size; ++i)
        {
            double h=pix.height()*(1.-table.i[i]);
            QPolygon poly;
            poly << QPoint(pix.width()-6, (int)h);
            poly << QPoint(pix.width(), (int)h+3);
            poly << QPoint(pix.width(), (int)h-3);
            tempPainter.drawPolygon(poly);

            poly.clear();
            poly << QPoint(5, (int)h);
            poly << QPoint(0, (int)h+3);
            poly << QPoint(0, (int)h-3);
            tempPainter.drawPolygon(poly);
        }

        tempPainter.drawText(pix.width()/2 - 3, 10, "1");
        tempPainter.drawText(pix.width()/2 - 3,pix.height()-2, "0");
        
        regenerate=false;
    }
    QPainter painter(this);
    painter.drawPixmap(0, 0, pix);
}

ColourTableEditor::ColourTableEditor(ColourTableList* parent, const ColourTable& table):
        QDialog(parent)
{
    QHBoxLayout* layout=new QHBoxLayout;

    QVBoxLayout* leftLayout=new QVBoxLayout;

    QGridLayout* nameRGBNumPointsLayout=new QGridLayout;

    nameRGBNumPointsLayout->addWidget(new QLabel("Name:"), 0, 0);
    nameEdit=new QLineEdit;
    nameEdit->setText(table.name);
    nameRGBNumPointsLayout->addWidget(nameEdit, 0, 1, 1, 3);

    rgbCheckbox=new QCheckBox("is RGB");
    nameRGBNumPointsLayout->addWidget(rgbCheckbox, 1, 0);
    connect(rgbCheckbox, SIGNAL(stateChanged(int)), this, SLOT(setRGB(int)));

    nameRGBNumPointsLayout->addWidget(new QLabel("Key points:"),1,1);
    sizeEdit=new QSpinBox;
    sizeEdit->setMinimum(2);
    sizeEdit->setMaximum(20);
    nameRGBNumPointsLayout->addWidget(sizeEdit, 1, 2, 1, 2);
    connect(sizeEdit, SIGNAL(valueChanged(int)), this, SLOT(setSize(int)));
    
    leftLayout->addLayout(nameRGBNumPointsLayout);

    QGroupBox* pointsGroupBox=new QGroupBox("Interpolation points");

    QGridLayout* pointsLayout=new QGridLayout;

    pointsLayout->addWidget(new QLabel("Point"),0,0,Qt::AlignCenter);
    HueLabel=new QLabel("Hue (deg.)");
    pointsLayout->addWidget(HueLabel, 0, 1, Qt::AlignHCenter);
    SaturationLabel=new QLabel("Saturation");
    pointsLayout->addWidget(SaturationLabel, 0, 2, Qt::AlignHCenter);
    ValueLabel=new QLabel("Value");
    pointsLayout->addWidget(ValueLabel, 0, 3, Qt::AlignHCenter);

    valuesContainer=new QWidget;
    QVBoxLayout* containerLayout=new QVBoxLayout;

    for(int i=0; i<20; ++i)
    {
        QGridLayout* tempLayout=new QGridLayout;
        QDoubleSpinBox* pos=new QDoubleSpinBox;
        pos->setMinimum(0.);
        pos->setMaximum(1.);
        pos->setSingleStep(0.05);
        if(i==0)
        {
            pos->setEnabled(false);
            pos->setValue(0.);
        }
        if(i==19)
        {
            pos->setEnabled(false);
            pos->setValue(1.);
        }
        if(i>=table.size-1 && i!=19) pos->setVisible(false);
        tempLayout->addWidget(pos, 0, 0);
        connect(pos, SIGNAL(valueChanged(double)), this, SLOT(updateView()));
        Positions.push_back(pos);

        QDoubleSpinBox* h=new QDoubleSpinBox;
        h->setMinimum(-1080.);
        h->setMaximum(1080.);
        h->setSingleStep(1.);
        tempLayout->addWidget(h, 0, 1);
        if(i>=table.size-1 && i!=19) h->setVisible(false);
        connect(h, SIGNAL(valueChanged(double)), this, SLOT(updateView()));
        Hues.push_back(h);

        QDoubleSpinBox* s=new QDoubleSpinBox;
        s->setMinimum(0.);
        s->setMaximum(1.);
        s->setSingleStep(0.05);
        tempLayout->addWidget(s, 0, 2);
        if(i>=table.size-1 && i!=19) s->setVisible(false);
        connect(s, SIGNAL(valueChanged(double)), this, SLOT(updateView()));
        Saturations.push_back(s);

        QDoubleSpinBox* v=new QDoubleSpinBox;
        v->setMinimum(0.);
        v->setMaximum(1.);
        v->setSingleStep(0.05);
        tempLayout->addWidget(v, 0, 3);
        if(i>=table.size-1 && i!=19) v->setVisible(false);
        connect(v, SIGNAL(valueChanged(double)), this, SLOT(updateView()));
        Values.push_back(v);

        containerLayout->addLayout(tempLayout);
    }
    
    valuesContainer->setLayout(containerLayout);

    QScrollArea * scroll=new QScrollArea;
    scroll->setWidget(valuesContainer);
    pointsLayout->addWidget(scroll, 1, 0, 5, 4);

    pointsGroupBox->setLayout(pointsLayout);
    
    leftLayout->addWidget(pointsGroupBox);

    QHBoxLayout* buttonsLayout=new QHBoxLayout;

    QPushButton * OKButton=new QPushButton("OK");
    connect(OKButton, SIGNAL(clicked()), this, SLOT(accept()));
    buttonsLayout->addWidget(OKButton);
    QPushButton * CancelButton=new QPushButton("Cancel");
    connect(CancelButton, SIGNAL(clicked()), this, SLOT(reject()));
    buttonsLayout->addWidget(CancelButton);
    
    leftLayout->addLayout(buttonsLayout);
    
    layout->addLayout(leftLayout);
    
    QVBoxLayout* rightLayout=new QVBoxLayout;

    previewWidget=new ColourTablePreview;
    rightLayout->addWidget(previewWidget);
    
    layout->addLayout(rightLayout);
    
    setLayout(layout);

    // Now update the fields accordingly to the table passed as an argument
    setRGB(table.isRGB);
    rgbCheckbox->setChecked(table.isRGB);

    for(int i=0; i<table.size-1; ++i)
    {
        Positions[i]->setValue(table.i[i]);
        Hues[i]->setValue(table.h[i]);
        Saturations[i]->setValue(table.s[i]);
        Values[i]->setValue(table.v[i]);
//         std::cout << "setValue " << table.h[i] << std::endl;
    }
    Positions[19]->setValue(table.i[table.size-1]);
    Hues[19]->setValue(table.h[table.size-1]);
    Saturations[19]->setValue(table.s[table.size-1]);
    Values[19]->setValue(table.v[table.size-1]);
    
    sizeEdit->setValue(table.size);
    setSize(table.size);
}

void ColourTableEditor::setSize(int s)
{
    for(int i=0; i<19; ++i)
    {
        Positions[i]->setVisible(i<s-1);
        Hues[i]->setVisible(i<s-1);
        Saturations[i]->setVisible(i<s-1);
        Values[i]->setVisible(i<s-1);
    }
    valuesContainer->resize(valuesContainer->minimumSizeHint());
    updateView();
}

extern void c_plhlsrgb(PLFLT h, PLFLT l, PLFLT s, PLFLT *p_r, PLFLT *p_g, PLFLT *p_b); // in plctrl.c
extern void c_plrgbhls(PLFLT r, PLFLT g, PLFLT b, PLFLT *p_h, PLFLT *p_l, PLFLT *p_s);


void ColourTableEditor::setRGB(int i)
{
    if(i==0)
    {
        HueLabel->setText("Hue (deg.)");
        SaturationLabel->setText("Saturation");
        ValueLabel->setText("Value");

        // conversion of the colours from RGB to HSV

        PLFLT H, S, V, R, G, B;
        
        for(int j=0; j<20; ++j)
        {
            R=Hues[j]->value();
            G=Saturations[j]->value();
            B=Values[j]->value();
            c_plrgbhls(R, G, B, &H, &V, &S);
            Hues[j]->setMinimum(-1080.);
            Hues[j]->setMaximum(1080.);
            Hues[j]->setSingleStep(1.);
            Hues[j]->setValue(H);
            Saturations[j]->setValue(S);
            Values[j]->setValue(V);
        }
    }
    else
    {
        HueLabel->setText("Red");
        SaturationLabel->setText("Green");
        ValueLabel->setText("Blue");
        
        PLFLT H, S, V, R, G, B;
        
        for(int j=0; j<20; ++j)
        {
            H=Hues[j]->value();
            S=Saturations[j]->value();
            V=Values[j]->value();
            c_plhlsrgb(H, V, S, &R, &G, &B);
            Hues[j]->setMinimum(0.);
            Hues[j]->setMaximum(1.);
            Hues[j]->setSingleStep(0.05);
            Hues[j]->setValue(R);
            Saturations[j]->setValue(G);
            Values[j]->setValue(B);
        }
    }
}

void ColourTableEditor::updateColourTable(ColourTable& table)
{
    int size=sizeEdit->value();

    table.clear();
    
    table.name=nameEdit->text();
    table.size=size;
    table.isRGB=rgbCheckbox->isChecked();
//     if(table.i!=NULL) delete[] table.i;
//     if(table.h!=NULL) delete[] table.h;
//     if(table.s!=NULL) delete[] table.s;
//     if(table.v!=NULL) delete[] table.v;

    table.i=new double[size];
    table.h=new double[size];
    table.s=new double[size];
    table.v=new double[size];

    QVector<double> pos(size);
    QVector<double> H(size);
    QVector<double> S(size);
    QVector<double> V(size);
    
    for(int i=0; i<size-1; ++i)
    {
        pos[i]=Positions[i]->value();
        H[i]=Hues[i]->value();
        S[i]=Saturations[i]->value();
        V[i]=Values[i]->value();
    }
    pos[size-1]=Positions[19]->value();
    H[size-1]=Hues[19]->value();
    S[size-1]=Saturations[19]->value();
    V[size-1]=Values[19]->value();

    sortAccordingToPositions(pos, H, S, V);

    for(int i=0; i<size; ++i)
    {
        table.i[i]=pos[i];
        table.h[i]=H[i];
        table.s[i]=S[i];
        table.v[i]=V[i];
    }
    
    table.updateIcon();
}

void ColourTableEditor::sortAccordingToPositions(QVector<double>& pos, QVector<double>& H, QVector<double>& S, QVector<double>& V)
{
    int size=pos.size();
    if(H.size()!=size || S.size()!=size || V.size()!=size)
    {
        return;
    }

    QVector<double> oldPos=pos;
    QVector<double> oldH=H;
    QVector<double> oldS=S;
    QVector<double> oldV=V;

//     QVector<int> indexes(size, -1);
    double value;
    int index;
    for(int i=0; i<size; ++i)
    {
        // find lowest value in oldPos
        value=5.;
        for(int j=0; j<size; ++j)
        {
            if(value>=oldPos[j])
            {
                value=oldPos[j];
                index=j;
            }
        }
        pos[i]=oldPos[index];
        H[i]=oldH[index];
        S[i]=oldS[index];
        V[i]=oldV[index];
        oldPos[index]=10.; // so that it is not selected in the next pass
    }
}

void ColourTableEditor::updateView()
{
    updateColourTable(previewWidget->table);

    previewWidget->regenerate=true;
    
    previewWidget->update();
}

// ColourTableItem::ColourTableItem(ColourTableList* parent, const ColourTable& table):
//     QListWidgetItem(table.name, parent)
// {
//     this->table=table;
//     setIcon(table.icon);
// }




ColourTableList::ColourTableList(QWidget* parent):
    QListWidget(parent)
{
    palette.clear();
}

void ColourTableList::getColourTables(ColourTablePalette& palette) const
{
//     palette.clear();
//     for(int i=0; i<count(); ++i)
//     {
//         palette.push_back(((ColourTableItem*)item(i))->table);
//     }
    //TODO
    palette.clear();
    foreach(ColourTable table, this->palette)
    {
        palette.push_back(table);
    }
}

void ColourTableList::setColourTables(const ColourTablePalette& palette)
{
    clear();
    this->palette.clear();

//     int i=0;
    foreach(ColourTable table, palette)
    {
//         std::cout << "COMPARE" << std::endl;
        this->palette.push_back(table);
        QListWidgetItem* item=new QListWidgetItem(table.name, this);
        item->setIcon(table.icon);
        
//         std::cout << table.toString().toLatin1().data() << std::endl;
//         std::cout << ((ColourTableItem*)item(i))->table.toString().toLatin1().data() << std::endl;
//         std::cout << "DONE" << std::endl;
//         i++;
    }

//     std::cout << "replop " <<((ColourTableItem*)item(0))->table.toString().toLatin1().data() << std::endl;
}

void ColourTableList::startEditor(const QModelIndex& index)
{
    if(index.isValid())
    {
//         ColourTableItem * _item=((ColourTableItem*)item(index.row()));
// // std::cout << "plop " << _item->table.toString().toLatin1().data() << std::endl;
//         if(_item!=NULL)
//         {
//             ColourTableEditor editor(this, _item->table);
//             editor.exec();
//         }
        //TODO
        ColourTableEditor editor(this, palette[index.row()]);
        editor.exec();
        if(editor.result()==QDialog::Accepted)
        {
//             palette[index.row()]=editor.getColourTable();
            editor.updateColourTable(palette[index.row()]); //TODO check overflows
//             std::cout << "after " << palette[index.row()].toString().toLatin1().data() << std::endl;
            clear();
            int i=0;
            foreach(ColourTable table, palette)
            {
                QListWidgetItem* item=new QListWidgetItem(table.name, this);
                item->setIcon(table.icon);
            }
        }
    }
}

void ColourTableList::editSelected()
{
    startEditor(currentIndex());
}


void ColourTableList::deleteSelected()
{
    if(currentIndex().isValid())
    {
        int row=currentIndex().row();

        if(row>=0)
        {
            // Not the most efficient thing to do, but at least it doesn't crash...
//             ColourTablePalette palette;
//             getColourTables(palette);
//             palette.remove(row);
//             setColourTables(palette);
            palette.remove(row);
            clear();
            int i=0;
            foreach(ColourTable table, palette)
            {
                QListWidgetItem* item=new QListWidgetItem(table.name, this);
                item->setIcon(table.icon);
            }
            
        }
    }
}

void ColourTableList::add()
{
//     ColourTable ct("New");
//     new ColourTableItem(this, ct);
    ColourTable newCT("newCT");
    palette.push_back(newCT);
    clear();
    int i=0;
    foreach(ColourTable table, palette)
    {
        QListWidgetItem* item=new QListWidgetItem(table.name, this);
        item->setIcon(table.icon);
    }
}
