#ifndef WL_LIST_H
#define WL_LIST_H

#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QStyle>
#include <QLinkedList>
#include <QRadioButton>
#include <QCheckBox>
#include <QItemDelegate>
#include "qbu.h"

#include "qpsd.h"

#include "WLItem.h"


class WList : public QTreeWidget
{
	Q_OBJECT


	protected:
		void mousePressEvent(QMouseEvent* event);
		
	public:
		WList(QWidget* parent);
		~WList();

		WLItem* _data;
		WLItem* _time;
		WLItem* _constants;
		WLItem* _temp;
		QMenu * RButtonMenu;		
		QDialog* objTypeSelector;
		QDialog* sortTypeSelector;
		arrDialog *arrTypeSelector;
		
		string sortOn;
		bool sortReverse;		
		void setNoSelection();
		

		void deleteItem(WLItem* item);
		
		QStringList mimeTypes() const;
		QMimeData* mimeData(const QList<QTreeWidgetItem *> items ) const;
		void dropEvent(QDropEvent *event);
//		bool dropMimeData(QTreeWidgetItem *target, int index, const QMimeData *data, Qt::DropAction action);
		Qt::DropActions supportedDropActions() const;
			
		WLItem* createItem(const QString& path);
		void collapseConst();
		
		void displayMenu(const QPoint& p);

		inline WLItem* itemFromPath(const QString& path) const
		{
			if(path == "Data") return _data;
			if(path == "Constants") return _constants;
			if(path == "Time Intervals") return _time;
			if(path == "Temp") return _temp;
			
			QString _path = path;
			if(_path.startsWith("Data/")) {
				_path.remove(0,5);
				return _data->getChildByPath(_path);
			}
			if(_path.startsWith("Time Intervals/")) {
				_path.remove(0,15);
				return _time->getChildByPath(_path);
			}
			if(_path.startsWith("Constants/")) {
				_path.remove(0,10);
				return _constants->getChildByPath(_path);
			}
			if(_path.startsWith("Temp/")) {
				_path.remove(0,5);
				return _temp->getChildByPath(_path);
			}
			return NULL;
			
		}
		
		inline bool notEditable(WLItem*item)
		{
			if(item==_data || item==_constants || item==_time || item==NULL) return true;
			return false;

		}
				
		void setUpConnections(WLItem* item);
		

		void emitPathChanged(const QString& oldPath, const QString& newPath){emit pathChanged(oldPath, newPath);}
		void emitDeleted(const QString& path) {emit deleted(path);}
		void emitObjChanged(const QString& path) {emit objChanged(path);}

		
	signals:
		
		void pathChanged(const QString& oldPath, const QString& newPath);
		void deleted(const QString& path);
		void objChanged(const QString& path);
		
		void xrefDropDetected(const QString& attrName);
		void refreshBrowser();
		void exportET();
		void raiseWindow();
		
	public slots:
		
		void dragEnterEvent(QDragEnterEvent *event);
		void dragLeaveEvent(QDragLeaveEvent *event);
		void dragMoveEvent(QDragMoveEvent *event){;}
		
		void openObjectEditor(QTreeWidgetItem *qitem, int column);
		void updateObjectName(QTreeWidgetItem *item, int column);
		void rename();
		void editObject();
		void newObject();
		void newFolder();
		
		void moveUp();
		void moveDown();
		void sortWLSelection(QList <QTreeWidgetItem*> &itemList, string &sortOn, bool reverse=false);
		void sortWL();
		void sortWLSelected();
		void sortWLPrefs();
		void setSortWLname(bool on);
		void setSortWLtime(bool on);
		void setSortWLtype(bool on);
		void setSortRev(bool on);

		void createText();
		void createTextArr();
		void createTime();
		void createTInterval();
		void createConstant();
		void createVConstant();
		void createArray();
		void sliceObj();

		void duplicateObj();
		void tif2ev();
		void ev2tif();
		void deleteObject();
		void removeTimeTags();
		void selectAllData();
		void copy();
        
		void deleteAllDataItems()
		{
			while(_data->childCount() > 0){
				WLItem *item = (WLItem*) _data->child(0);
				_data->removeChild(item);
				delete item;
			}
		}
		
		void deleteAllTempItems()
		{
			while(_temp->childCount() > 0){
				WLItem *item = (WLItem*) _temp->child(0);
				_temp->removeChild(item);
				delete item;
			}
		}
		
		
		void deleteAllTimeIntervalItems()
		{
			while(_time->childCount() > 0){
				WLItem *item = (WLItem*) _time->child(0);
				_time->removeChild(item);
				delete item;
			}
		}
		
		void deleteAllConstantItems()
		{
			while(_constants->childCount() > 0){
				WLItem *item = (WLItem*) _constants->child(0);
				_constants->removeChild(item);
				delete item;
			}
		}
		
};



#endif
