//  qexui.cc
//
// Data Export Widget
//
// Nov 2001: Tony Allen
// Jan 2014: Tony Allen : migrate to DVOS
//


#include "qdutil.h"
#include "qwui.h"
#include "qexui.h"


#define  COMMAdelim 0
#define  SPACEdelim 1

#define  NEWLINEdelim 0
#define  DOLLARdelim 1
#define  HASHdelim 2
#define  TABdelim 3
 

extern  qwuiWindow* qsasWindow;

DataExportWindow::DataExportWindow( QWidget *parent, const char *name ) 
	: QFileDialog(parent)
{
	this->setPalette(qApp->palette());
    
    this->setAcceptMode(QFileDialog::AcceptSave);
    
    this->setOption(QFileDialog::DontUseNativeDialog, true);

    QStringList ext;
	ext.append("Cluster Exchange Format for CAA (*.cef)");
	ext.append("Common Data Format (*.cdf)");
	ext.append("ASCII Tabular (*.qft)");
	ext.append("ASCII Delimited (*.qfd)");
	setNameFilters(ext);
	
	_OverWrite = new QCheckBox("Overwrite file", this);
	_OverWrite->setChecked(false);
	_OverWrite->setMaximumHeight(18);
	
	_Compress = new QCheckBox("Compress CDF file", this);
	_Compress->setChecked(true);
	_Compress->setMaximumHeight(18);
	
	accuracy = new QSpinBox(this);
	accuracy->setRange(0,9);
	accuracy->setToolTip("Accuracy after decimal place in seconds, SS.sss\n In CDF, Epoch 16 is used for accuracy > 3 decimal places");
	if(QSAS_profiles->current() != NULL) accuracy->setValue(QSAS_profiles->current()->secDecimal);
	else accuracy->setValue(3);
	
	options_groupbox = new QGroupBox( "ASCII File Settings", this);
	QGridLayout* options_layout = new QGridLayout( options_groupbox);
	options_layout->setSpacing(4);
	options_layout->setColumnStretch(1,10);
	
	QLabel* DataDelim_label = new QLabel("Data Delimiter: ");
	options_layout->addWidget( DataDelim_label, 0, 0, Qt::AlignRight );
	_DataDelim = new QComboBox(options_groupbox);
	_DataDelim->setEditable(false);
	_DataDelim->insertItem(COMMAdelim, "Comma");
	_DataDelim->insertItem(SPACEdelim, "Space");
	_DataDelim->insertItem(TABdelim, "Tab");
	options_layout->addWidget( _DataDelim, 0, 1, Qt::AlignLeft );
	
	QLabel* RecDelim_label = new QLabel("Record Delimiter: ",options_groupbox);
	options_layout->addWidget( RecDelim_label, 1, 0, Qt::AlignRight );

	_RecDelim = new QComboBox(options_groupbox);
	_RecDelim->setEditable(false);
	_RecDelim->insertItem(DOLLARdelim, "Dollar");
	_RecDelim->insertItem(NEWLINEdelim, "Newline");
	_RecDelim->insertItem(HASHdelim, "Hash");
	_RecDelim->insertItem(TABdelim, "Tab");
	options_layout->addWidget( _RecDelim, 1, 1, Qt::AlignLeft );

	_AttachHead = new QCheckBox( "Attached Header", options_groupbox);
	_AttachHead->setChecked(true);
	options_layout->addWidget( _AttachHead, 3, 0, Qt::AlignLeft );
	
	QLabel *accuracyLab = new  QLabel("Time decimal:");
	options_layout->addWidget(accuracyLab, 5, 0);
	options_layout->setAlignment(accuracyLab, Qt::AlignRight);
	options_layout->addWidget(accuracy, 5, 1);
    options_layout->setAlignment(accuracy, Qt::AlignLeft);
	
	accuracyDbl = new QSpinBox(this);
	accuracyDbl->setRange(2,18);
	accuracyDbl->setToolTip("Maximum number of significant digits for doubles");
	accuracyDbl->setValue(8);
    connect(accuracyDbl, SIGNAL(valueChanged(int)), this, SLOT(setPrecision(int)));
	
	QLabel *accuracyLabDbl = new  QLabel("Max float precision:");
	options_layout->addWidget(accuracyLabDbl, 6, 0);
	options_layout->setAlignment(accuracyLab, Qt::AlignRight);
	options_layout->addWidget(accuracyDbl, 6, 1);
    options_layout->setAlignment(accuracyDbl, Qt::AlignLeft);
    
    this->layout()->addWidget(_OverWrite);

	this->layout()->addWidget(_Compress); 

	cdfBackward = new QCheckBox("write CDF v2.7", this);
	cdfBackward->setChecked(false);
	cdfBackward->setMaximumHeight(18);	
	this->layout()->addWidget(cdfBackward);
	
	this->layout()->addWidget(new QWidget());//blank
	this->layout()->addWidget(options_groupbox);
	  
    this->setMinimumSize(this->sizeHint() * 1.25);
	
	startDir = QDir::home();

	connect(this, SIGNAL(filterSelected(const QString &)), this, SLOT(setFilter(const QString &)));
    connect(this, SIGNAL(directoryEntered(const QString &)), this, SLOT(setDir()));
	
}

// ------------------------------------------

void DataExportWindow::setFilter(const QString & filter){
	if(filter.contains("cef", Qt::CaseInsensitive) ){
		_RecDelim->setCurrentIndex(0);
	}

}

// ------------------------------------------

void DataExportWindow::setPrecision(int prec){
	DvString tmp;
	tmp.setDblPrecision(prec); // this sets the global precision in DvString
	QieDataCont->precIsSet = true;
}


// ------------------------------------------

void DataExportWindow::setDir(){
	startDir = directory();
}

// ------------------------------------------

void DataExportWindow::writeFile(){

	if(QuSlotDebug())
	{
		printf("DataExportWindow::writeFile \n");
		fflush(stdout);
	}
 
	// get extension from filter
	QString ext = selectedNameFilter();
	int posn = ext.lastIndexOf(")");
	ext.truncate(posn);
	posn = ext.indexOf(".");
	ext.remove(0,posn);
  
	QieDataCont->outFile.extn = ext.toLatin1().data();
   
	QString name_text;
	QStringList names =  selectedFiles();
	if(!names.isEmpty()) name_text=names[0];
  
	QieDataCont->outFile.name = QexNewName(name_text);
    
	// overwrite file toggle
	bool replace = false;
	bool compress = false;
	
	if(_OverWrite->isChecked() )replace = true;
	if(_Compress->isChecked() ) compress = true;
  
	QieDataCont->outFile.attr_delim = ',';      // not accessible via Qexui

	// data delimiter
	if ( _DataDelim->currentIndex() == COMMAdelim )
		QieDataCont->outFile.data_delim = ',';
	else if ( _DataDelim->currentIndex() == SPACEdelim )
		QieDataCont->outFile.data_delim = ' ';
	else if ( _DataDelim->currentIndex() == TABdelim )
		QieDataCont->outFile.data_delim = '\t';
	else   {
		QieDataCont->outFile.data_delim = ' '; // set something, but not used
	}

	// record delimiter
	if ( _RecDelim->currentIndex() == DOLLARdelim )
		QieDataCont->outFile.end_of_line = '$';
	else if ( _RecDelim->currentIndex() == NEWLINEdelim )
		QieDataCont->outFile.end_of_line = '\n';
	else if ( _RecDelim->currentIndex() == HASHdelim )
		QieDataCont->outFile.end_of_line = '#';
	else if ( _RecDelim->currentIndex() == TABdelim )
		QieDataCont->outFile.end_of_line = '\t';
	else   {
		QieDataCont->outFile.end_of_line = '\n'; // set something, but not used
	}
    
	// attached header
	bool detached = true;
	if( _AttachHead->isChecked() ) detached = false;
	
	// cdf backward flag
	QieDataCont->outFile.CDFbackward = false;
	if(cdfBackward->isChecked()) QieDataCont->outFile.CDFbackward = true;
  
	//  Call appropriate QiWrite routine for selected output file type.
	bool writeOK = false;
	
  	if ( QieDataCont->outFile.extn == ".cdf" ) {
    	writeOK = QieDataCont->writeCDF(replace, compress);     
  	}
  	else{
        if( QieDataCont->notHomogeneous()){
            (void)QMessageBox::warning(this,"Write File","Cannot write file type with different length variables. Use CDF.");
            return;
        }
        else{
            writeOK = QieDataCont->writeFlat(replace, detached);    
        }
	}  
    
	if (writeOK){
    	QString txt = "File written to ";
    	txt += QString(QieDataCont->outFile.name.c_str());
    	txt += QString(QieDataCont->outFile.extn.c_str());
    	emit QexMessage(txt);
	}
	else{ 
		QString txt = "File write failed, does it exist already?";
    	emit QexMessage(txt);
	}
  
}

//-----------------------------------

void DataExportWindow::initName(){
	selectFile(QString("QSAS_data"));
		
	setDirectory(startDir);
}

//-----------------------------------

void DataExportWindow::buildExportQIEObj (DvJoinList &ObjList){ 

	if(ObjList.size() == 0) return;
	
	DvNode *node = ObjList.first();

	QieDataCont->nRecords = 0;
	while(node){
    	if ( node->obj().is_nil() ) {
			node = node->next;
			continue;  // skip empty returns
		}
		if(node->obj()->seqSize() > QieDataCont->nRecords) QieDataCont->nRecords = node->obj()->seqSize();

 		// add joined objects to export file-object 
		QieDataCont->addObj(node->name().afterR('/'), node->obj(), accuracy->value());
				
    	node = node->next;
	}
	
	return;
}


//-----------------------------------

void QexWriteFile( QWidget *parent, DvJoinList &ObjList )
{
	static DataExportWindow *QEX_widget=NULL; // created first call
	QSize qsasGUISize = qsasWindow->frameSize();
	if(QEX_widget == NULL) QEX_widget = new DataExportWindow(parent,"Data_Export_Window");
	
	QEX_widget->QieDataCont = new QieData(); // fresh version every call to avoid confusion
	QEX_widget->setWindowTitle("Write File");
	QEX_widget->move(qsasGUISize.width(),10);
	QEX_widget->setFileMode(QFileDialog::AnyFile);
  
	QEX_widget->initName(); 
	
	
	QEX_widget->buildExportQIEObj(ObjList);

	QObject::connect(QEX_widget, SIGNAL(QexMessage(QString )), parent, SLOT(messageHandler(QString)));
	if( QEX_widget->exec() == QDialog::Accepted){
    	QEX_widget->writeFile();
	}
	qApp->processEvents();
	delete QEX_widget->QieDataCont;

}


// ---------------------------

DvString QexNewName(QString text){
	DvString name;
  
	QString working = QString(text.toLatin1().data());
  
	// remove extension if present
	int posn = working.lastIndexOf(".cef",-1, Qt::CaseInsensitive);
	if (posn == -1) posn = working.lastIndexOf(".cdf",-1, Qt::CaseInsensitive);
	if (posn == -1) posn = working.lastIndexOf(".qft",-1, Qt::CaseInsensitive);
	if (posn == -1) posn = working.lastIndexOf(".qfd",-1, Qt::CaseInsensitive);
	if (posn != -1) {
    	working.truncate(posn);  
	}

	name = working.toLatin1().data();
  
	return name;
}


