//  qsurface.h
//
//  Tony Allen
//  Oct 2010

#ifndef QSAS_3DVIEW_UI_H_
#define QSAS_3DVIEW_UI_H_

#include <qsplitter.h>
#include <qstatusbar.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qtabbar.h>
#include <qlayout.h>

#include <QApplication>
#include <QMimeData>
#include <QStyle>
#include <QHeaderView>
#include <QPainter>
#include <QScrollArea>

#include "qt_version.h"
#include "qslot.h"
#include "QbTimeEditors.h"
#include "IntervalDialog.h"

#include "floatspinbox.h"
#include "IntervalWidgets.h"
#include "ColourTable.h"
#include "qwui.h"

#include "qpui.h"
#include "PlotWindow.h"
#include "qsurface.h" // for threevector and triangle

#define Q3D_VERSION "1.0"


class polygon;


// class  Q3dWindow
// -----------------------

class Q3dWindow : public QMainWindow
{
	Q_OBJECT
 
 	// GUI widgets
	QTabWidget *tab_widget;
	QStatusBar* _status_bar;
	QTextEdit *textArea;
	
	// Data tab widgets
 	QguWLinput *dataSlot;
	RecordSlider *atSlider;
	bool autoPlot;
	QSlider *elevation;
	QSlider *azimuth;
	QSpinBox *delay;
	
	// plot tab widgets
	QLineEdit *titleSlot;
	QLineEdit *RXLabSlot;
	QLineEdit *ThYLabSlot;
	QLineEdit *PhiZLabSlot;
	QComboBox *plotType;
	QComboBox *axisType;
	FloatSpinBox *alphaSpin;
	QLabel * ribbonLab;
	
	QLabel* shape_label;
	QComboBox *symbol_shape;
	QLabel* size_label;
	FloatSpinBox *symbol_size;
	FloatSpinBox *beta;

	QCheckBox *RXLog;
	QCheckBox *RXAuto;
	QLineEdit *RXMin;
	QLineEdit *RXMax;
	QCheckBox *YLog;
	QCheckBox *ThYAuto;
	QLineEdit *ThYMin;
	QLineEdit *ThYMax;
	QCheckBox *ZLog;
	QCheckBox *PhiZAuto;
	QLineEdit *PhiZMin;
	QLineEdit *PhiZMax;
	QLabel * RXMinLab;
	QLabel * RXMaxLab;
	QLabel * ThYMinLab;
	QLabel * ThYMaxLab;
	QLabel * PhiZMinLab;
	QLabel * PhiZMaxLab;
	QLabel * RaxisLab;
	QLabel * ThAxisLab;
    QLabel * PhiAxisLab;
    FloatSpinBox *axis_width_fld;
	
	QCheckBox *colourLog;
	QCheckBox *colourAuto;
	QLineEdit *colMin;
	QLineEdit *colMax;
	ColourTableButton *colourTable;
	QLineEdit *colLabSlot;
	FloatSpinBox *title_size_fld;
	FloatSpinBox *text_size_fld;
	
	// page tab widgets
	QComboBox *page_size;
	QComboBox *pageUnits;
	int oldUnits;
	QComboBox *dpiPage;
	QLineEdit *pageWidth;
	QLineEdit *pageHeight;
	QComboBox *pageOrientation;
	QCheckBox *newWindow;
    PlotID *plotWin_id;
	// flags and pointers
	bool isPolar;
	bool userTitle;
	bool userRLab;
	bool userThLab;
	bool userPhiLab;
	bool userColLab;
	bool fullTh; //indicates whether bins span over 360 degree
	bool fullPhi; //indicates whether bins span over 360 degree
	bool zLogEmpty;
	bool isArcfill;
	bool escPushed;
	bool validData;
	int RXindex;
	bool phiFirst;
	DvObject_var dobj;
	DvObject_var rDep;
	DvObject_var thetaDep;
	DvObject_var phiDep;
	PLFLT *z;
	PLFLT *zLog;
	PLFLT *theta_plus;
	PLFLT *theta_minus;
	PLFLT *phi_plus;
	PLFLT *phi_minus;
	PLFLT *r_plus;
	PLFLT *r_minus;
	PLFLT *r;
	PLFLT *theta;
	PLFLT *phi;
	PLFLT width;
	PLFLT height;

    PlotWindow* win;
	int DPI;
	int oldDPI;
	double conv;
	bool oldLandscape;
	string timeText;

	// from ppp
	int nr; //length of R/X array
	int ntheta; //length of theta/Y array
	int nphi; //length of phi/Z array
	int ndata; //total no. of data points = nr*ntheta*nphi
	int nr_start; // starting radial/x index (> userRmin)
	int nr_end; // end radial/x index (< userRmax)
	int nth_start; // starting y index (> userThmin)
	int nth_end; // end y index (< userThmax
	int nphi_start; // starting z index (> userPhimin)
	int nphi_end; // end z index (< userPhimax)
	double contStep;
	double ang2radTh;
	double rad2degTh;
	double ang2radPhi;
	double rad2degPhi;
	double rfillVal;
	double thetafillVal;
	double phifillVal;
	double zfillVal;
	PLFLT zmin;
	PLFLT zmax;
	PLFLT rmax;
	PLFLT rmin;
	PLFLT rminNonZero;
	PLFLT thetaminNonZero;
	PLFLT phiminNonZero;
	PLFLT userRmax;
	PLFLT userRmin;
	PLFLT userThmax;
	PLFLT userThmin;
	PLFLT userPhimax;
	PLFLT userPhimin;
	PLFLT rOffset;
	PLFLT thetamin;
	PLFLT thetamax;
	PLFLT phimin;
	PLFLT phimax;
	PLFLT room; //how much larger (factor) than the maximum radius of the plot should the 3d box be?
    PLFLT alpha;	//Alpha value for the background when mesh plot is chosen.
	PLFLT az;
    PLFLT el;
	PLcGrid2 cgrid;
    QComboBox *setRX;

	// --------
	
	public:
		Q3dWindow( QWidget *parent=0);
		~Q3dWindow();
        void saveAsXML(QString path);
        void restoreXML(QString path, bool replace=true);
		void setwindow(bool forceSameWindow=false);
		void setData();
		bool initData();
		void setTitle();
		inline PLFLT min(PLFLT x, PLFLT y) { return (x < y ? x : y); }
		inline PLFLT max(PLFLT x, PLFLT y) { return (x > y ? x : y); }
		inline PLFLT SaferLog10(PLFLT x) { return (x < 1.0e-35) ? -35.0 : log10(x); }
		PLFLT near_nice_number(PLFLT x);
		PLFLT nice_min(PLFLT zmin, PLFLT stepsize);
		int colscale();
		void drawcolscale();
		void select_vpor(int n);
		bool getDimensions(int &nx, int &ny, int &nz);
		void Q3dviewAppendTextDisplay(const char * txt);
		void clearTextArea();
		void setColLab();
		void setAxisLabels();
		void nonZeroMin();
		void setDeltaAngle(PLFLT *result, const char *delta, PLFLT *angle, int nAngle, DvObject_var &angleDep, double fillVal, double ang2rad);
		void setDeltaRad(PLFLT *result, const char *delta);
		QDomElement toXMLElement(QDomDocument&) const;
		void fromXMLElement(QDomElement&);
		void setGUItoPolar(bool isPolar);
		void doPlotPolar();
		void doPlotCartesian();
		void cartesian_scatterplot();
		void polar_scatterplot();
		void drawtheta(bool firstdraw);
		void drawaxis();
		void drawphi();
		void polar_wedgeplot();
		PLFLT get_mappeddepth(threevector *tv);
        QString _filename;        ///< Filename used to save the current session or working list content
        QString SAVE_EXTENSION;   ///< File extension used to save the current session or working list content

		
	public slots:
		void status(const QString& str) {_status_bar->showMessage(str,2000);};
		void doPlot(bool forceSameWindow=false);
		void doRun();
		void doAnimate();
        void saveAsXML();
        void restoreXML();
		void setDataObj();
		void setRecord(bool andPlot=true);
		void launchManual();
		void launch3dviewPlotHelp();
		void launchVersion();
		void autosetCol();
		void autosetRX();
		void autosetThY();
		void autosetPhiZ();
		void setUserTitle();
		void setUserRXLab();
		void setUserThYLab();
		void setUserPhiZLab();
		void setColUser();
		void setLogRX();
		void setLogY();
		void setLogZ();
		void setLogCol();
		void setAuto();
		void changeRX();
		void changeThY();
		void changePhiZ();
		void setUserColLab();
		void setPage();
		void changeView();
		void setAxisOpt();
		void setPlotType(int);
		void stepAnimate();
		void stopAnimate();
        void saveSession(QString path){saveAsXML(path+"/saveSet.qx_3dview_plot");}
        void restoreSession(QString path){restoreXML(path+"/saveSet.qx_3dview_plot");}
		
	signals:
		void reValidateData();

	protected:
		void keyPressEvent(QKeyEvent * e );
		QPushButton *plotBtn;
		QPushButton *animateBtn;
		QPushButton *runBtn;
        QPushButton *dismissBtn;
        int startElevation;
        int startAzimuth;
        int startSlider;
};



void Q3dMakeWindow(qwuiWindow*, const char*);
void Q3dviewShow();
void MinMax3dgrid(PLFLT data[], int nx, int ny, int nz, PLFLT &min, PLFLT &max, double fillVal);
void quicksortthreevector(vector <threevector*> &vect, int start, int end, PLFLT el, PLFLT az);
void quicksortthreevector(vector <threevector*> &vect, vector <threevector*> &v2, vector <threevector*> &v3, int start, int end, PLFLT el, PLFLT az);

PLFLT arrminNonZero(PLFLT a[], int size);


# endif // QSAS_3DVIEW_UI_H_
