// qpolar_window.cc
//


#include "qpolar.h"
#include "qwui.h"
#include "qdutil.h"


PLINT excludedRegion(PLFLT x, PLFLT y);
PLFLT global_rmax;

extern qwuiWindow* qsasWindow;
QpolarWindow *QPolar_widget=static_cast<QpolarWindow*>(NULL);


void QPolarMakeWindow( QWidget *parent,const char* Title )
{
	if(QPolar_widget==NULL)
	{
		QPolar_widget = new QpolarWindow(parent);
		QSize qsasGUISize = qsasWindow->frameSize();
		QPolar_widget->adjustSize();  
		QPolar_widget->move((int)(qsasGUISize.width()), 25);

		QPolar_widget->connect(parent, SIGNAL(saveSessionRequest(QString)), QPolar_widget, SLOT(saveSession(QString)) );
		QPolar_widget->connect(parent, SIGNAL(restoreSessionRequest(QString)), QPolar_widget, SLOT(restoreSession(QString)) );

		QPolar_widget->setWindowTitle(Title);
		QPolar_widget->_filename+=QString("/saveSet");
		QPolar_widget->SAVE_EXTENSION=QString(".qx_polar_plot");
		QPolar_widget->_filename+=QPolar_widget->SAVE_EXTENSION;
		
	}

}

void QpolarShow()
{
    QPolar_widget->show();
}


//     QpolarWindow CLASS
//     ----------------

QpolarWindow::QpolarWindow( QWidget *parent ):QMainWindow(0)
{
	// Constructor
	if(QuSlotDebug())
	{
		printf("QpolarWindow::QpolarWindow \n");
		fflush(stdout);
	}
	
	// initialise defaults as they may get used before setPage() is called
	DPI = 72;
	conv = (double) DPI;
	
	// set up colour group palettes
	QPalette QPolar_normal_palette = QPalette(qApp->palette());
	this->setPalette(QPolar_normal_palette);

	setAcceptDrops(true);
	setFocusPolicy(Qt::WheelFocus);

	QSplitter *_top_splitter=new QSplitter(Qt::Vertical, this);
	setCentralWidget( _top_splitter);

	tab_widget = new QTabWidget( _top_splitter);
	int margins = QSAS_profiles->current()->GUImargins;
	tab_widget->setContentsMargins(margins, margins, margins, margins);

	// data tab
	// ---------
	
	QWidget *dataTab=new QWidget();
	QGridLayout *dataTabLayout=new QGridLayout(dataTab);
	dataTabLayout->setSpacing(QSAS_profiles->current()->GUImargins);
	
	dataTabLayout->setMargin(QSAS_profiles->current()->GUImargins);	
	dataTabLayout->setColumnStretch(0, 1);
	dataTabLayout->setColumnStretch(1, 3);
	dataTabLayout->setColumnStretch(2, 3);
	dataTabLayout->setColumnStretch(3, 3);
	dataTabLayout->setColumnStretch(4, 3);
	dataTabLayout->setColumnStretch(5, 10);
	dataTabLayout->setColumnStretch(6, 10);
	
	// data input
	QLabel * dataLabel = new QLabel("2D data: ");
	dataTabLayout->addWidget(dataLabel, 0, 0, Qt::AlignRight);

	dataSlot = new QguWLinput(dataTab, "data");
	dataSlot->setMinimumWidth(400);
	dataSlot->setToolTip("Accepts a 2D array or sequence of 2D arrays - use pulldown to subsample");
	dataSlot->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_2D, QS_ANY, QS_ANY));
	dataSlot->set3DSubsetDef("rp");
	connect(dataSlot, SIGNAL(edited_OK()), this, SLOT(setDataObj()) );
	dataTabLayout->addWidget(dataSlot, 0, 1, 1, 6);
	

	//time selection
	atSlider = new  RecordSlider("At ", dataTab, "atSlider");
	atSlider->setToolTip("Select time (or record) at which to plot if a sequence of 2D records");
	dataTabLayout->addWidget(atSlider, 1, 0, 1, 7);
	connect(atSlider, SIGNAL(recordSliderChanged()), this, SLOT(setRecord()) );

	autoPlot = new QCheckBox("Auto-replot");
	autoPlot->setToolTip("Automatically replot when time slider is changed");
	dataTabLayout->addWidget(autoPlot, 2, 5, Qt::AlignLeft);

	swapR = new QCheckBox("Swap R-theta");
	swapR->setToolTip("Force other dimension to be used for radial data (useful when metadata is missing and plot guesses wrongly)");
	dataTabLayout->addWidget(swapR, 2, 6, Qt::AlignRight);
	connect(swapR, SIGNAL(toggled(bool)), this, SLOT(setSwapRTheta()) );

	QLabel* delayLabel = new QLabel("Animation delay: ");
	dataTabLayout->addWidget(delayLabel, 2, 0, Qt::AlignRight);

	delay = new QSpinBox();
	delay->setMinimum(0);
	delay->setMaximum(500);
	delay->setValue(0);
	delay->setSingleStep(10);
	delay->setToolTip("Delay in msec between plots in animations");
	dataTabLayout->addWidget(delay, 2, 1, Qt::AlignLeft);

	dataTab->adjustSize();
	tab_widget->addTab(dataTab, "Data");
	
	// Options Tab
	// ----------
	QWidget *optionsTab=new QWidget();
	QGridLayout *optionsTabLayout=new QGridLayout(optionsTab);
	optionsTabLayout->setSpacing(QSAS_profiles->current()->GUImargins);
	optionsTabLayout->setMargin(QSAS_profiles->current()->GUImargins);	
	
	QLabel * titleLab = new QLabel("Plot Title: ");
	optionsTabLayout->addWidget(titleLab, 0, 0, Qt::AlignRight);
	
	titleSlot = new QLineEdit();
	titleSlot->setMinimumWidth(200);
	connect(titleSlot, SIGNAL(textEdited(const QString& )), this, SLOT(setUserTitle()));
	optionsTabLayout->addWidget(titleSlot, 0, 1, 1, 6);

	autoTitle = new QCheckBox("Auto-Title");
	optionsTabLayout->addWidget(autoTitle, 0, 7, Qt::AlignLeft);
	autoTitle->setChecked(true);
	connect(autoTitle, SIGNAL(toggled(bool)), this, SLOT(setTitle()) );

	QLabel * RaxisLab = new QLabel("R axis Text: ");
	optionsTabLayout->addWidget(RaxisLab, 1, 3, Qt::AlignRight);
	
	RLabSlot = new QLineEdit();
	RLabSlot->setMinimumWidth(160);
	connect(RLabSlot, SIGNAL(textEdited(const QString& )), this, SLOT(setUserRLab()));
	optionsTabLayout->addWidget(RLabSlot, 1, 4, 1, 1);

	QLabel * ThAxisLab = new QLabel("Angle axis Text: ");
	optionsTabLayout->addWidget(ThAxisLab, 1, 6, Qt::AlignRight);
	
	ThLabSlot = new QLineEdit();
	ThLabSlot->setMinimumWidth(160);
	connect(ThLabSlot, SIGNAL(textEdited(const QString& )), this, SLOT(setUserThLab()));
	optionsTabLayout->addWidget(ThLabSlot, 1, 7, 1, 1);

	QLabel * typeLab = new QLabel("Plot Type: ");
	optionsTabLayout->addWidget(typeLab, 1, 0, Qt::AlignRight);
	plotType = new QComboBox();
	plotType->insertItem(0, "Shaded");
	plotType->insertItem(1, "Arc-Filled");
	plotType->setCurrentIndex(1);
	plotType->setToolTip("Arc-Filled: uses blocks of colour on segments of arc.\nShaded: uses smoothed colours.");
	optionsTabLayout->addWidget(plotType, 1, 1, Qt::AlignLeft);
	
	QLabel * axisLab = new QLabel("Axis Type: ");
	optionsTabLayout->addWidget(axisLab, 2, 0, Qt::AlignRight);
	axisType = new QComboBox();
	axisType->insertItem(0, "x=0, y=0 only");
	axisType->insertItem(1, "Axis at bottom");
	axisType->insertItem(2, "Box");
	axisType->setToolTip("Type and position of axes");
	optionsTabLayout->addWidget(axisType, 2, 1, Qt::AlignLeft);

    QLabel* axis_width_label=new QLabel("Axis width: ");
    optionsTabLayout->addWidget(axis_width_label,2,3,Qt::AlignRight);
    axis_width_fld = new FloatSpinBox(0.1,20.,1,0.1, NULL, "axis_width");
    axis_width_fld->setValue(QSAS_profiles->current()->LineWidth);
    axis_width_fld->setFixedWidth(50);
    optionsTabLayout->addWidget(axis_width_fld,2,4,Qt::AlignLeft);
    

	QLabel * rScaleLab = new QLabel("Axis extent: ");
	optionsTabLayout->addWidget(rScaleLab, 2, 6, Qt::AlignRight);
	rScale = new FloatSpinBox(1.0, 1.4, 2, 0.01, NULL, "r scale");
	rScale->setValue(1.3);
	rScale->setToolTip("Length of axis scaled to r max");
	optionsTabLayout->addWidget(rScale, 2, 7, Qt::AlignLeft);
	
	QLabel * dataPtsLab = new QLabel("Data Points: ");
	optionsTabLayout->addWidget(dataPtsLab, 3, 0, Qt::AlignRight);

	dataPtOpts = new QComboBox();
	dataPtOpts->insertItem(0, "No Data Points");
	dataPtOpts->insertItem(1, "Show Data Points");
	dataPtOpts->insertItem(2, "Points with values");
	dataPtOpts->setToolTip("Shows the actual input data point locations in r-theta");
	optionsTabLayout->addWidget(dataPtOpts, 3, 1, Qt::AlignLeft);

	QLabel * spaceLab = new QLabel("   ");
	optionsTabLayout->addWidget(spaceLab, 3, 2);
	
	QLabel * fractDataLab = new QLabel("Label % of points: ");
	optionsTabLayout->addWidget(fractDataLab, 3, 3, Qt::AlignRight);
	fractData = new QSpinBox();
	fractData->setMinimum(1);
	fractData->setMaximum(100);
	fractData->setValue(100);
	fractData->setToolTip("Label this percentage of data points (random selection).");
	optionsTabLayout->addWidget(fractData, 3, 4, Qt::AlignLeft);


	QLabel * spaceLab1 = new QLabel("   ");
	optionsTabLayout->addWidget(spaceLab1, 3, 5);
	
	QLabel * dataOriginLab = new QLabel("Label above val: ");
	optionsTabLayout->addWidget(dataOriginLab, 3, 6, Qt::AlignRight);
	dataOrigin = new QLineEdit();
	dataOrigin->setMinimumWidth(30);
	dataOrigin->setMaximumWidth(100);
	dataOrigin->setToolTip("Label values above this numerical threshold (or set 'Label All').");
	dataOrigin->setText("Label All");
	optionsTabLayout->addWidget(dataOrigin, 3, 7, Qt::AlignLeft);


	showCont = new QCheckBox("Add Contours");
	optionsTabLayout->addWidget(showCont, 4, 0, Qt::AlignLeft);

    QLabel* line_width_label=new QLabel("Contour width: ");
    optionsTabLayout->addWidget(line_width_label,4,1,Qt::AlignRight);
    line_width_fld = new FloatSpinBox(0.1,20.,1,0.1, NULL, "line_width");
    line_width_fld->setValue(QSAS_profiles->current()->LineWidth);
    line_width_fld->setFixedWidth(50);
    optionsTabLayout->addWidget(line_width_fld,4,2,Qt::AlignLeft);

	QLabel * contNumLab = new QLabel("Contour interval: ");
	optionsTabLayout->addWidget(contNumLab, 4, 3, Qt::AlignRight);
	contSpace=new QLineEdit();
	contSpace->setText("Default");
	contSpace->setToolTip("contour interval as a numeric value, (increment in power of 10 if log colour scale)");
	optionsTabLayout->addWidget(contSpace, 4, 4, Qt::AlignLeft);
	
			
	QLabel * contLabLab = new QLabel("Contour Label: ");
	optionsTabLayout->addWidget(contLabLab, 4, 6, Qt::AlignRight);
	contLabelType = new QComboBox();
	contLabelType->insertItem(0, "No Contour Label");
	contLabelType->insertItem(1, "Label Contours");
	contLabelType->insertItem(2, "Label Colourscale");
	contLabelType->insertItem(3, "Label Both");
	optionsTabLayout->addWidget(contLabelType, 4, 7, Qt::AlignLeft);
	
	radiusLog = new QCheckBox("Log Radius");
	radiusLog->setChecked(true);
	optionsTabLayout->addWidget(radiusLog, 5, 0, Qt::AlignLeft);
	connect(radiusLog, SIGNAL(toggled(bool)), this, SLOT(setLogRad()) );

	radAuto = new QCheckBox("Auto radius");
	radAuto->setChecked(true);
	connect(radAuto, SIGNAL(toggled(bool)), this, SLOT(setAutoRad()));
	optionsTabLayout->addWidget(radAuto, 5, 1, Qt::AlignRight);

	QLabel * RadMinLab = new QLabel("Min Radius: ");
	optionsTabLayout->addWidget(RadMinLab, 5, 3, Qt::AlignRight);
	RadMin = new QLineEdit();
	RadMin->setMinimumWidth(30);
	RadMin->setMaximumWidth(100);
	RadMin->setToolTip("Minimum radial value (log(0) not allowed)");
	connect(RadMin, SIGNAL(textEdited(const QString& )), this, SLOT(changeRadius()));
	optionsTabLayout->addWidget(RadMin, 5, 4, Qt::AlignLeft);
	
	QLabel * RmaxLab = new QLabel("Max Radius: ");
	optionsTabLayout->addWidget(RmaxLab, 5, 6, Qt::AlignRight);
	RadMax = new QLineEdit();
	RadMax->setMinimumWidth(30);
	RadMax->setMaximumWidth(100);
	RadMax->setToolTip("Maximum radial value ");
	connect(RadMax, SIGNAL(textEdited(const QString& )), this, SLOT(changeRadius()));
	optionsTabLayout->addWidget(RadMax, 5, 7, Qt::AlignLeft);
	
	colourLog = new QCheckBox("Log ColourScale");
	colourLog->setChecked(true);
	connect(colourLog, SIGNAL(toggled(bool)), this, SLOT(setLogCol()));
	optionsTabLayout->addWidget(colourLog, 6, 0, Qt::AlignLeft);

	colourAuto = new QCheckBox("Auto ColourScale");
	colourAuto->setChecked(true);
	connect(colourAuto, SIGNAL(toggled(bool)), this, SLOT(autosetCol()));
	optionsTabLayout->addWidget(colourAuto, 6, 1, Qt::AlignRight);

	QLabel * colMinLab = new QLabel("Min value: ");
	optionsTabLayout->addWidget(colMinLab, 6, 3, Qt::AlignRight);
	colMin = new QLineEdit();
	colMin->setMinimumWidth(50);
	colMin->setMaximumWidth(100);
	colMin->setText("0.0");
	colMin->setToolTip("Minimum data value (log(0) not allowed)");
	connect(colMin, SIGNAL(textEdited(const QString & )), this, SLOT(setColUser()));
	optionsTabLayout->addWidget(colMin, 6, 4, Qt::AlignLeft);

	QLabel * colMaxLab = new QLabel("Max value: ");
	optionsTabLayout->addWidget(colMaxLab, 6, 6, Qt::AlignRight);
	colMax = new QLineEdit();
	colMax->setMinimumWidth(50);
	colMax->setMaximumWidth(100);
	colMax->setToolTip("Maximum data value (log(0) not allowed)");
	colMax->setText("1.0e6");
	connect(colMax, SIGNAL(textEdited(const QString & )), this, SLOT(setColUser()));
	optionsTabLayout->addWidget(colMax, 6, 7, Qt::AlignLeft);

	QLabel* colour_table_label = new QLabel("Colour Table: ");
	optionsTabLayout->addWidget(colour_table_label, 7, 0, Qt::AlignRight);
    colourTable = new ColourTableButton;
    optionsTabLayout->addWidget(colourTable, 7, 1, Qt::AlignLeft);

	QLabel * colLabLab = new QLabel("Colour bar label: ");
	optionsTabLayout->addWidget(colLabLab, 7, 3, Qt::AlignRight);
	
	colLabSlot = new QLineEdit();
	colLabSlot->setMinimumWidth(200);
	connect(colLabSlot, SIGNAL(textEdited(const QString& )), this, SLOT(setUserColLab()));
	optionsTabLayout->addWidget(colLabSlot, 7, 4, 1, 3);

	QLabel* title_size_label = new QLabel("Title font size: ");
	optionsTabLayout->addWidget(title_size_label, 8, 0, Qt::AlignRight);
	title_size_fld = new FloatSpinBox(0.01, 40.,1, .1, NULL, "title_char_size");
	title_size_fld->setFixedWidth(50);
	optionsTabLayout->addWidget(title_size_fld, 8, 1, Qt::AlignLeft);
	title_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+0.5);

	QLabel* text_size_label = new QLabel("Text font size: ");
	optionsTabLayout->addWidget(text_size_label, 8, 3, Qt::AlignRight);
	text_size_fld = new FloatSpinBox(0.01, 40.,1, .1, NULL, "text_char_size");
	text_size_fld->setFixedWidth(50);
	optionsTabLayout->addWidget(text_size_fld, 8, 4, Qt::AlignLeft);
	text_size_fld->setValue(QSAS_profiles->current()->BaseFontSize);

	useLocalTime = new QCheckBox("Phi as local time");
	connect(useLocalTime, SIGNAL(toggled(bool)), this, SLOT(setLocalTime()));
	optionsTabLayout->addWidget(useLocalTime, 8, 6, Qt::AlignLeft);

	tab_widget->addTab(optionsTab, "Plot Options");
	
	// Page Tab
	// ----------
	QWidget *pageTab=new QWidget();

	QVBoxLayout *page_properties_top_layout=new QVBoxLayout(pageTab);
	page_properties_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_properties_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QGroupBox *page_size_groupbox = new QGroupBox(pageTab);
	page_size_groupbox->setTitle("Size");

	QGridLayout *page_size_layout = new QGridLayout;
	page_size_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_size_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* page_size_label = new QLabel("Page size: ");
	page_size_layout->addWidget(page_size_label, 1, 0, Qt::AlignRight);
	page_size = new QComboBox();
	page_size->insertItem(QPUI_A4, "A4");
	page_size->insertItem(QPUI_CUSTOM, "Custom");
	page_size->insertItem(QPUI_EXECUTIVE, "Executive");
	page_size->insertItem(QPUI_LEGAL, "Legal");
	page_size->insertItem(QPUI_LETTER, "Letter");
	page_size->insertItem(QPUI_DEFAULT, "Default");

	QFontMetrics page_size_fm(page_size->font());
	int page_size_w = page_size_fm.width("Executive");
	int page_size_h = page_size_fm.height();
	page_size->setMinimumSize(page_size_w, page_size_h);

	page_size->setToolTip("Select page size");
	page_size_layout->addWidget(page_size, 1, 1, Qt::AlignLeft);
	
	if(QSAS_profiles->current()->PageSize=="A4")
		page_size->setCurrentIndex(QPUI_A4);
	else if(QSAS_profiles->current()->PageSize=="Executive")
		page_size->setCurrentIndex(QPUI_EXECUTIVE);
	else if(QSAS_profiles->current()->PageSize=="Legal")
		page_size->setCurrentIndex(QPUI_LEGAL);
	else if(QSAS_profiles->current()->PageSize=="Letter")
		page_size->setCurrentIndex(QPUI_LETTER);
	else if(QSAS_profiles->current()->PageSize=="Default")
		page_size->setCurrentIndex(QPUI_DEFAULT);
	else page_size->setCurrentIndex(QPUI_CUSTOM);
	
	QLabel* units_label = new QLabel("Units: ");
	page_size_layout->addWidget(units_label, 1, 2, Qt::AlignRight);
	pageUnits = new QComboBox();
	pageUnits->insertItem(QPUI_PIXELS, "pixels");
	pageUnits->insertItem(QPUI_INCHES, "inches");
	pageUnits->insertItem(QPUI_CM, "cm");
	pageUnits->insertItem(QPUI_MM, "mm");
	pageUnits->insertItem(QPUI_PSPOINTS, "PS points");
	QFontMetrics units_fm(pageUnits->font());
	int units_w = units_fm.width("PS points");
	int units_h = page_size_fm.height();
	pageUnits->setMinimumSize(units_w, units_h);

	pageUnits->setToolTip("Select units for page size");
	page_size_layout->addWidget(pageUnits, 1, 3, Qt::AlignLeft);

	
	oldUnits = pageUnits->currentIndex();
	
	QLabel* dpi_label = new QLabel("Resolution: ");
	page_size_layout->addWidget(dpi_label, 1, 4, Qt::AlignRight);
	dpiPage = new QComboBox();
	dpiPage->insertItem(QPUI_72_DPI, "72 dpi");
	dpiPage->insertItem(QPUI_150_DPI, "150 dpi");
	dpiPage->insertItem(QPUI_300_DPI, "300 dpi");
	dpiPage->insertItem(QPUI_600_DPI, "600 dpi");
	dpiPage->insertItem(QPUI_1200_DPI, "1200 dpi");
	switch(QSAS_profiles->current()->PageDPI)
	{
		case 72: default:
			dpiPage->setCurrentIndex(QPUI_72_DPI);
			break;
		case 150:
			dpiPage->setCurrentIndex(QPUI_150_DPI);
			break;
		case 300:
			dpiPage->setCurrentIndex(QPUI_300_DPI);
			break;
		case 600:
			dpiPage->setCurrentIndex(QPUI_600_DPI);
			break;
		case 1200:
			dpiPage->setCurrentIndex(QPUI_1200_DPI);
			break;
	}

	page_size_layout->addWidget(dpiPage, 1, 5, Qt::AlignLeft);
	
	QLabel* width_label = new QLabel("Width: ");
	page_size_layout->addWidget(width_label, 3, 0, Qt::AlignRight);
	pageWidth = new QLineEdit;
	pageWidth->setFixedWidth(100);
	pageWidth->setValidator(new QDoubleValidator(pageWidth));

	page_size_layout->addWidget(pageWidth, 3, 1, Qt::AlignLeft);

	QLabel* height_label = new QLabel("Height: ");
	page_size_layout->addWidget(height_label, 3, 2, Qt::AlignRight);
	pageHeight = new QLineEdit;
	pageHeight->setFixedWidth(100);
	pageHeight->setValidator(new QDoubleValidator(pageHeight));
	page_size_layout->addWidget(pageHeight, 3, 3, Qt::AlignLeft);
	
	
	// set appropriate text in width and height fields
	if(QSAS_profiles->current()->PageSize=="Custom")
	{
		width = QSAS_profiles->current()->PageWidth;
		height = QSAS_profiles->current()->PageHeight;
		pageWidth->setText(QString().setNum(width));
		pageHeight->setText(QString().setNum(height));
	}

	QLabel* orientation_label = new QLabel("Orientation: ");
	page_size_layout->addWidget(orientation_label, 5, 0, Qt::AlignRight);
	pageOrientation = new QComboBox();
	pageOrientation->insertItem(QPUI_PORTRAIT, "Portrait");
	pageOrientation->insertItem(QPUI_LANDSCAPE, "Landscape");
	QFontMetrics orientation_fm(pageOrientation->font());
	int orientation_w = orientation_fm.width("Landscape");
	int orientation_h = orientation_fm.height();
	pageOrientation->setMinimumSize(orientation_w, orientation_h);

	pageOrientation->setToolTip("Select portrait or landscape");
	page_size_layout->addWidget(pageOrientation, 5, 1, Qt::AlignLeft);
	
	if(QSAS_profiles->current()->PageOrientation=="Landscape")
		pageOrientation->setCurrentIndex(QPUI_LANDSCAPE);
	else pageOrientation->setCurrentIndex(QPUI_PORTRAIT);


	
	page_size_groupbox->setLayout(page_size_layout);

	page_properties_top_layout->addWidget(page_size_groupbox);

	QGroupBox *page_output_groupbox = new QGroupBox(this);

	page_output_groupbox->setTitle("Output");

	QGridLayout *page_output_layout = new QGridLayout;
	page_output_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_output_layout->setMargin(QSAS_profiles->current()->GUImargins);


	newWindow = new QCheckBox("New window for each plot");
	page_output_layout->addWidget(newWindow,0,0,Qt::AlignLeft);


    plotWin_id = new PlotID("Polar Plot");
    page_output_layout->addWidget(plotWin_id, 0, 3, Qt::AlignLeft);
   
	page_output_groupbox->setLayout(page_output_layout);

	page_properties_top_layout->addWidget(page_output_groupbox);


	tab_widget->addTab(pageTab, "Page");

	tab_widget->adjustSize();

	// feedback text area
	textArea = new QTextEdit(_top_splitter);
	textArea->setReadOnly(true);


	// set up menu bar
	QMenu* file_menu = 	menuBar()->addMenu("&File");
	menuBar()->addSeparator();
	QMenu* help_menu = 	menuBar()->addMenu("&Help" );


	file_menu->clear();
    file_menu->addAction("Save As XML...", this, SLOT(saveAsXML()));
	file_menu->addSeparator();
    file_menu->addAction("Restore XML...", this, SLOT(restoreXML()));

	help_menu->clear();
	help_menu->addAction("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help_menu->addSeparator();
	help_menu->addAction("Polar Plot Help", this, SLOT( launchPolarPlotHelp() ));
	help_menu->addSeparator();
	help_menu->addAction("Module Version", this, SLOT( launchVersion() ));

	_status_bar = statusBar();
	_status_bar->showMessage(" ");

	QWidget * buttonsWidget = new QWidget(_top_splitter);
	QHBoxLayout *buttons_layout = new QHBoxLayout(buttonsWidget);

	plotBtn = new QPushButton("Plot",this);
	plotBtn->setMinimumSize(plotBtn->sizeHint());
	plotBtn->setFocusPolicy(Qt::StrongFocus);
	connect(plotBtn, SIGNAL(clicked()), this, SLOT(doPlot()));
	buttons_layout->addWidget(plotBtn);
	plotBtn->setDefault(true);

	runBtn = new QPushButton("Animate Time",this);
	runBtn->setMinimumSize(runBtn->sizeHint());
	runBtn->setFocusPolicy(Qt::StrongFocus);
	connect(runBtn, SIGNAL(clicked()), this, SLOT(doRun()));
	buttons_layout->addWidget(runBtn);

	dismissBtn = new QPushButton( "Dismiss", this);
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	dismissBtn->setFocusPolicy(Qt::StrongFocus);
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	buttons_layout->addWidget(dismissBtn);
	
	this->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));


	_top_splitter->setStretchFactor(0, 10);
	_top_splitter->setStretchFactor(1, 10);
	_top_splitter->setStretchFactor(1, 1);
	
	adjustSize();
	userThLab = false;
	userRLab = false;
	userColLab = false;
	
	// ensure pointers are NULL for trap on free()
	z = 0;
	zLog = 0;
	nr = 0;
	ntheta = 0;
	cgrid.nx = 0;
	cgrid.ny = 0;
	r = 0;
	r_plus = 0;
	r_minus = 0;
	theta = 0;
	theta_plus = 0;
	theta_minus = 0;
	
	connect(pageUnits, SIGNAL(currentIndexChanged(int)), this, SLOT(setPage()));
	connect(dpiPage, SIGNAL(currentIndexChanged(int)), this, SLOT(setPage()));
	connect(page_size, SIGNAL(currentIndexChanged(int)), this, SLOT(setPage()));
	setPage();
	
}

// ------------------------

QpolarWindow::~QpolarWindow()
{
	if (z != 0) plFree2dGrid(z, nr, ntheta+1); // z is always r,theta even if dobj is theta,r
		
	if(nr != 0 && ntheta != 0) {
		plFree2dGrid(cgrid.xg, nr, ntheta+1);
		plFree2dGrid(cgrid.yg, nr, ntheta+1);
	}
	
	if (r != 0) delete[] r;
	if (r_plus != 0) delete[] r_plus;
	if (r_minus != 0) delete[] r_minus;
	if (theta != 0) delete[] theta;
	if (theta_plus != 0) delete[] theta_plus;
	if (theta_minus != 0) delete[] theta_minus;

}

// ------------------------

void QpolarWindow::setLocalTime()
{
    ThLabSlot->setText("");
    setAxisLabels();
}
// ------------------------

void QpolarWindow::setDataObj()
{
	if( !initData() ) {
		QpolarAppendTextDisplay("Data object needs to be at least 2D with Depend_1 and Depend_2");
		return;
	}
	setData();
	setTitle();
	this->autosetCol();
	this->autosetRad();
}


// ------------------------

void QpolarWindow::setRecord()
{
	setData();
	setTitle();
	this->autosetCol();
	this->autosetRad();
	if( autoPlot->isChecked()) doPlot();
}


//--------------

void QpolarWindow::setSwapRTheta()
{
	if( !initData() ) {
		QpolarAppendTextDisplay("Data object needs to be at least 2D with Depend_1 and Depend_2");
		return;
	}
	setData();
	autosetCol();
	autosetRad();
	setTitle();
	
}

// ------------------------

void QpolarWindow::autosetRad()
{

	if(dobj.is_nil()) return;
	
	if(radAuto->isChecked())
	{

		RadMax->setText(QString("%1").arg(rmax));
		if(radiusLog->isChecked()) RadMin->setText(QString("%1").arg(rminNonZero));
		else RadMin->setText(QString("%1").arg(rmin));
	}
}
// ------------------------

void QpolarWindow::autosetCol()
{
	if(dobj.is_nil()) return;
			
	if(colourAuto->isChecked())
	{
		// set min/max
		//plMinMax2dGrid(z, nr, ntheta, &zmax, &zmin);
		if(colourLog->isChecked()){
			nonZeroMinMax();
		}
		else getMinMax();
		
		colMax->setText(QString("%1").arg(zmax));
		colMin->setText(QString("%1").arg(zmin));
	}
}

// ------------------------

void QpolarWindow::nonZeroMinMax()
{
	// used for log colour scale
	if(z == 0) return;
	
	zmin = z[0][0];
	zmax = z[0][0];
	if(zmin < 1.e-31)
	{
		zmin = 1.e+31; // initialise to high value
		zmax = 1.e-30; // initialise to safe value
	}
	
	PLFLT val;
	for(int i=0; i<nr; i++){
		for(int j=0; j<ntheta; j++){
			val = z[i][j];
			if(val < zmin && val > 1.e-30) zmin = val;
			if(val > zmax ) zmax = val;
		}
	}
	
	if(zmax - zmin < 1.e-31) zmin = 0.5*zmax;

}
// ------------------------

void QpolarWindow::getMinMax()
{
	if(z == 0) return;

	// used for linear colour scale
	zmin = z[0][0];
	zmax = z[0][0];
	PLFLT val;
	for(int i=0; i<nr; i++){
		for(int j=0; j<ntheta; j++){
			val = z[i][j];
			if(val < zmin ) zmin = val;
			if(val > zmax ) zmax = val;
		}
	}
	
	if(zmax - zmin < 1.e-31) zmax += 1.;

}
// ------------------------

void QpolarWindow::setColUser()
{
	
	colourAuto->setChecked(false);

}

// ------------------------

void QpolarWindow::setLogRad()
{
	if(rmin <= 0) radAuto->setChecked(true); // ensure negative values get reset accordingly
	
	setData();
	setTitle();
	this->autosetCol();
	this->autosetRad();
}

// ------------------------

void QpolarWindow::setLogCol()
{
	bool OK;
	double min = colMin->text().toDouble(&OK);
	if( ( !OK || min <= 1.0e-30 ) && colourLog->isChecked() ) 
	{
		colourAuto->setChecked(true); // ensure negative values get reset accordingly
	
		setData();
		setTitle();
		this->autosetCol();
	}
}

// ------------------------

void QpolarWindow::setAutoRad()
{	
	setData();
	setTitle();
	this->autosetCol();
	this->autosetRad();
}

// ------------------------

void QpolarWindow::changeRadius()
{
	
	radAuto->setChecked(false);
	setData();
	setTitle();
	this->autosetCol();

}




void QpolarWindow::setTitle()
{
	if(dobj.is_nil()) return;
	if(thetaDep.is_nil()) return;
	
	setColLab(); // also set default col label
	setAxisLabels();
	
	if(autoTitle->isChecked()) 
	{

		string title = dataSlot->text().toLatin1().data();
		int posn = title.rfind("/");
		if (posn != string::npos) title = title.substr(posn+1);
		titleSlot->setText(title.c_str());

	}
	timeText = "";
	if( atSlider->record <= atSlider->maxRecord){
		timeText = atSlider->ttags->asStr(atSlider->record);
	}
}

void QpolarWindow::setUserTitle()
{
	autoTitle->setChecked(false);
}


void QpolarWindow::setUserRLab()
{
	userRLab = true;

}


void QpolarWindow::setUserThLab()
{
    if(ThLabSlot->text().isEmpty()) userThLab = false;
    else userThLab = true;
}


void QpolarWindow::setAxisLabels()
{
	if(thetaDep.is_nil()) return;

	if(!userThLab){
		string thetalabel;
		if (thetaDep->xref_exists(LABLAXIS)) {
			thetalabel = string(thetaDep->getXrefText(LABLAXIS));
		} 
		else{
			thetalabel = string("Angle ");
		}
        
        if(useLocalTime->isChecked()){
            thetalabel += " (local time)";
        }
        else{
        
            if (thetaDep->xref_exists(UNITS)) {
                thetalabel += " (";
                thetalabel += string(thetaDep->getXrefText(UNITS));
                thetalabel += ")";
            }
            else{
                thetalabel += " (deg)";
            }
        }
        
		ThLabSlot->setText(thetalabel.c_str());

	}
	
	if(!userRLab)
	{	
		string rlabel;

		if (rDep->xref_exists("LABLAXIS")) {
			rlabel = string(rDep->getXrefText(LABLAXIS));
		}
		else rlabel = string("r");
		
		if (rDep->xref_exists(UNITS)) {
			rlabel += " (";
			rlabel += string(rDep->getXrefText(UNITS));
			rlabel += ")";
		}

		RLabSlot->setText(rlabel.c_str());

	}

	
}

void QpolarWindow::setColLab()
{
	if(dobj.is_nil()) return;
	
	if(userColLab) return;

	string zlabel;
	
	if (dobj->xref_exists(LABLAXIS)) {
		zlabel = string(dobj->getXrefText(LABLAXIS));
	}
	else zlabel = string("z");
	
	if (dobj->xref_exists("UNITS")){
		zlabel += " (";
		zlabel += string(dobj->getXrefText(UNITS));
		zlabel += ")";
	}

	colLabSlot->setText(zlabel.c_str());


}

void QpolarWindow::setUserColLab()
{
	userColLab = true;

}

void QpolarWindow::keyPressEvent(QKeyEvent * e )
{  
	if(QuSlotDebug())
	{
		printf("QpolarWindow::keyPressEvent \n");
		fflush(stdout);
	}

	// use escape key to toggle default buttons
	if(e->key() == Qt::Key_Escape){
		if(   plotBtn->isDefault()) {
			plotBtn->setDefault(false);
			dismissBtn->setDefault(true);
		}
		else if(dismissBtn->isDefault())  {
			dismissBtn->setDefault(false);
			plotBtn->setDefault(true);
		}
		return;
	}
	// use return or enter key to perform default
	if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
		if(plotBtn->isDefault()) doPlot();
		else if (dismissBtn->isDefault()) close();
		return;
	}

	// else pass on to list view 
	e->ignore(); // let selection change

}




// --------------------------------------

void QpolarWindow::launchPolarPlotHelp()
{
	// slot invoked from Plotting Help menu sub item
	// launches browser showing Qsas Plot help docuumentation

	if(QuSlotDebug())
	{
		printf("QpolarWindow::launchPolarPlotHelp \n");
		fflush(stdout);
	}

	qsasWindow->help_window->manEntry("QSAS_2D_polar.html");
}

void QpolarWindow::launchManual()
{
	// slot invoked from Qsas Manual Help menu sub item
	// launches browser showing Qsas help index

	if(QuSlotDebug())
	{
		printf("QpolarWindow::launchManual \n");
		fflush(stdout);
	}

	qsasWindow->help_window->manEntry("QSAS_help_index.html");
}

void QpolarWindow::launchVersion()
{
	// slot invoked from Module Version Help menu sub item
	// launches browser showing Qsas version numbers

	if(QuSlotDebug())
	{
		printf("QpolarWindow::launchVersion \n");
		fflush(stdout);
	}

	QString text;
	text  = QString("Version Numbers...\n");
	text += QString(QSAS_VERSION_ID);
	text += "\n Polar 2D  ";
	text += QString(QPOLAR_VERSION);
	QMessageBox::information( this, "QSAS information", text.toLatin1().data(), "OK" );
}

QDomElement QpolarWindow::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Tabs");
    
    xml.appendChild(createTextXMLNode(doc, "autoTitle", autoPlot->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "userRLab", userRLab));
    xml.appendChild(createTextXMLNode(doc, "userThLab", userThLab));
    xml.appendChild(createTextXMLNode(doc, "userColLab", userColLab));

    xml.appendChild(createTextXMLNode(doc, "userRmax", userRmax));
    xml.appendChild(createTextXMLNode(doc, "userRmin", userRmin));

    xml.appendChild(createTextXMLNode(doc, "dataSlot", dataSlot->text()));
	xml.appendChild(createTextXMLNode(doc, "atSlider", atSlider->record));
    xml.appendChild(createTextXMLNode(doc, "swapR", swapR->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "delay", delay->value()));

    xml.appendChild(createTextXMLNode(doc, "titleSlot", titleSlot->text()));
    xml.appendChild(createTextXMLNode(doc, "RLabSlot", RLabSlot->text()));
    xml.appendChild(createTextXMLNode(doc, "ThLabSlot", ThLabSlot->text()));
	
    xml.appendChild(createTextXMLNode(doc, "plotType", plotType->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "axisType", axisType->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "rScale", rScale->value()));

    xml.appendChild(createTextXMLNode(doc, "dataPtOpts", dataPtOpts->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "fractData", fractData->value()));
    xml.appendChild(createTextXMLNode(doc, "dataOrigin", dataOrigin->text()));
	
    xml.appendChild(createTextXMLNode(doc, "showCont", showCont->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "contSpace", contSpace->text()));
    xml.appendChild(createTextXMLNode(doc, "contLabelType", contLabelType->currentIndex()));
	
    xml.appendChild(createTextXMLNode(doc, "radiusLog", radiusLog->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "radAuto", radAuto->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "RadMin", RadMin->text()));
    xml.appendChild(createTextXMLNode(doc, "RadMax", RadMax->text()));

    xml.appendChild(createTextXMLNode(doc, "colourLog", colourLog->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "colourAuto", colourAuto->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "colMin", colMin->text()));
    xml.appendChild(createTextXMLNode(doc, "colMax", colMax->text()));


    ColourTable ct=colourTable->getCurrentTable();

    int _colour_table_type;
    int _colour_table_id;
    if(stdPalette.contains(ct))
    {
        _colour_table_type=0;
        _colour_table_id=stdPalette.indexOf(ct);
    }
    else
    {
        _colour_table_type=1;
        _colour_table_id=savedPalette.indexOf(ct);
    }
    xml.appendChild(createTextXMLNode(doc, "Colour_Table_Type", _colour_table_type));
    xml.appendChild(createTextXMLNode(doc, "Colour_Table_ID", _colour_table_id));


    xml.appendChild(createTextXMLNode(doc, "colLabSlot", colLabSlot->text()));

    xml.appendChild(createTextXMLNode(doc, "page_size", page_size->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "pageUnits", pageUnits->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "dpiPage", dpiPage->currentIndex()));
	
    xml.appendChild(createTextXMLNode(doc, "pageWidth", pageWidth->text()));
    xml.appendChild(createTextXMLNode(doc, "pageHeight", pageHeight->text()));
	
    xml.appendChild(createTextXMLNode(doc, "pageOrientation", pageOrientation->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "newWindow", newWindow->isChecked()));

    xml.appendChild(createTextXMLNode(doc, "autoPlot", autoPlot->isChecked()));

    return xml;
}


void QpolarWindow::fromXMLElement(QDomElement& el)
{
    autoPlot->setChecked(0); // stop plotting when restoring
    QString s;
	if(getXMLNodeText(el, "autoTitle", s))
    {
		autoTitle->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "userRLab", s))
    {
		userRLab = s.toInt();
    }
	if(getXMLNodeText(el, "userThLab", s))
    {
		userThLab = s.toInt();
    }
	if(getXMLNodeText(el, "userColLab", s))
    {
		userColLab = s.toInt();
    }
	if(getXMLNodeText(el, "userRmax", s))
    {
		userRmax = s.toDouble();
    }
	if(getXMLNodeText(el, "userRmin", s))
    {
		userRmin = s.toDouble();
    }
	if(getXMLNodeText(el, "swapR", s))
    {
        swapR->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "delay", s))
    {
        delay->setValue(s.toInt());
    }	
	if(getXMLNodeText(el, "titleSlot", s))
    {
        titleSlot->setText(s);
    }
	if(getXMLNodeText(el, "RLabSlot", s))
    {
        RLabSlot->setText(s);
    }
	if(getXMLNodeText(el, "ThLabSlot", s))
    {
        ThLabSlot->setText(s);
    }

	if(getXMLNodeText(el, "plotType", s))
    {
        plotType->setCurrentIndex(s.toInt());
    }
	if(getXMLNodeText(el, "axisType", s))
    {
        axisType->setCurrentIndex(s.toInt());
    }
	if(getXMLNodeText(el, "rScale", s))
    {
        rScale->setValue(s.toDouble());
    }

	if(getXMLNodeText(el, "dataPtOpts", s))
    {
        dataPtOpts->setCurrentIndex(s.toInt());
    }
	if(getXMLNodeText(el, "fractData", s))
    {
        fractData->setValue(s.toInt());
    }
	if(getXMLNodeText(el, "dataOrigin", s))
    {
        dataOrigin->setText(s);
    }

	if(getXMLNodeText(el, "showCont", s))
    {
        showCont->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "contSpace", s))
    {
        contSpace->setText(s);
    }
	if(getXMLNodeText(el, "contLabelType", s))
    {
        contLabelType->setCurrentIndex(s.toInt());
    }

	if(getXMLNodeText(el, "radiusLog", s))
    {
        radiusLog->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "radAuto", s))
    {
        radAuto->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "RadMin", s))
    {
        RadMin->setText(s);
    }
	if(getXMLNodeText(el, "RadMax", s))
    {
        RadMax->setText(s);
    }
	
	if(getXMLNodeText(el, "colourLog", s))
    {
        colourLog->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "colourAuto", s))
    {
        colourAuto->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "colMin", s))
    {
        colMin->setText(s);
    }
	if(getXMLNodeText(el, "colMax", s))
    {
        colMax->setText(s);
    }

    int _colour_table_type(0);
    int _colour_table(0);
    if(getXMLNodeText(el, "Colour_Table_ID", s))
    {
        _colour_table=s.toInt();
    }
    if(getXMLNodeText(el, "Colour_Table_Type", s))
    {
        _colour_table_type=(s.toInt());
    }
    if(_colour_table_type!=0)
    {
        colourTable->setSessionPalette(_colour_table+importedPaletteOffset);
    }
    else
    {
        colourTable->setStdPalette(_colour_table);
    }


 	if(getXMLNodeText(el, "colLabSlot", s))
    {
        colLabSlot->setText(s);
    }
	
	if(getXMLNodeText(el, "page_size", s))
    {
        page_size->setCurrentIndex(s.toInt());
    }
	if(getXMLNodeText(el, "pageUnits", s))
    {
        pageUnits->setCurrentIndex(s.toInt());
    }
	if(getXMLNodeText(el, "dpiPage", s))
    {
        dpiPage->setCurrentIndex(s.toInt());
    }	
		
	if(getXMLNodeText(el, "pageWidth", s))
    {
        pageWidth->setText(s);
    }
	if(getXMLNodeText(el, "pageHeight", s))
    {
        pageHeight->setText(s);
    }

		if(getXMLNodeText(el, "pageOrientation", s))
    {
        pageOrientation->setCurrentIndex(s.toInt());
    }
	if(getXMLNodeText(el, "newWindow", s))
    {
        newWindow->setChecked(s.toInt());
    }


	if(getXMLNodeText(el, "dataSlot", s))
    {
        dataSlot->setText(s);
		dataSlot->reValidate(true);
    }

	if(getXMLNodeText(el, "atSlider", s))
    {
        atSlider->adjustRec(s.toInt());
    }
    qApp->processEvents();
    
	if(getXMLNodeText(el, "autoPlot", s))
    {
        autoPlot->setChecked(s.toInt());
    }

}

void QpolarWindow::saveAsXML(QString path)
{

	ofstream file;
	file.open(path.toLatin1().data(),ios::out);

	if(file.is_open())
	{
		QDomDocument doc("QSAS_polar_plot_save");

		QDomElement root=doc.createElement("QSAS_Polar_Plot");
		doc.appendChild(root);
		string guiID;
		guiID=QSAS_VERSION_ID;

		QDomElement version=createTextXMLNode(doc, "QSAS_Version", guiID);
		root.appendChild(version);

		root.appendChild(this->toXMLElement(doc));
            
		QString xml = doc.toString();
		file << xml.toLatin1().data();
		file.close();
	}

}

void QpolarWindow::saveAsXML()
{
    QString path=QFileDialog::getSaveFileName(this, "Save as...", QString("saveSet"), "QSAS polar plot save (*.qx_polar_plot);; XML files (*.xml)");
    if(!path.isEmpty())
    {
		if(!path.contains(".qx_polar_plot") && !path.contains(".xml")){
			path += ".qx_polar_plot";
		}
		saveAsXML(path);
	} 
}

void QpolarWindow::restoreXML()
{
    QString path=QFileDialog::getOpenFileName(this, "Open...", QString(), "QSAS polar plot save (*.qx_polar_plot);; XML files (*.xml);; All files (*)");
	restoreXML(path);
}

void QpolarWindow::restoreXML(QString path, bool replace)
{
	// restore
	
	if(!path.isEmpty())
    {
        QDomDocument doc;
        QFile myFile(path);
        doc.setContent(&myFile);
        myFile.close();

        QDomElement docElem = doc.documentElement();
        if(!docElem.isNull())
        {

            string guiID="0.0.00";

            QDomElement version=docElem.firstChildElement("QSAS_Version");
            if(!version.isNull())
            {
                guiID=version.firstChild().nodeValue().toLatin1().data();
            }
            setRestoreVersion(guiID.c_str());

            QDomElement tabElement=docElem.firstChildElement("Tabs");

            if(!tabElement.isNull())
            {
                this->fromXMLElement(tabElement);

            }
			return;
        }
    }
	return;
}


// ------------------------

void QpolarWindow::QpolarAppendTextDisplay(const char *txt)
{
	textArea->append(QString(txt));
	textArea->update();
	
}


void QpolarWindow::clearTextArea()
{
	textArea->clear();
}

// ------------------------

bool QpolarWindow::getDimensions(int &nx, int &ny)
{
	if( dobj.is_nil() || dobj->nDims() != 2) {
		QpolarAppendTextDisplay("matrix not 2D");
		return false;
	}
	
	nx = dobj->Dims()[0];
	ny = dobj->Dims()[1];
	
	return true;

}
// ------------------------

bool QpolarWindow::initData()
{
	clearTextArea();

	if( dobj.is_ok()) {
		// data has been set successfully previously
		// free memory used and allocate with dimensions of new data object
		dobj = new DvObject();
		
		if (z != 0) plFree2dGrid(z, nr, ntheta); // z is always r,theta even if dobj is theta,r
		if (zLog != 0) plFree2dGrid(zLog, nr, ntheta); // z is always r,theta even if dobj is theta,r
		if(nr != 0 && ntheta != 0) {
			plFree2dGrid(cgrid.xg, nr, ntheta+1);
			plFree2dGrid(cgrid.yg, nr, ntheta+1);
		}

		z = 0;
		zLog = 0;
		nr = 0;
		ntheta = 0;
		cgrid.nx = 0;
		cgrid.ny = 0;
		
		if (r != 0) delete[] r;
		if (r_plus != 0) delete[] r_plus;
		if (r_minus != 0) delete[] r_minus;
		if (theta != 0) delete[] theta;
		if (theta_plus != 0) delete[] theta_plus;
		if (theta_minus != 0) delete[] theta_minus;
		r = 0;
		r_plus = 0;
		r_minus = 0;
		theta = 0;
		theta_plus = 0;
		theta_minus = 0;
		
		
	}
	
	dobj = dataSlot->getObject();
	
	if(dobj.is_nil()) return false;
	
	// Initialising Data
	
	// set time slider
		
	atSlider->ttags = dobj->getDep0(); // may be nil
	
	if( dobj->seqSize() == 1)
	{
		atSlider->recBar->setRange(0, 0);
		atSlider->setEnabled(false);
		atSlider->maxRecord = 0;
	}
	else{
		atSlider->recBar->setRange(0, atSlider->ttags->seqSize()-1);
		atSlider->setEnabled(true);
		atSlider->maxRecord = atSlider->ttags->seqSize()-1;
		
	}
	atSlider->recBar->setValue(0);
	
	// Obtaining r and theta from xref
	int nx, ny;
	if( !getDimensions(nx, ny) ) {
		QpolarAppendTextDisplay("Input object must be 2D matrix or sequence of 2D matrices");
		return false;
	}
	
	
	//Make sure that depend 1 and 2 exist. If they don't, add our own arbitary depend.
	if (!(dobj->xref_exists(DEPEND_1))) {
		vector <size_t> dimx(1);
		dimx[0] = nx;
		DvObject_var bin1 = new DvObject(0.0, dimx);

		for (int i = 0; i < nx; i++)
			bin1->dbl(i) = i+1;

		DvString lablaxis("(index)");
		bin1->change_xref(LABLAXIS, lablaxis);
		dobj->change_xref(DEPEND_1, bin1);

	}
	if (!(dobj->xref_exists(DEPEND_2))) {
		//Add our own depend according to the bin number.
		vector <size_t> dimy(1);
		dimy[0] = ny;
		DvObject_var bin2 = new DvObject(0.0, dimy);

		for (int i = 0; i < ny; i++)
			bin2->dbl(i) = i+1;

		DvString lablaxis("(index)");
		bin2->change_xref(LABLAXIS, lablaxis);
		dobj->change_xref(DEPEND_2, bin2);
	}

	// find which dimension is theta and get fillvals and units
	rfillVal 		= NAN;
	thetafillVal 	= NAN;
	zfillVal 		= NAN;
	
	DvObject_var aObj; // local ptr used for attrs
	
	if(dobj->xref_exists(FILL_VALUE)) 
	{
		aObj = dobj->get_xref(FILL_VALUE);
		if (aObj.is_ok() && aObj->is_dbl()) zfillVal = aObj->dbl();
	}

	// find depend that is an angle variable and set R and Th depends
	DvObject_var dep1 = dobj->get_xref(DEPEND_1);
	DvObject_var dep2 = dobj->get_xref(DEPEND_2);
	
	if(dep1.is_nil() || dep2.is_nil()) return false;
	
	orderIsRTh = true;
	
	if(dep1->isAngle()) 
	{
		orderIsRTh = false;
	}
	
	// Allow user to force other choice (useful for angle vs angle)
	if(swapR->isChecked()) 
	{
		orderIsRTh = !orderIsRTh;
	}

	if(orderIsRTh){
		rDep = dep1;
		thetaDep = dep2;
		nr = nx;
		ntheta = ny;
	}
	else{
		rDep = dep2;
		thetaDep = dep1;
		nr = ny;
		ntheta = nx;
	}
	
	// find fillval for Theta
	if(thetaDep->xref_exists(FILL_VALUE)) 
	{
		aObj = thetaDep->get_xref(FILL_VALUE);
		if (aObj.is_ok() && aObj->is_dbl()) thetafillVal = aObj->dbl();
	}
	
	// find fillval for R
	if(rDep->xref_exists(FILL_VALUE)) 
	{
		aObj = rDep->get_xref(FILL_VALUE);
		if (aObj.is_ok() && aObj->is_dbl()) rfillVal = aObj->dbl();
	}


	r = new PLFLT[nr];
	r_plus = new PLFLT[nr];
	r_minus = new PLFLT[nr];

	theta = new PLFLT[ntheta];
	theta_plus = new PLFLT[ntheta];
	theta_minus = new PLFLT[ntheta];

	plAlloc2dGrid(&z, nr, ntheta+1);
	plAlloc2dGrid(&zLog, nr, ntheta+1);

	// allocate plplot x,y grid
	
	plAlloc2dGrid(&cgrid.xg, nr, ntheta+1);
	plAlloc2dGrid(&cgrid.yg, nr, ntheta+1);

	return true;
}

// ------------------------

void QpolarWindow::setData()
{
	if(dobj.is_nil() || !dobj->is_dbl()) return;
	if(thetaDep.is_nil() || !thetaDep->is_dbl()) return;
	
	zLogEmpty = true;
	
	// ensure we don't free cgrid arrays if they are not allocated (done later)
	cgrid.nx = 0;
	cgrid.ny = 0;
	

	// Load dimension data
	
	// theta dimension
	
	if( thetaDep->isDeg() ) 
	{
		rad2deg =  180. / M_PI;
		ang2rad = 1.0 / rad2deg;
	}
	else 
	{
		rad2deg =  180. / M_PI;
		ang2rad = 1.0;
	}
	
	// allow for record varying depends
	int thetaAtRec = 0;
	if(thetaDep->seqSize() == dobj->seqSize()) thetaAtRec = atSlider->record;
			

	if( ntheta != thetaDep->Dims()[0]){
		QpolarAppendTextDisplay("data dimension does not match depend for theta");
		return;
	}

	for (int j = 0; j < ntheta; j++) {
		double val = thetaDep->dbl( thetaAtRec * ntheta + j);
		if (val == thetafillVal || !isfinite(val)) theta[j] = 0.;
		else theta[j] = val * ang2rad;
	}
		
	// r dimension
	
	// allow for record varying depends
	int rAtRec = 0;
	if(rDep->seqSize() == dobj->seqSize()) rAtRec = atSlider->record;
	
	nr = rDep->Dims()[0]; // reset here as it changes due to removal of fills or r below RadMin
	
	// n is total number of data points
	n = nr * ntheta;
	
	for (int i = 0; i < nr; i++) {
		double val = rDep->dbl( rAtRec * nr + i);
		if (val == rfillVal || !isfinite(val) ) r[i] = 0;
		else r[i] = val;
	}

	//  Get delta plus and minus for theta
	// if deltas don't exist set bin edge to same as point
	
	dThetaSet = setDeltaTheta(theta_plus, DELTA_PLUS);
	dThetaSet = (setDeltaTheta(theta_minus, DELTA_MINUS) || dThetaSet) ;
	
	
	//  Get delta plus and minus for r
	// if deltas don't exist set bin edge to same as point

	dRSet = setDeltaRad(r_plus, DELTA_PLUS);
	dRSet = (setDeltaRad(r_minus, DELTA_MINUS) || dRSet) ;
	
	
	full = false;
	if (   abs(sin((theta_plus[0] - theta_minus[ntheta - 1]) / 2)) < 0.01
		|| abs(sin((theta_plus[ntheta - 1] - theta_minus[0]) / 2)) < 0.01) {
		full = true;
				
	} 


	// fill the z data array

	// get Matrix object, take record if dobj is a seq
	int atRec = atSlider->record;

	// fill z[][]

	for (int i = 0; i < nr; i++) {
				
		if(orderIsRTh)
		{
			for (int j = 0; j < ntheta; j++) {
				z[i][j] = dobj->dbl(atRec, i, j);
				if (z[i][j] == zfillVal) z[i][j] = 0.;
			}
		} 
		else {
			for (int j = 0; j < ntheta; j++) {
				z[i][j] = dobj->dbl(atRec, j, i);
				if (z[i][j] == zfillVal) z[i][j] = 0.;
			}
		}
	}
	
	
	// set up ordered arrays	

	 sort_r_theta(); // sorts r and theta in ascending order and adjusts z 
	 				 // also applies new order to deltas and initialises rmin and rmax

	QString txt = QString("Min,Max r values: %1, %2").arg(rmin).arg(rmax);
	QpolarAppendTextDisplay(txt.toLatin1().data());

	// remove -ve values if log axis
	
	if(rmin <= 0 && radiusLog->isChecked()) {
		rmin = rminNonZero;

		QString txt = QString("Negative or 0 r values: rmin changed to %1 ").arg(rmin);
		QpolarAppendTextDisplay(txt.toLatin1().data());

		for (int i = 0; i < nr; i++) {
			if(r[i] < rmin) r[i] = rmin;
			if(r_plus[i] < rmin) r_plus[i] = 1.1*rmin;
			if(r_minus[i] < rmin) r_minus[i] = rmin;
		}
	}
	
	// set user R Max/Min to use in plots
	if(radAuto->isChecked()){
		userRmax = rmax;
		if(radiusLog->isChecked()) userRmin = rminNonZero;
		else userRmin = rmin;
	}	
	else{
		bool OK;
		double userValue = RadMin->text().toDouble(&OK);
		if(OK) userRmin = (PLFLT) userValue;
		else userRmin = rmin;

		userValue = RadMax->text().toDouble(&OK);
		if(OK) userRmax = (PLFLT) userValue;
		else userRmax = rmax;
	}

	nr_start = 0;
	// move nr_start to above userRmin 
	for(nr_start=0; nr_start<nr-1; nr_start++)
	{
		if(r_plus[nr_start] > userRmin ) break;
	}
	
	nr_end = nr;
	// move nr_end to below userRmax, but never lower than 2
	for(nr_end=nr; nr_end>1; nr_end--)
	{
		if(r_minus[nr_end-1] < userRmax ) break;
	}
	
	
	for (int i = 0; i < nr; i++) {
		// plot starts when r_plus above userRmin, so catch small r and r_minus
		if(r[i] < userRmin) r[i] = userRmin;
		if(r_minus[i] < userRmin) r_minus[i] = userRmin;
	}	
	
	for (int i = 0; i < nr; i++) {
		// plot ends when r_minus above userRmax, so catch large r and r_plus
		if(r[i] > userRmax) r[i] = userRmax;
		if(r_plus[i] > userRmax) r_plus[i] = userRmax;
	}	
	
	
	if (radiusLog->isChecked()) {
		//log graph

		if (userRmin < 1.e-30) {
			txt = QString("Log Radial Min: %1 too small").arg(userRmin);
			QpolarAppendTextDisplay(txt.toLatin1().data());
			userRmin = rminNonZero;
			txt = QString("Reset to: %1").arg(userRmin);
			QpolarAppendTextDisplay(txt.toLatin1().data());
		}
		
		

		for (int i = 0; i < nr; i++) {
			r[i] = SaferLog10(r[i] / userRmin);
			r_plus[i] = SaferLog10(r_plus[i] / userRmin);
			r_minus[i] = SaferLog10(r_minus[i] / userRmin);
		}
		userRmax = SaferLog10(userRmax / userRmin);

	}
	else {
		for (int i = 0; i < nr; i++) {
			r[i] -= userRmin;
			r_plus[i] -= userRmin;
			r_minus[i] -= userRmin;
		}
		userRmax -= userRmin;

		
	}

	cgrid.nx = nr;
	if(full) cgrid.ny = ntheta + 1;
	else cgrid.ny = ntheta;	


}

// --------------------------------------

bool QpolarWindow::setDeltaTheta(PLFLT *result, const char *delta)
{
	if(thetaDep.is_nil()) return false;

	double sign = 1;
	if(strcmp(delta, DELTA_MINUS) == 0) sign = -1;
	
	bool Set = false; // reset to true if either delta exists 
	
	DvObject_var obj_delta = thetaDep->get_xref(delta);
	if( obj_delta.is_ok() && obj_delta->is_dbl()) {
		
		int atRec = 0;
		if(obj_delta->seqSize() > atSlider->record) atRec = atSlider->record;

		if (obj_delta->arraySize() == 1) {						//Sometimes the delta_plus is a scalar instead of an array.
			double val = obj_delta->dbl(atRec);

			if ( val != thetafillVal){
				Set = true;
				for (int j = 0; j < ntheta; j++)  result[j] =  theta[j] + (val * sign ) * ang2rad;
			} 
			else{
				QpolarAppendTextDisplay("No suitable delta found for Theta");
				for (int j = 0; j < ntheta; j++)  result[j] = theta[j];			
			}
		}
		else {
			Set = true;
			for (int j = 0; j < ntheta; j++) {
				double val = obj_delta->dbl(atRec * ntheta + j);
				if(val != thetafillVal){
					result[j] = theta[j] + (val * sign) * ang2rad;
				}
				else
					result[j] = theta[j];
			}
		}
	}
	else{
		QpolarAppendTextDisplay("No delta_plus not found for theta");
		for (int j = 0; j < ntheta; j++){
			result[j] = theta[j];
		}
	}
	return Set;
}

// --------------------------------------

bool QpolarWindow::setDeltaRad(PLFLT *result, const char *delta)
{

	bool Set = false; // reset to true if either delta exists 

	double sign = 1.;
	if(strcmp(delta, DELTA_MINUS) == 0) sign = -1.;

	DvObject_var obj_delta = rDep->get_xref(delta);
	
	if (obj_delta.is_ok() && obj_delta->is_dbl()) {
		
		// if sequence get matrix for this record
		int atRec = 0;
		if(obj_delta->seqSize() > atSlider->record) atRec = atSlider->record;
		
		if (obj_delta->arraySize() == 1) {

			double val = obj_delta->dbl(atRec);
			if (val != rfillVal) { //Sometimes delta+ is a scalar instead of an array.
				Set = true;
				for (int i = 0; i < nr; i++){
					result[i] = val * sign + r[i];
				}
			} 
			else{
				QpolarAppendTextDisplay("No suitable delta_plus not found for r");
				for (int i = 0; i < nr; i++){
					result[i] = r[i];
				}
			}
		} 
		else {
			Set = true;
			for (int i = 0; i < nr; i++) {
				double val = obj_delta->dbl(atRec * nr + i);
				if(val != rfillVal)
					result[i] = val * sign + r[i];
				else
					result[i] = r[i];
			}
		}
	}
	else{
		QpolarAppendTextDisplay("No delta_plus not found for r");
		for (int i = 0; i < nr; i++){
			result[i] = r[i];
		}
	}

	return Set;
}
	

// --------------------------------------

void QpolarWindow::doRun()
{
	static bool animationRunning=false;

	if(runBtn->text()  == QString(" || ") ){
		qApp->processEvents(); // get rid of multiple clicks now
		QpolarAppendTextDisplay("\n Animation paused");
		runBtn->setText("Animate Time");
		animationRunning = false;
		this->raise();
	}
	else{
		
		// find max colour
		if(colourAuto->isChecked()){
			// force fixed col max/min
			QpolarAppendTextDisplay("\n Finding Colour range...");
			atSlider->adjustRec(0);
			setRecord();
			double colMinVal = zmin;
			double colMaxVal = zmax;
			for(int nt=1; nt<=atSlider->maxRecord; nt++){
				atSlider->adjustRec(nt);
				setRecord();
				if(colMinVal > zmin && zmin > 2.e-31) colMinVal = zmin;
				if(colMaxVal < zmax) colMaxVal = zmax;
			}
			colMin->setText(QString("%1").arg(colMinVal));
			colMax->setText(QString("%1").arg(colMaxVal));
			colourAuto->setChecked(false);
		}
		
		// start at beginning if at end
		if(atSlider->record >= atSlider->maxRecord ) atSlider->adjustRec(0);
		
		QpolarAppendTextDisplay("\n Run time animation");
		runBtn->setText(" || ");
		animationRunning = true;
		if(delay->value() != 0){
			while(animationRunning  ){
				QTime wait;
				wait.start();
				while(wait.elapsed() < delay->value()) ; // do nothing
				
				doPlot(true);
				if(atSlider->record >= atSlider->maxRecord ) break;
				atSlider->incRec();
				qApp->processEvents();
			}
		}
		else while(animationRunning  ){
			doPlot(true);
			if(atSlider->record >= atSlider->maxRecord ) break;
			atSlider->incRec();
			qApp->processEvents();
		}
		this->raise();
		runBtn->setText("Animate Time");
		animationRunning=false;

	}
}

// --------------------------------------

void QpolarWindow::doPlot(bool animation)
{
	QpolarAppendTextDisplay("\nNew Plot");

    // slot invoked when Plot push button clicked
  	if(QuSlotDebug())
	{
		printf("QpolarWindow::doPlot \n");
		fflush(stdout);
	}
	
	if(dobj.is_nil()) return;
	
	// check that data has not been deleted (only edited_OK() signal resets objects
	if(	dataSlot->reValidate(false /* don't emit edited_OK */) == false) 
	{
		QpolarAppendTextDisplay("Data no longer valid\n");
		return;
	}
	
	if(nr <= 0 || ntheta <= 0){
		QpolarAppendTextDisplay("Zero bin dimension\n");
		return;
	}

	// User settings
		
	bool OK;			
	double _zmin = colMin->text().toDouble(&OK);
	if(OK) zmin = (PLFLT) _zmin;
	double _zmax = colMax->text().toDouble(&OK);
	if(OK) zmax = (PLFLT) _zmax;
	
	
	fract = fractData->value() / 100.;		//fraction of data points to be labeled.
	
	double r_scale = rScale->text().toDouble(&OK);
	if(OK) rroom = (PLFLT) r_scale;		//Space of box: how much bigger should box be compared to r max?
	else rroom = 1.3;

	// Initialize PLplot.
    
	setwindow(animation);

	// To make sure the new width will be validated:
	// plwid only stores changes in width
	// If no change from the previous plot, the default
	// width is not stored in the device buffer.
	// Thus we force a change, to store it.
	plwid(100);
	pladv(0);

	
	if (colourLog->isChecked()) {
		zmin = SaferLog10(zmin);
		zmax = SaferLog10(zmax);
	}

	select_vpor(-1);


	plcol0(0);
	plschr(QPUI_CHAR_HEIGHT, 1.0*title_size_fld->value());

	pllab("", "", titleSlot->text().toLatin1().data());

	plschr(QPUI_CHAR_HEIGHT, 1.2*text_size_fld->value());

	select_vpor(0);

	plwid(4);

	pllab("", ThLabSlot->text().toLatin1().data(), timeText.c_str());

	plwind(-userRmax * rroom, userRmax * rroom, -userRmax * rroom, userRmax * rroom);
	
	// set colour table
	colourTable->getCurrentTable().apply();

	if (plotType->currentIndex() == 1){
		isArcfill = true;   
		arcfill();
	}
	else isArcfill = false;

	if (plotType->currentIndex() == 0 || showCont->isChecked() ) // calling shade and contour routines
		shade_cont();


	if (dataPtOpts->currentIndex() != 0)
		plot_data_pts();


	drawaxis(RLabSlot->text().toLatin1().data());

	drawcolscale();
	

#ifdef Q_OS_MACX
// fix Mac OS problem with plot not showing until focus change
	
	if(animation) return; // avoid flicker

	win->show();
#endif


}

// -------------------------------------

// Sorts a regular grid of r and theta in ascending order. Theta is set to 0-2*pi before the sorting is done. Output theta is in radians.
void QpolarWindow::sort_r_theta() {

	// make a single array out of z
	PLFLT w[ntheta * nr];

	for (int j = 0; j < ntheta; j++) {
		for (int i = 0; i < nr; i++) {
			w[j * nr + i] = z[i][j];
		}
	}

	// 2d array sorting in terms of theta.
	quick_sort_2d(theta, w, 0, ntheta - 1, nr, theta_plus, theta_minus);


	//Assigning w back to z
	for (int j = 0; j < ntheta; j++) {
		for (int i = 0; i < nr; i++) {
			z[i][j] = w[j * nr + i];
		}
	}

	// unpack again for r sort
	for (int i = 0; i < nr; i++) {
		for (int j = 0; j < ntheta; j++) {
			w[i * ntheta + j] = z[i][j];
		}
	}

	// 2d array sorting in terms of r
	quick_sort_2d(r, w, 0, nr - 1, ntheta, r_plus, r_minus);


	//Assigning w back to z
	for (int i = 0; i < nr; i++) {
		for (int j = 0; j < ntheta; j++) {
			z[i][j] = w[i * ntheta + j];
		}
	}
	
	
	rmax = r_plus[nr-1];
	rmin = r_minus[0];
	rminNonZero = rmin;
	if(rminNonZero < 1.e-30) {
		for(int i=0; i<nr-1; i++){
			// if -ve will be moved along to 0.0, so justy move up far enough (rmin is subtracted later)
			if(r[i] - rmin > 1.e-30) {
				rminNonZero = (r[i] + rmin) * 0.5;
				break;
			}
			else if(r_plus[i] - rmin> 1.e-30) {
				rminNonZero = (r_plus[i] + rmin) * 0.5;
				break;
			}
			else if(r_minus[i+1] - rmin> 1.e-30) {
				rminNonZero = (r_minus[i+1] + rmin) * 0.5;
				break;
			}
		}
	}
	
}

//----------------------------

//Sets up page and window.
void QpolarWindow::setwindow(bool animation) {


	bool landscape;
    bool getNewWindow = (plotWin_id->currentWindow == NULL || (newWindow->isChecked() && !animation) );
    QPrinter::PageSize pageSize;
    switch(page_size->currentIndex())
    {
        case QPUI_A4:  default: case QPUI_DEFAULT:
        {
            pageSize = QPrinter::A4;
            break;
        }
        case QPUI_CUSTOM:
        {
            pageSize = QPrinter::Custom;
            break;
        }
        case QPUI_EXECUTIVE:
        {
            pageSize = QPrinter::Executive;
            break;
        }
        case QPUI_LEGAL:
        {
            pageSize = QPrinter::Legal;
            break;
        }
        case QPUI_LETTER:
        {
            pageSize = QPrinter::Letter;
        }
            
    }
	
    QPrinter::Orientation orientation;
	if(pageOrientation->currentIndex() == 0) {
		landscape = false;
		orientation = QPrinter::Portrait;
	}
	else {
		landscape = true;
		orientation = QPrinter::Landscape;
	}
    
    bool OK;
    double slotValue = pageWidth->text().toDouble(&OK);
    if(OK) width = slotValue;
    
    slotValue = pageHeight->text().toDouble(&OK);
    if(OK) height = slotValue;
    
	if( !getNewWindow )
	{	
		// pick up existing window
        
        win = plotWin_id->focusWindow(width, height, DPI, pageSize, orientation);
        return;
		
	}

	if(width == height) width+=1;		// Bug saving jpgs with the same width and height, entire jpg turns black. KaHo
		win = plotWin_id->newPlotWindow(width, height, DPI, pageSize, orientation);

    win->setSmooth(true);
	if(animation) win->move(0,0);
	
	pladv(0); // this is done again later, but clears a bug with plwid being ignored

}



// ----------------------

// Sets the page tab feedback
void QpolarWindow::setPage()
{
	bool readonly = true;
	if(page_size->currentIndex() == QPUI_CUSTOM) readonly = false;

	switch(dpiPage->currentIndex())
	{
		case QPUI_72_DPI:
		{
			DPI = 72;
			break;
		}
		case QPUI_150_DPI:
		{
			DPI = 150;
			break;
		}
		case QPUI_300_DPI:
		{
			DPI = 300;
			break;
		}
		case QPUI_600_DPI:
		{
			DPI = 600;
			break;
		}
		case QPUI_1200_DPI:
		{
			DPI = 1200;
			break;
		}
	}
	
	switch(pageUnits->currentIndex())
	{
		case QPUI_PIXELS: 
		{
			conv = (double) DPI;
			break;
		}
		case QPUI_INCHES: 
		{
			conv = 1.0;
			break;
		}
		case QPUI_CM: 
		{
			conv = 2.54;
			break;
		}
		case QPUI_MM: 
		{
			conv = 25.4;
			break;
		}
		case QPUI_PSPOINTS: 
		{
			conv = (double) DPI;
			break;
		}
	}


	if(readonly)
	{
		switch(page_size->currentIndex()){
			case QPUI_A4: default:
			{
				width = 8.3 * conv;
				height = 11.694 * conv;
				break;
			}
			case QPUI_EXECUTIVE:
			{
				width = 7.5 * conv;
				height = 10.0 * conv;
				break;
			}
			case QPUI_LEGAL:
			{
				width = 8.5 * conv;
				height = 14 * conv;
				break;
			}
			case QPUI_LETTER:
			{
				width = 8.5 * conv;
				height = 11.0 * conv;
				break;
			}
			case QPUI_DEFAULT:
			{
				width = 8.3 * conv;
				height = 11.694 * conv;
				break;
			}
		}
		pageWidth->setText(QString().setNum(width));
		pageHeight->setText(QString().setNum(height));
		
	}
	pageWidth->setReadOnly(readonly);
	pageHeight->setReadOnly(readonly);
	
	pageWidth->setEnabled(!readonly);
	pageHeight->setEnabled(!readonly);
    
    
	for(int i=0; i<plotWin_id->size(); i++){
		plotWin_id->windowAt(i)->setPrintRes(DPI);
	}


}

// ---------------------------

// Method to quicksort x array and pseudo 2d arrays in ascending order according to the xarray.
// eg. for depend x[i], y[j] and data w[i*ny + j], accepts x and w.
// If wanted to sort y, need to re-order w to become w[j*nx + i]
void QpolarWindow::quick_sort_2d(PLFLT xarray[], PLFLT z[], int start, int end, int length, PLFLT x_plus[], PLFLT x_minus[]) {
	if (start < end) {
		PLFLT pivot = xarray[end];
		PLFLT pivot_plus = x_plus[end];
		PLFLT pivot_minus = x_minus[end];
		PLFLT w[length];
		int j;
		for (j = 0; j < length; j++) {
			w[j] = z[end * length + j];
		}
		int bottom = start - 1;
		int top = end;

		bool done = false;
		while (!done) {
			while (!done) {
				bottom += 1;

				if (bottom == top) {
					done = true;
					break;
				}
				if (xarray[bottom] > pivot) {
					xarray[top] = xarray[bottom];
					x_plus[top] = x_plus[bottom];
					x_minus[top] = x_minus[bottom];
					for (j = 0; j < length; j++) {
						z[top * length + j] = z[bottom * length + j];
					}
					break;
				}
			}
			while (!done) {
				top = top - 1;

				if (top == bottom) {
					done = true;
					break;
				}
				if (xarray[top] < pivot) {
					xarray[bottom] = xarray[top];
					x_plus[bottom] = x_plus[top];					
					x_minus[bottom] = x_minus[top];					
					for (j = 0; j < length; j++) {
						z[bottom * length + j] = z[top * length + j];
					}
					break;
				}
			}
		}
		xarray[top] = pivot;
		x_plus[top] = pivot_plus;
		x_minus[top] = pivot_minus;
		
		for (j = 0; j < length; j++) {
			z[top * length + j] = w[j];
		}

		quick_sort_2d(xarray, z, start, top - 1, length, x_plus, x_minus);
		quick_sort_2d(xarray, z, top + 1, end, length, x_plus, x_minus);
	}
}


//	near_nice_number() and nice_min() are used to find nice values to draw contour lines.
//	nice_min() rounds down zmin to a nice number. How much it gets rounded down depends on the stepsize of the contour lines.
//	near_nice_number() is used to round stepsize into nice numbers.
//	Hence each contour level sits on (nice min) + integer * (nice number)


//	find a nice number near to x. Nice means 1,2, or 5 times power of 10
//	The Nice number is always larger in magnitude than x
PLFLT QpolarWindow::near_nice_number(PLFLT x) {
	double xlog, r, temp;
	int pow_of_10, pos, inum;

	pos = (x > 0); // for x < 0, need to strip and keep sign info
	xlog = SaferLog10(fabs(x));
	pow_of_10 = (int) xlog; // find nearest power of 10; if < 0, dec by 1


	if (pow_of_10 <= 0)
		pow_of_10--;
	temp = xlog - pow_of_10; // temp is now log of number part only

	if (temp > 1) {
		temp -= 1;
		pow_of_10++;
	} // fudge for vals 0.1-0.9
	r = pow(10.0, temp);
	//   inum = (int) r;
	inum = (int) r;
	if (r / inum > 1.0)
		inum++;
/*	if (inum > 5)
		inum = 10;
	else if (inum > 2)
		inum = 5;
	else if (inum > 1)
		inum = 2;
	else
		inum = 1;*/
	r = inum * pow(10.0, pow_of_10);
	if (!pos)
		r *= -1;

	return (PLFLT) r;

}

PLFLT QpolarWindow::nice_min(PLFLT zmin, PLFLT stepsize) {
	int pos = (zmin > 0);
	PLFLT nice_min;
	int steppow_of_10 = (int)floor(SaferLog10(stepsize));

	int zminpow_of_10 = (int)floor(SaferLog10(abs(zmin)));

	PLFLT tmp = SaferLog10(abs(zmin)) - zminpow_of_10;

	if (zminpow_of_10 - steppow_of_10 != 0) {
		if (pos)
			nice_min = floor(pow(10.0, tmp - steppow_of_10 + zminpow_of_10 - 1));
		else
			nice_min = ceil(pow(10.0, tmp - steppow_of_10 + zminpow_of_10 - 1));
		nice_min *= pow(10.0, steppow_of_10 + 1);

	} else //if the step size is close to the value of zmin don't round down zmin too much.
	{
		if (pos)
			nice_min = floor(pow(10.0, tmp - steppow_of_10 + zminpow_of_10));
		else
			nice_min = ceil(pow(10.0, tmp - steppow_of_10 + zminpow_of_10));
		nice_min *= pow(10.0, steppow_of_10);
	}

	if (!pos)
		nice_min *= -1;

	return nice_min;
}


PLINT excludedRegion(PLFLT x, PLFLT y)
{
	if( x*x + y*y > global_rmax*global_rmax ) 
	{
		return 0;
		
	}
	return 1;
}

//Accepts the data and produces the shades. Also calls contouring routine.
void QpolarWindow::shade_cont() {
	
	// logs may have been taken in arcfill
	if (colourLog->isChecked() && !isArcfill) {
		for (int i = nr_start; i < nr_end; i++) {  // only take logs used
			for (int j = 0; j < ntheta; j++) {
				zLog[i][j] = SaferLog10(z[i][j]);
			}
		}
	}

	// if full 360 covered add closing column = first column
	if (full) {
		if(colourLog->isChecked()){
			for (int i = nr_start; i < nr_end; i++) {
				zLog[i][ntheta] = zLog[i][0];
			}
		}
		else{
			for (int i = nr_start; i < nr_end; i++) {
				z[i][ntheta] = z[i][0];
			}
		}
	}


	// fill x,y grid
	for (int i = nr_start; i < nr_end; i++) {
		for (int j = 0; j < ntheta; j++) {
			cgrid.xg[i-nr_start][j] = r[i] * cos(theta[j]);
			cgrid.yg[i-nr_start][j] = r[i] * sin(theta[j]);
		}
	}

	if (full) {
		for (int i = nr_start; i < nr_end; i++) {
			cgrid.xg[i-nr_start][ntheta] = r[i] * cos(theta[0]);
			cgrid.yg[i-nr_start][ntheta] = r[i] * sin(theta[0]);
		}
	}

	if (plotType->currentIndex() == 0) {
		PLFLT *shedge;
		int ns = 128;
		shedge = new PLFLT[ns + 1];
		
		if(colourLog->isChecked()) shedge[0] = -31;
		else shedge[0]= 0.;
		
		for (int i = 1; i < ns + 1; i++) {
			shedge[i] = zmin + (zmax - zmin) * (PLFLT) i / (PLFLT) ns;
		}

		global_rmax = userRmax;
		
		if(colourLog->isChecked()){
			plshades((const PLFLT**)&(zLog[nr_start]), nr_end - nr_start, ntheta + full, excludedRegion, -1., 1., -1., 1.,
			//plshades(&(zLog[nr_start]), nr_end - nr_start, ntheta + full, excludedRegion, -1., 1., -1., 1.,
				shedge, ns + 1, 2, 0, 0, plfill, false, pltr2, (void *) &cgrid);
		}
		else {
			plshades((const PLFLT**)&(z[nr_start]), nr_end - nr_start, ntheta + full, excludedRegion, -1., 1., -1., 1.,
			//plshades(&(z[nr_start]), nr_end - nr_start, ntheta + full, excludedRegion, -1., 1., -1., 1.,
				shedge, ns + 1, 2, 0, 0, plfill, false, pltr2, (void *) &cgrid);
		}
 		delete[] shedge;
	}

	if (showCont->isChecked()) {
		PLFLT clevel[100];
        
        plwid(QtWidth2PlWidth(line_width_fld->value()));
              
		int steps = cont_lvl(clevel);


		plcol0(0);
		pl_setcontlabelformat(4, 3);
		bool labelCont = false;
		if(contLabelType->currentIndex() == 1 || contLabelType->currentIndex() == 3) labelCont = true;
		pl_setcontlabelparam(0.006, 0.4, 0.2, labelCont);
		if(colourLog->isChecked()){
			plcont((const PLFLT**)&(zLog[nr_start]), nr_end - nr_start, ntheta + full, 1, nr_end - nr_start, 1, ntheta + full, clevel, steps, pltr2, (void *) &cgrid);		
			//plcont(&(zLog[nr_start]), nr_end - nr_start, ntheta + full, 1, nr_end - nr_start, 1, ntheta + full, clevel, steps, pltr2, (void *) &cgrid);		
		}
		else {
			plcont((const PLFLT**)&(z[nr_start]), nr_end - nr_start, ntheta + full, 1, nr_end - nr_start, 1, ntheta + full, clevel, steps, pltr2, (void *) &cgrid);
			//plcont(&(z[nr_start]), nr_end - nr_start, ntheta + full, 1, nr_end - nr_start, 1, ntheta + full, clevel, steps, pltr2, (void *) &cgrid);
		}
	}
	

}


//Accepts data and fills the bins according to the bin boundaries. Boundaries are defined arbitarily if not availible. 
void QpolarWindow::arcfill() {

	plwid(4);
	int i, j;
	plpsty(0);
	
	if(zmax <= zmin){
		QpolarAppendTextDisplay("Bad z range.\n");
		return;
	}

	
	if (colourLog->isChecked() && zLogEmpty) {
		zLogEmpty = false;
		for (int i = nr_start; i < nr_end; i++) {  // only take logs used
			for (int j = 0; j < ntheta; j++) {
				zLog[i][j] = SaferLog10(z[i][j]);
			}
		}
	}

	if ( !dRSet) {
		//Automatically Determining r bin edges
		for (i = 0; i < nr - 1; i++) {
			r_plus[i] = 0.5 * (r[i] + r[i + 1]);
		}
		r_plus[nr - 1] = r[nr - 1] * 1.05;
		QpolarAppendTextDisplay("Cannot find Bin Boundaries for radial bins, defining arbitary boundaries.\n");
	}

	//Automatically Determining theta bin edges
	if ( !dThetaSet) {
		for (j = 0; j < ntheta - 1; j++) {
			theta_plus[j] = theta_minus[j + 1] = 0.5 * (theta[j + 1] + theta[j]);
		}
		theta_plus[ntheta - 1] = theta_plus[ntheta - 1] + 0.5 * (theta[ntheta - 1] - theta[ntheta - 2]) ;
		theta_minus[0]	= theta[0] - 0.5* (theta[1] - theta[0]);
		
		
		QpolarAppendTextDisplay("Cannot find Bin Boundaries for angular bins, defining arbitary bin boundaries.\n");
	}

		
	PLFLT col;
	PLFLT zval; // local variable to avoid changing data
	PLFLT z_range_inv; 
	
	bool isZLog = colourLog->isChecked();
	z_range_inv = 1.0 / (zmax - zmin);
	
	for (j = 0; j < ntheta; j++) {
		PLFLT thPlus = theta_plus[j]  * rad2deg;
		PLFLT thMinus = theta_minus[j]  * rad2deg;
		for (i = nr_end - 1; i >= nr_start; i--) {
			if (isfinite(z[i][j])) {
				if(isZLog) {
					zval = zLog[i][j];
				}
				else {
					zval = z[i][j];
				}
				
				// isfinite test finds NAN and inf
				
				col = (zval - zmin) * z_range_inv;
				if(col < 0.) col=0.;
				if(col >= 0.999) col=0.999;
				plcol1(col);
			} 
			else
				plcol0(15);
			
		
			// draw shortest arc
			if( thPlus - thMinus < 180.)
				plarc(0.0, 0.0, r_plus[i], r_plus[i], thMinus, thPlus, 0, 1);
				//plarc(0.0, 0.0, r_plus[i], r_plus[i], thMinus, thPlus, 1);
			else
				plarc(0.0, 0.0, r_plus[i], r_plus[i], thPlus, thMinus, 0, 1);
				//plarc(0.0, 0.0, r_plus[i], r_plus[i], thPlus, thMinus, 1);

		}
		
	}
}

// Plots data points at bin centre. Plots an "X" in regions with fill values.
void QpolarWindow::plot_data_pts() {


	int i, j, n, fill_count = 0;
	PLFLT *x, *y;

	double cadence = 0.99999999 /fract;
	double threshold = cadence;
	
	bool showValues = false;
	if(dataPtOpts->currentIndex() == 2) showValues = true;
	PLFLT lowerLimit = zmin;
	if( colourLog->isChecked()) lowerLimit = pow(10., zmin); // convert zmin back to linear

	bool OK;
	double limit = dataOrigin->text().toDouble(&OK); 
	if(OK) {
		lowerLimit = (PLFLT) limit;
	}
	
	x = new PLFLT[nr * ntheta];
	y = new PLFLT[nr * ntheta];

	plcol0(1);

	pljoin(0, 0, 0, .5);		// Coding around: @Bug2: Somehow plplot requires a line to be drawn before we can call plpoin. So we draw this line overlapping the axis.

	plcol0(0);

	int counter=0;
	for (i = nr_start; i < nr_end; i++) {
		for (j = 0; j < ntheta; j++) {		
			if (isfinite(z[i][j])) { // this test fails for NAN
				n = j + i * ntheta - fill_count;
				x[n] = r[i] * cos(theta[j]);
				y[n] = r[i] * sin(theta[j]);

				if ( showValues && z[i][j] >= lowerLimit) {
					counter++;
					if( counter >= threshold ){
						threshold += cadence;
						plschr(QPUI_CHAR_HEIGHT, .9*text_size_fld->value());
						char value[50];
						sprintf(value, "%.2e", z[i][j]);
						plptex(x[n] + userRmax / 200, y[n] + userRmax / 200, 1, 1, 0, value);
					}
				}
			} 
			else {
				fill_count++;
			}
		}
	}
	plcol0(0);


	plpoin(nr * ntheta - fill_count, x, y, 1);

	//plotting fillVal data
	n = 0;

	for (i = 0; i < nr; i++) {
		for (j = 0; j < ntheta; j++) {
			if (z[i][j] != z[i][j]) {
				x[n] = r[i] * cos(theta[j]);
				y[n] = r[i] * sin(theta[j]);
				n++;
			}
		}
	}
	plpoin(fill_count, x, y, 5);
	delete[] x;
	delete[] y;
}


//Draws the colourscale. This is achived by drawing 2000 lines in a window so it looks continuous.
void QpolarWindow::drawcolscale() {

	bool OK;

	plwid(QtWidth2PlWidth(axis_width_fld->value()));

	select_vpor(5);

	if (pageOrientation->currentIndex() == 1) {
		
		// Landscape
		plwind(151.0, 161.0, zmin, zmax);
		PLFLT x[4], y[4];
	
		x[0]=151.0;
		x[1]=151.0;
		x[2]=161.0;
		x[3]=161.0;

		PLFLT zstep=(zmax-zmin)/200.;
		PLFLT i=0.;
		for(PLFLT col=zmin;col<zmax;col+=zstep, i+=1./200.)
		{
			y[0] = col;
			y[3] = col;
			y[1] = col+zstep;
			y[2] = y[1];
			if(i>=1.) i=0.999;
			plcol1(i);
			plfill(4, x, y);
		}

		plcol0(0);
		if (colourLog->isChecked())
			plbox("", 0.0, 0, "bnlstv", 0.0, 0);
		else
			plbox("", 0.0, 0, "bnstv", 0.0, 0);

		plmtex("l", 6, 0.5, 0.5, colLabSlot->text().toLatin1().data());

		//Drawing contour lines next to the box


		if (contLabelType->currentIndex() == 2 || contLabelType->currentIndex() == 3) {

			char level[50];
			PLFLT clevel[100];
		
			int steps = cont_lvl(clevel);

			for (int i = 0; i < steps; i++) {
				pljoin(156, clevel[i], 161, clevel[i]);pljoin(clevel[i], 156, clevel[i], 161);
				if (colourLog->isChecked()) 	
					sprintf(level, "10#u%.2f", clevel[i]);
				else 
					sprintf(level, "%-.2g", clevel[i]);
				if (clevel[i] - zmin >= 0)
					plmtex("rv", 0.5, (clevel[i] - zmin) / (zmax - zmin), 0, level);
			}
		}

		//Other colours
		PLFLT xm[] = { 0, 0, 1, 1 };
		PLFLT ym[] = { 0, 1, 1, 0 };
		char max[50], min[50];

		if (colourLog->isChecked()) {
			sprintf(max, ">10#u%.2f", zmax);
			sprintf(min, "<10#u%.2f", zmin);
		} 
		else {
			sprintf(max, ">%.2e", zmax);
			sprintf(min, "<%.2e", zmin);
		}

		select_vpor(6);
		plwind(0, 1, 0, 1);
		plcol1(1.0);
		plfill(4, xm, ym);
		plcol0(0);
		plbox("bc", 0.0, 0, "bc", 0.0, 0);
		plmtex("t", 1.5, 0.5, 0.5, max);

		select_vpor(7);
		plwind(0, 1, 0, 1);
		plcol1(0.0);
		plfill(4, xm, ym);
		plcol0(0);
		plbox("bc", 0.0, 0, "bc", 0.0, 0);
		plmtex("b", 1.5, 0.5, 0.5, min);
	}
	else{				
		
		// Portrait
		// Put the colourbar horizontally so to maxmize use of space.
		
		PLFLT titleOffset = 0.0;
		if(contLabelType->currentIndex() == 2 || contLabelType->currentIndex() == 3) titleOffset = 5;
		
		plwind(zmin, zmax, 151.0, 161.0);

		PLFLT x[4], y[4];
	
		y[0]=151.0;
		y[1]=151.0;
		y[2]=161.0;
		y[3]=161.0;

		PLFLT zstep=(zmax-zmin)/200.;
		PLFLT i=0.;
		for(PLFLT col=zmin;col<zmax;col+=zstep, i+=1./200.)
		{
			x[0] = col;
			x[3] = col;
			x[1] = col+zstep;
			x[2] = x[1];
			if(i>=1.) i=0.999;
			plcol1(i);
			plfill(4, x, y);
		}

		plschr(QPUI_CHAR_HEIGHT, 1.1*text_size_fld->value());

		plcol0(0);
		if (colourLog->isChecked())
			plbox("bnlstv", 0.0, 0, "", 0.0, 0);
		else
			plbox("bnstv", 0.0, 0, "", 0.0, 0);

		plmtex("t", 2.0+titleOffset, 0.5, 0.5, colLabSlot->text().toLatin1().data());


		// Drawing contour lines next to the box		
		if (titleOffset > 0.5) {
			// happens when contours are labelled on colour bar
			char level[50];
			PLFLT clevel[100];
		
			int steps = cont_lvl(clevel);
			for (int i = 0; i < steps; i++) {
				pljoin(clevel[i], 156, clevel[i], 161);
				if (colourLog->isChecked()) 	
					sprintf(level, "10#u%.2f", clevel[i]);
				else 
					sprintf(level, "%-.2g", clevel[i]);
				if (clevel[i] - zmin >= 0)
					plmtex("tv", 0.5, (clevel[i] - zmin) / (zmax - zmin), 0., level);
			}
		}

		//Other colours
		PLFLT xm[] = { 0, 0, 1, 1 };
		PLFLT ym[] = { 0, 1, 1, 0 };
		char max[50], min[50];

		if (colourLog->isChecked()) {
			sprintf(max, ">10#u%.2f", zmax);
			sprintf(min, "<10#u%.2f", zmin);
		} else {
			sprintf(max, ">%.2g", zmax);
			sprintf(min, "<%.2g", zmin);
		}

		plschr(QPUI_CHAR_HEIGHT, text_size_fld->value());

		select_vpor(6);
		plwind(0, 1, 0, 1);
		plcol1(1.0);
		plfill(4, xm, ym);
		plcol0(0);
		plbox("bc", 0.0, 0, "bc", 0.0, 0);
		plmtex("b", 1.5, 0.0, 0.0, max);

		select_vpor(7);
		plwind(0, 1, 0, 1);
		plcol1(0.0);
		plfill(4, xm, ym);
		plcol0(0);
		plbox("bc", 0.0, 0, "bc", 0.0, 0);
		plmtex("b", 1.5, 0.0, 0.0, min);
		
		plschr(QPUI_CHAR_HEIGHT, 1.2*text_size_fld->value());

	}

}

//Draws the axis and the angle labels.
void QpolarWindow::drawaxis(const char *rlabel) {

	plschr(QPUI_CHAR_HEIGHT, 1.2*text_size_fld->value());
	plwid(QtWidth2PlWidth(axis_width_fld->value()));
	plcol0(0);
	select_vpor(1);
	
	//The axis is drawn from 4 quadrats. The window-viewport mapping of some quadrats are inverted to get positive value on the left/bottom of the plot.
	if (axisType->currentIndex() == 0) { //Central axis at x = 0 and y = 0 only.
		if (radiusLog->isChecked()) {
			PLFLT rend = rroom * userRmax + SaferLog10(userRmin);
			PLFLT rstart = SaferLog10(userRmin);
			plwind(rstart, rend, rstart, rend );
			plaxes(0, 0, "bnsltv", 0.0, 0, "bsltv", 0.0, 0);
			select_vpor(2);
			plwind(rend, rstart, rstart, rend);
			plaxes(0, 0, "bsltv", 0.0, 0, "csltv", 0.0, 0);
			select_vpor(3);
			plwind(rend, rstart, rend, rstart);
			plaxes(0, 0, "csltv", 0.0, 0, "csltv", 0.0, 0);
			select_vpor(4);
			plwind(rstart, rend, rend, rstart);
			plaxes(0, 0, "csltv", 0.0, 0, "bsltv", 0.0, 0);
		} 
		else {
			PLFLT rend = rroom * userRmax + userRmin;
			PLFLT rstart = userRmin;
			plwind(rstart, rend, rstart, rend);
			plaxes(0, 0, "bnst", 0.0, 0, "bst", 0.0, 0);
			select_vpor(2);
			plwind(rend, rstart, rstart, rend);
			plaxes(0, 0, "bst", 0.0, 0, "cst", 0.0, 0);
			select_vpor(3);
			plwind(rend, rstart, rend, rstart);
			plaxes(0, 0, "cst", 0.0, 0, "cst", 0.0, 0);
			select_vpor(4);
			plwind(rstart, rend, rend, rstart);
			plaxes(0, 0, "cst", 0.0, 0, "bst", 0.0, 0);
		}
	} 
	else if (axisType->currentIndex() == 1) { //Label Axis Under Plot
		if (radiusLog->isChecked()) {
			PLFLT rend = rroom * userRmax + SaferLog10(userRmin);
			PLFLT rstart = SaferLog10(userRmin);
			plwind(rstart,rend, rstart, rend);
			plaxes(0, 0, "bsltv", 0.0, 0, "bsltv", 0.0, 0);
			select_vpor(2);
			plwind(rend, rstart, rstart, rend);
			plaxes(0, 0, "bsltv", 0.0, 0, "csltv", 0.0, 0);
			select_vpor(3);
			plwind(rend, rstart, rend, rstart);
			plaxes(0, 0, "csltv", 0.0, 0, "csltv", 0.0, 0);
			select_vpor(4);
			plwind(rstart, rend, rend, rstart);
			plaxes(0, 0, "csltv", 0.0, 0, "bsltv", 0.0, 0);
			select_vpor(8);
			plwind(rstart, rend, rend, rstart);
			plaxes(0, 0, "bnsltv", 0.0, 0, "", 0.0, 0);
		} 
		else {
			PLFLT rend = rroom * userRmax + userRmin;
			PLFLT rstart = userRmin;
			plwind(rstart, rend, rstart, rend);
			plaxes(0, 0, "bst", 0.0, 0, "bst", 0.0, 0);
			select_vpor(2);
			plwind(rend, rstart, rstart, rend);
			plaxes(0, 0, "bst", 0.0, 0, "cst", 0.0, 0);
			select_vpor(3);
			plwind(rend, rstart, rend, rstart);
			plaxes(0, 0, "cst", 0.0, 0, "cst", 0.0, 0);
			select_vpor(4);
			plwind(rstart, rend, rend, rstart);
			plaxes(0, 0, "cst", 0.0, 0, "bst", 0.0, 0);
			select_vpor(8);
			plwind(rstart, rend, rend, rstart);
			plaxes(0, 0, "bnst", 0.0, 0, "", 0.0, 0);
		}
	} else if (axisType->currentIndex() == 2) { //Box and Axis
		if (radiusLog->isChecked()) {
			PLFLT rend = rroom * userRmax + SaferLog10(userRmin);
			PLFLT rstart = SaferLog10(userRmin);
			plwind(rstart, rend, rstart, rend);
			plaxes(0, 0, "bcsltv", 0.0, 0, "bcsltv", 0.0, 0);
			select_vpor(2);
			plwind(rend, rstart, rstart, rend);
			plaxes(0, 0, "bcsltv", 0.0, 0, "bcsltv", 0.0, 0);
			select_vpor(3);
			plwind(rend, rstart, rend, rstart);
			plaxes(0, 0, "bcsltv", 0.0, 0, "bcsltv", 0.0, 0);
			select_vpor(4);
			plwind(rstart, rend, rend, rstart);
			plaxes(0, 0, "bcnsltv", 0.0, 0, "cbsltv", 0.0, 0);
		} 
		else {
			PLFLT rend = rroom * userRmax + userRmin;
			PLFLT rstart = userRmin;
			plwind(rstart, rend, rstart, rend);
			plaxes(0, 0, "bcst", 0.0, 0, "cbst", 0.0, 0);
			select_vpor(2);
			plwind(rend, rstart, rstart, rend);
			plaxes(0, 0, "bcst", 0.0, 0, "cbst", 0.0, 0);
			select_vpor(3);
			plwind(rend, rstart, rend, rstart);
			plaxes(0, 0, "bcst", 0.0, 0, "bcst", 0.0, 0);
			select_vpor(4);
			plwind(rstart, rend, rend, rstart);
			plaxes(0, 0, "bncst", 0.0, 0, "cbst", 0.0, 0);
		}
	}

	pllab(rlabel, "", "");


	//Drawing Angluar labels.
	
	PLFLT rBound = 1.1 * userRmax;
	if(rBound > 0.98 * rroom * userRmax) rBound = 0.98 * rroom * userRmax;

	char text[10];
	PLFLT theta1, dx, dy, offset;
	select_vpor(0);
	plwind(-userRmax * rroom, userRmax * rroom, -userRmax * rroom, userRmax * rroom);

	pllsty(1);	//This is meant to set the line style as a dash line, but Qt doesn't support it?
	for (int i = 1; i <= 11; i++) {
		theta1 = i * M_PI / 6;
		dx = 1.03 * rBound * cos(theta1);
		dy = 1.03 * rBound * sin(theta1);

		// Draw radial spokes for polar grid.
		plcol0(0);
		pljoin(dx / 1.06, dy / 1.06, dx, dy);
		if(useLocalTime->isChecked()) {
            int pt = (int) ROUND(theta1 * 12 / M_PI);
            if(pt < 9) sprintf(text, "%01d", pt);
            else  sprintf(text, "%02d", pt);
        }
        else sprintf(text, "%d", (int) ROUND(theta1 * 180 / M_PI));
        
		// Write labels for angle.

		if (theta1 < 9.99) {
			offset = 0.45;
		} else if (theta1 < 99.9) {
			offset = 0.30;
		} else {
			offset = 0.15;
		}

		if ( ((int) (theta1*180/M_PI))%90 != 0) {		//don't label 0,90,180,270 deg because it will overlap with axis && don't label angle when using box option.
			if (dx >= -0.00001) {
				plptex(dx, dy, dx, dy, -offset, text);
			} else {
				plptex(dx, dy, -dx, -dy, 1. + offset, text);
			}
		}
	}
    
    pllsty(3);
    plarc(0, 0, rBound, rBound, 0, 360, 0, 0);
}

// Sets up contour levels and puts it in an array. Returns the number of contour lines.
int QpolarWindow::cont_lvl(PLFLT clevel[]) {

	bool OK;
	contStep = contSpace->text().toDouble(&OK);
	
	int steps = 0;
	if(!OK){

		PLFLT stepsize = (zmax - zmin) / 10;
		PLFLT nice_stepsize = near_nice_number(stepsize);
		PLFLT nice_zmin = nice_min(zmin, nice_stepsize);


		for (int i = 0; nice_stepsize * i + nice_zmin < zmax; i++) {
			if (nice_zmin + i * nice_stepsize > zmin) {
				clevel[steps] = nice_zmin + i * nice_stepsize;
				steps++;
			}
		}
	}
	else{
					
		PLFLT nice_zmin = nice_min(zmin, contStep);
		for (int i = 0; contStep * i + nice_zmin < zmax && steps < 100; i++) {
			if (nice_zmin + i * contStep > zmin) {
				clevel[steps] = nice_zmin + i * contStep;
				steps++;
			}
		}

	}
	return steps;
}



//Since lots of viewports are defined, only this part needs to be changed if the window size changes.
void QpolarWindow::select_vpor(int n)
{
	PLFLT x0, x1, y0, y1, xmean, ymean;

	PLFLT xcutoff;	//xcutoff, ycutoff is the one-sided fraction of the page that will be trimmed off if we cut a 1:sqrt(2) page out of the window.
	PLFLT ycutoff;

	bool landscape;
	
	//Getting plot option 

	if(pageOrientation->currentIndex() == 0) landscape = false;
	else landscape = true;

	//Try to fit the largest 1:sqrt(2) page into the window.
//	if(landscape){
//		xcutoff = ((PLFLT) width < height*sqrt(2)? 0:((width-height*sqrt(2))/2.)/(PLFLT)width);	//if page is very wide xcutoff will be big. vice versa
//		ycutoff = ((PLFLT) width > height*sqrt(2)? 0:((height-width/sqrt(2))/2.)/(PLFLT)height);
//	}
//	else{
		xcutoff = (width*sqrt(2.) < (PLFLT) height? 0:((width-height/sqrt(2.))/2.)/(PLFLT)width);
		ycutoff = (width*sqrt(2.) > (PLFLT) height? 0:((height-width*sqrt(2.))/2.)/(PLFLT)height);
//	}

	if (landscape) {
		x0 = 0.2 + xcutoff;
		y0 = 0.2 + ycutoff;
		y1 = 0.8 - ycutoff;
		x1 = x0 + (y1 - y0) * ((PLFLT) width / (PLFLT) height);
		xmean = (x0 + x1) / 2;
		ymean = (y0 + y1) / 2;

		switch (n) {
		case -1: //vpor for the title
			plvpor(0.02, 0.98, 0, y1 + 0.05);
			break;
		case 0: //main vpor
			plvpor(x0, x1, y0, y1);
			break;
		case 1: //top right of main
			plvpor(xmean, x1, ymean, y1);
			break;
		case 2: //top left of main
			plvpor(x0, xmean, ymean, y1);
			break;
		case 3: //bottom left of main
			plvpor(x0, xmean, y0, ymean);
			break;
		case 4: //bottom right of main
			plvpor(xmean, x1, y0, ymean);
			break;
		case 5: //col scale
			plvpor(x1 + 0.15, x1 + 0.20, y0 + 0.07, y1 - 0.07);
			break;
		case 6: //max col
			plvpor(x1 + 0.15, x1 + 0.20, y1 - 0.06, y1 - 0.03);
			break;
		case 7: //min col
			plvpor(x1 + 0.15, x1 + 0.20, y0 + 0.03, y0 + 0.06);
			break;
		case 8: //axes label
			plvpor(xmean, x1, y0 - 0.1, y0);
			break;
		}
	} else {
		x0 = 0.10 + xcutoff;
		x1 = 0.90 - xcutoff;
		y0 = 0.60 - (x1 - x0) * 0.5 * ((PLFLT) width / (PLFLT) height);
		y1 = 0.60 + (x1 - x0) * 0.5 * ((PLFLT) width / (PLFLT) height);
		xmean = (x0 + x1) / 2;
		ymean = (y0 + y1) / 2;

		switch (n) {
		case -1: //vpor for the title
			plvpor(0.02, 0.98, 0, y1);
			break;
		case 0: //main vpor
			plvpor(x0, x1, y0, y1);
			break;
		case 1: //top right of main
			plvpor(xmean, x1, ymean, y1);
			break;
		case 2: //top left of main
			plvpor(x0, xmean, ymean, y1);
			break;
		case 3: //bottom left of main
			plvpor(x0, xmean, y0, ymean);
			break;
		case 4: //bottom right of main
			plvpor(xmean, x1, y0, ymean);
			break;
		case 5: //col scale
			plvpor(x0 + 0.10, x1 - 0.10, 0.10, 0.15);
			break;
		case 6: //max col
			plvpor(x1 - 0.07, x1 - 0.03, 0.10, 0.15);
			break;
		case 7: //min col
			plvpor(x0 + 0.03, x0 + 0.07, 0.10, 0.15);
			break;
		case 8: //axes label
			plvpor(xmean, x1, y0 - 0.03, y0);
			break;
		}
	}
}


