// qpui_legend_options.cc
//
// Plot Interface Implementation - Legend Options
//
// Janet Barnes
// November 2004
//
// Tony Allen
// April 2008

#include "qpui.h"

#include <qtooltip.h>
#include <qcolordialog.h>

extern QPalette QPUI_normal_palette;

//     QpLegendTab CLASS
//     ---------------------


QpLegendTab::QpLegendTab(QTabWidget *tabWidget,  const QString& name, PanelItem * panel):
		QpTab(tabWidget, name, QP_LEGEND_TAB)
{
	// Comstructor

	// keep track of item which owns this tab
	I_Panel = panel; 

	create();

	connect( _show_legend_fld, SIGNAL(toggled(bool)), this, SLOT(setShowLegend(bool)) );
	connect( _legend_border_fld, SIGNAL(activated(int)), this, SLOT(setLegendBorder(int)) );
	connect( _legend_background_fld, SIGNAL(activated(int)), this, SLOT(setLegendBackground(int)) );
	connect(_legend_num_rows_fld, SIGNAL(valueChanged(int)), this, SLOT(setLegendNumRows(int)));
	connect(_legend_num_cols_fld,SIGNAL(valueChanged(int)), this, SLOT(setLegendNumCols(int)));
	connect(_legend_width_fld, SIGNAL(valueChanged(double)), this, SLOT(setLegendWidth(double)));
	connect(_legend_height_fld,SIGNAL(valueChanged(double)), this, SLOT(setLegendHeight(double)));
	connect(_legend_max_char_length_fld, SIGNAL(valueChanged(int)), this, SLOT(setLegendMaxCharLength(int)));
	connect(_legend_line_length_fld, SIGNAL(valueChanged(double)), this, SLOT(setLegendLineLength(double)));
	connect(_legend_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setLegendCharSize(double)));
	connect( _legend_just_fld, SIGNAL(activated(int)), this, SLOT(setLegendJust(int)) );

	connect( _legend_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setLegendFamily(int)) );
	connect( _legend_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setLegendStyle(bool)) );
	connect( _legend_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setLegendWeight(bool)) );
	connect( _legend_align_fld, SIGNAL(activated(int)), this, SLOT(setLegendAlign(int)) );
	connect( _legend_loffset_fld, SIGNAL(valueChanged(double)),this, SLOT(setLegendLoffset(double)) );
	connect( _legend_toffset_fld, SIGNAL(valueChanged(double)),this, SLOT(setLegendToffset(double)) );
}

QpLegendTab::QpLegendTab(QpLegendTab* tab, PanelItem* panel):
	QpTab((QTabWidget*)(tab->parent()), "Legend", QP_LEGEND_TAB)
{
	I_Panel = panel; 
	
	create();
	
	_show_legend_fld->setChecked(tab->_show_legend_fld->isChecked());
	_legend_border_fld->setCurrentIndex(tab->_legend_border_fld->currentIndex());
	_legend_background_fld->setCurrentIndex(tab->_legend_background_fld->currentIndex());
	_legend_num_rows_fld->setValue(tab->_legend_num_rows_fld->value());
	_legend_num_cols_fld->setValue(tab->_legend_num_cols_fld->value());
	_legend_char_size_fld->setValue(tab->_legend_char_size_fld->value());
	_legend_max_char_length_fld->setValue(tab->_legend_max_char_length_fld->value());
	_legend_line_length_fld->setValue(tab->_legend_line_length_fld->value());
	_legend_just_fld->setCurrentIndex(tab->_legend_just_fld->currentIndex());
	PLINT family, style, weight;
	tab->_legend_font_fld->getFontParams(family, style, weight);
	_legend_font_fld->setFontParams(family, style, weight);
	
	_legend_align_fld->setCurrentIndex(tab->_legend_align_fld->currentIndex());
	_legend_loffset_fld->setValue(tab->_legend_loffset_fld->value());
	_legend_toffset_fld->setValue(tab->_legend_toffset_fld->value());
	_legend_width_fld->setValue(tab->_legend_width_fld->value());
	_legend_height_fld->setValue(tab->_legend_height_fld->value());

	connect( _show_legend_fld, SIGNAL(toggled(bool)), this, SLOT(setShowLegend(bool)) );
	connect( _legend_border_fld, SIGNAL(activated(int)), this, SLOT(setLegendBorder(int)) );
	connect( _legend_background_fld, SIGNAL(activated(int)), this, SLOT(setLegendBackground(int)) );
	connect(_legend_num_rows_fld, SIGNAL(valueChanged(int)), this, SLOT(setLegendNumRows(int)));
	connect(_legend_num_cols_fld,SIGNAL(valueChanged(int)), this, SLOT(setLegendNumCols(int)));
	connect(_legend_width_fld, SIGNAL(valueChanged(double)), this, SLOT(setLegendWidth(double)));
	connect(_legend_height_fld,SIGNAL(valueChanged(double)), this, SLOT(setLegendHeight(double)));
	connect(_legend_max_char_length_fld, SIGNAL(valueChanged(int)), this, SLOT(setLegendMaxCharLength(int)));
	connect(_legend_line_length_fld, SIGNAL(valueChanged(double)), this, SLOT(setLegendLineLength(double)));
	connect(_legend_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setLegendCharSize(double)));
	connect( _legend_just_fld, SIGNAL(activated(int)), this, SLOT(setLegendJust(int)) );
	connect( _legend_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setLegendFamily(int)) );
	connect( _legend_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setLegendStyle(bool)) );
	connect( _legend_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setLegendWeight(bool)) );
	connect( _legend_align_fld, SIGNAL(activated(int)), this, SLOT(setLegendAlign(int)) );
	connect( _legend_loffset_fld, SIGNAL(valueChanged(double)),this, SLOT(setLegendLoffset(double)) );
	connect( _legend_toffset_fld, SIGNAL(valueChanged(double)),this, SLOT(setLegendToffset(double)) );
}

void QpLegendTab::create()
{
	QVBoxLayout *legend_top_layout = new QVBoxLayout(this);
	legend_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	legend_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QGroupBox *legend_properties_groupbox = new QGroupBox(this);

	legend_properties_groupbox->setTitle("Properties");

// 	QWidget* legend_properties_widget = new QWidget(legend_properties_groupbox, "legend_properties_widget");

	QVBoxLayout *legend_properties_layout = new QVBoxLayout;
	legend_properties_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	legend_properties_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_show_legend_fld = new QCheckBox("Show legend");
	_show_legend_fld->setChecked(false);
	legend_properties_layout->addWidget(_show_legend_fld, 0, Qt::AlignLeft);

	/*QGroupBox **/legend_box_groupbox = new QGroupBox(legend_properties_groupbox);
	legend_box_groupbox->setTitle("Box");


	QGridLayout *legend_box_layout = new QGridLayout;
	legend_box_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	legend_box_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* legend_border_label = new QLabel("Border: ");
	legend_box_layout->addWidget(legend_border_label, 1, 0, Qt::AlignRight);
	_legend_border_fld = new QComboBox();
	_legend_border_fld->insertItem(QPUI_NO_BORDER, "No border");
	_legend_border_fld->insertItem(QPUI_LINE, "Line");
	QFontMetrics legend_border_fm(_legend_border_fld->font());
	int legend_border_w = legend_border_fm.width("No border");
	int legend_border_h=legend_border_fm.height();
	_legend_border_fld->setMinimumSize(legend_border_w, legend_border_h);

	_legend_border_fld->setToolTip("Select style of legend border");
	legend_box_layout->addWidget(_legend_border_fld, 1, 1, Qt::AlignLeft);
	_legend_border_fld->setCurrentIndex(QPUI_LINE);

	QLabel* legend_background_label = new QLabel("Background: ");
	legend_box_layout->addWidget(legend_background_label, 1, 2, Qt::AlignRight);
	_legend_background_fld = new QComboBox();
	_legend_background_fld->insertItem(QPUI_TRANSPARENT, "Transparent");
	_legend_background_fld->insertItem(QPUI_OPAQUE, "Opaque");
	QFontMetrics legend_background_fm(_legend_background_fld->font());
	int legend_background_w = legend_background_fm.width("Transparent");
	int legend_background_h = legend_background_fm.height();
	_legend_background_fld->setMinimumSize(legend_background_w, legend_background_h);

	_legend_background_fld->setToolTip("Select legend background");
	legend_box_layout->addWidget(_legend_background_fld, 1, 3, Qt::AlignLeft);

	QLabel* num_rows_label = new QLabel("Number of rows: ");
	legend_box_layout->addWidget(num_rows_label, 3, 0, Qt::AlignRight);
	_legend_num_rows_fld = new QSpinBox;
	_legend_num_rows_fld->setMinimum(1);
	_legend_num_rows_fld->setFixedWidth(100);
	legend_box_layout->addWidget(_legend_num_rows_fld, 3, 1, Qt::AlignLeft);
		       
	QLabel* num_cols_label = new QLabel("Number of cols: ");
	legend_box_layout->addWidget(num_cols_label, 3, 2, Qt::AlignRight);
	_legend_num_cols_fld = new QSpinBox;
	_legend_num_cols_fld->setMinimum(1);
	_legend_num_cols_fld->setFixedWidth(100);
	legend_box_layout->addWidget(_legend_num_cols_fld, 3, 3, Qt::AlignLeft);

	QLabel* legend_width_label = new QLabel("Width fraction: ");
	legend_box_layout->addWidget(legend_width_label, 5, 0, Qt::AlignRight);
	_legend_width_fld = new FloatSpinBox(0.00, 40.00, 2, .05, legend_box_groupbox, "legend_width");
	_legend_width_fld->setFixedWidth(100);
	legend_box_layout->addWidget(_legend_width_fld, 5, 1, Qt::AlignLeft);
	_legend_width_fld->setValue(QPUI_LEGEND_WIDTH);
	
	QLabel* legend_height_label = new QLabel("Height fraction: ");
	legend_box_layout->addWidget(legend_height_label, 5, 2, Qt::AlignRight);
	_legend_height_fld = new FloatSpinBox(0.00, 40.00, 2, .05, legend_box_groupbox, "legend_height");
	_legend_height_fld->setFixedWidth(100);
	legend_box_layout->addWidget(_legend_height_fld, 5, 3, Qt::AlignLeft);
	_legend_height_fld->setValue(QPUI_LEGEND_HEIGHT);

	legend_box_layout->setRowStretch(0,0);
	legend_box_layout->setRowStretch(1,1);
	legend_box_layout->setRowStretch(2,0);
	legend_box_layout->setRowStretch(3,1);
	legend_box_layout->setRowStretch(4,0);
	legend_box_layout->setRowStretch(5,1);
	legend_box_layout->setRowStretch(6,0);

	legend_box_layout->setColumnStretch(0,0);
	legend_box_layout->setColumnStretch(1,0);
	legend_box_layout->setColumnStretch(2,0);
	legend_box_layout->setColumnStretch(3,0);
	legend_box_layout->setColumnStretch(4,1);

	legend_properties_layout->addWidget(legend_box_groupbox);
	legend_properties_layout->addSpacing(4);
	legend_box_groupbox->setEnabled(false);
	legend_box_groupbox->setLayout(legend_box_layout);

	legend_text_groupbox = new QGroupBox(legend_properties_groupbox);
	legend_text_groupbox->setTitle("Text");
	
	QGridLayout *legend_text_layout = new QGridLayout;
	legend_text_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	legend_text_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* legend_max_char_length_label = new QLabel("Max char length: ");
	legend_text_layout->addWidget(legend_max_char_length_label, 1, 0, Qt::AlignRight);
	_legend_max_char_length_fld = new QSpinBox;
	_legend_max_char_length_fld->setMinimum(5);
	_legend_max_char_length_fld->setMaximum(80);
	_legend_max_char_length_fld->setFixedWidth(100);
	legend_text_layout->addWidget(_legend_max_char_length_fld, 1, 1, Qt::AlignLeft);
	_legend_max_char_length_fld->setValue(30);

	QLabel* legend_line_length_label = new QLabel("Line length fraction: ");
	legend_text_layout->addWidget(legend_line_length_label, 1, 2, Qt::AlignRight);
	_legend_line_length_fld = new FloatSpinBox(0.1, 0.9, 2, .05, legend_text_groupbox, "legend_line_length");
	_legend_line_length_fld->setFixedWidth(100);
	legend_text_layout->addWidget(_legend_line_length_fld, 1, 3, Qt::AlignLeft);
	_legend_line_length_fld->setValue(0.2);

	QLabel* legend_char_size_label = new QLabel("Character size: ", legend_text_groupbox);
	legend_text_layout->addWidget(legend_char_size_label, 3, 0, Qt::AlignRight);
	_legend_char_size_fld = new FloatSpinBox(0.05, 40., 2, .05, legend_text_groupbox, "legend_char_size");
	_legend_char_size_fld->setFixedWidth(100);
	legend_text_layout->addWidget(_legend_char_size_fld, 3, 1, Qt::AlignLeft);
	_legend_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_LEGEND_CHAR_SIZE_ADD);
	
	QLabel* legend_just_label = new QLabel("Justification: ");
	legend_text_layout->addWidget(legend_just_label, 5, 0, Qt::AlignRight);
	_legend_just_fld = new QComboBox();
	_legend_just_fld->insertItem(QPUI_VCENTRE, "Centre");
	_legend_just_fld->insertItem(QPUI_TOP, "Top");
	_legend_just_fld->insertItem(QPUI_BOTTOM, "Bottom");
	QFontMetrics legend_just_fm(_legend_just_fld->font());
	int legend_just_w = legend_just_fm.width("centre");
	int legend_just_h = legend_just_fm.height();
	_legend_just_fld->setMinimumSize(legend_just_w, legend_just_h);

	_legend_just_fld->setToolTip("Select legend text justification");
	legend_text_layout->addWidget(_legend_just_fld, 5, 1, Qt::AlignLeft);

	QLabel* legend_font_label = new QLabel("Font: ");
	legend_text_layout->addWidget(legend_font_label, 5, 2, Qt::AlignRight);
	_legend_font_fld = new FontButton;

	_legend_font_fld->setToolTip("Select legend text font");
	legend_text_layout->addWidget(_legend_font_fld, 5, 3, Qt::AlignLeft);

	legend_text_layout->setRowStretch(0,0);
	legend_text_layout->setRowStretch(1,1);
	legend_text_layout->setRowStretch(2,0);
	legend_text_layout->setRowStretch(3,1);
	legend_text_layout->setRowStretch(4,0);
	legend_text_layout->setRowStretch(5,1);
	legend_text_layout->setRowStretch(6,0);

	legend_text_layout->setColumnStretch(0,0);
	legend_text_layout->setColumnStretch(1,0);
	legend_text_layout->setColumnStretch(2,0);
	legend_text_layout->setColumnStretch(3,0);
	legend_text_layout->setColumnStretch(4,1);

	legend_text_groupbox->setLayout(legend_text_layout);
	
	legend_properties_layout->addWidget(legend_text_groupbox);
	legend_properties_layout->addSpacing(4);
	
	legend_properties_groupbox->setLayout(legend_properties_layout);
	
	legend_top_layout->addWidget(legend_properties_groupbox);
	legend_text_groupbox->setEnabled(false);

	legend_position_groupbox=new QGroupBox(this);

	legend_position_groupbox->setTitle("Legend Position");

	QGridLayout *legend_position_layout = new QGridLayout;
	legend_position_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	legend_position_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* legend_align_label = new QLabel("Alignment: ");
	legend_position_layout->addWidget(legend_align_label, 1, 0, Qt::AlignRight);
	_legend_align_fld = new QComboBox();
	_legend_align_fld->insertItem(QPUI_INSIDE_TOP_RIGHT, "Inside top right");
	_legend_align_fld->insertItem(QPUI_INSIDE_TOP_LEFT, "Inside top left");
	_legend_align_fld->insertItem(QPUI_INSIDE_BOTTOM_RIGHT, "Inside bottom right");
	_legend_align_fld->insertItem(QPUI_INSIDE_BOTTOM_LEFT, "Inside bottom left");
	_legend_align_fld->insertItem(QPUI_ABOVE_TOP_CENTRE, "Above top centre");
	_legend_align_fld->insertItem(QPUI_OUTSIDE_TOP_RIGHT, "Outside top right");
	_legend_align_fld->insertItem(QPUI_USER_DEFINED, "User defined");
	QFontMetrics legend_align_fm(_legend_align_fld->font());
	int legend_align_w = legend_align_fm.width("Inside bottom right");
	int legend_align_h = legend_align_fm.height();
	_legend_align_fld->setMinimumSize(legend_align_w, legend_align_h);

	_legend_align_fld->setToolTip("Select legend box alignment");
	legend_position_layout->addWidget(_legend_align_fld,1,1,Qt::AlignLeft);
	_legend_align_fld->setCurrentIndex(QPUI_INSIDE_TOP_RIGHT);

	QLabel* legend_loffset_label = new QLabel("Left offset: ");
	legend_position_layout->addWidget(legend_loffset_label, 3, 0, Qt::AlignRight);
	_legend_loffset_fld = new FloatSpinBox( -40., 40., 2, .05, legend_position_groupbox, "legend_loffset");
	_legend_loffset_fld->setFixedWidth(100);
	_legend_loffset_fld->setFocusPolicy(Qt::StrongFocus);
	legend_position_layout->addWidget(_legend_loffset_fld, 3, 1, Qt::AlignLeft);
	_legend_loffset_fld->setValue(0.75);

	QLabel* legend_toffset_label = new QLabel("Top offset: ");
	legend_position_layout->addWidget(legend_toffset_label, 3, 2, Qt::AlignRight);
	_legend_toffset_fld = new FloatSpinBox(-40., 40., 2, .05, legend_position_groupbox, "legend_toffset");
	_legend_toffset_fld->setFixedWidth(100);
	_legend_toffset_fld->setFocusPolicy(Qt::StrongFocus);
	legend_position_layout->addWidget(_legend_toffset_fld, 3, 3, Qt::AlignLeft);
	_legend_toffset_fld->setValue(0.0);

	_legend_loffset_fld->setEnabled(false);
	_legend_toffset_fld->setEnabled(false);

	legend_position_layout->setRowStretch(0,0);
	legend_position_layout->setRowStretch(1,0);
	legend_position_layout->setRowStretch(2,0);
	legend_position_layout->setRowStretch(3,0);
	legend_position_layout->setRowStretch(4,1);

	legend_position_layout->setColumnStretch(0,0);
	legend_position_layout->setColumnStretch(1,0);
	legend_position_layout->setColumnStretch(2,0);
	legend_position_layout->setColumnStretch(3,0);
	legend_position_layout->setColumnStretch(4,1);
	
	legend_position_groupbox->setLayout(legend_position_layout);

	legend_top_layout->addWidget(legend_position_groupbox);
	legend_top_layout->addSpacing(4);	       legend_top_layout->addStretch(10);
	legend_position_groupbox->setEnabled(false);
}

// ---------------------------------------------

void QpLegendTab::setShowLegend(bool choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setShowLegend " << endl;

	legend_box_groupbox->setEnabled(choice);
	legend_text_groupbox->setEnabled(choice);
	legend_position_groupbox->setEnabled(choice);
   
	updateNumTraces();
   
    // only tell list view to make changes to selected trace items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)

    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(SHOW_LEGEND, 0, 0.0, "", choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendBorder(int choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendBorder " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_BORDER, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendBackground(int choice)
{
     if( QuSlotDebug() ) cout << " QpLegendTab::setLegendBackground " << endl;
  // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_BACKGROUND, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendNumRows(int choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendNumRows " << endl;

	updateNumTraces();
	
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_NUM_ROWS, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendNumCols(int choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendNumCols " << endl;
	updateNumTraces();
	
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_NUM_COLS, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendWidth(double choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendWidth " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   setLegendAlign(_legend_align_fld->currentIndex());
   
    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_WIDTH, 0, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendHeight(double choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendHeight " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   setLegendAlign(_legend_align_fld->currentIndex());
   
    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_HEIGHT, 0, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendMaxCharLength(int choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendMaxCharLength " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_MAX_CHAR_LENGTH, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendLineLength(double choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendLineLength " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_LINE_LENGTH, 0, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendCharSize(double choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendCharSize " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_CHAR_SIZE, 0, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendJust(int choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendJust " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_JUST, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendFamily(int choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendFamily " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_FONT_FAMILY, choice);
}

void QpLegendTab::setLegendStyle(bool choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendStyle " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_FONT_STYLE,0,0.,"", choice);
}

void QpLegendTab::setLegendWeight(bool choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendWeight " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_FONT_WEIGHT,0,0.,"", choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendAlign(int choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendAlign " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   	// calculate loffset and toffset based on alignment
	float legend_width = (float) _legend_width_fld->value();
	float legend_height = (float) _legend_height_fld->value();
	
	switch(choice){
	
		case QPUI_INSIDE_TOP_RIGHT:
			_legend_loffset_fld->setValue(1.0 - legend_width);
			_legend_toffset_fld->setValue(0.0);
			_legend_loffset_fld->setEnabled(false);
			_legend_toffset_fld->setEnabled(false);
			break;
		case QPUI_INSIDE_TOP_LEFT:
			_legend_loffset_fld->setValue(0.0);
			_legend_toffset_fld->setValue(0.0);
			_legend_loffset_fld->setEnabled(false);
			_legend_toffset_fld->setEnabled(false);
			break;
		case QPUI_INSIDE_BOTTOM_RIGHT:
			_legend_loffset_fld->setValue(1.0 - legend_width);
			_legend_toffset_fld->setValue(1.0 - legend_height);
			_legend_loffset_fld->setEnabled(false);
			_legend_toffset_fld->setEnabled(false);
			break;
		case QPUI_INSIDE_BOTTOM_LEFT:
			_legend_loffset_fld->setValue(0.0);
			_legend_toffset_fld->setValue(1.0 - legend_height);
			_legend_loffset_fld->setEnabled(false);
			_legend_toffset_fld->setEnabled(false);
			break;
		case QPUI_ABOVE_TOP_CENTRE:
			_legend_loffset_fld->setValue(0.5 - 0.5 * legend_width);
			_legend_toffset_fld->setValue(- legend_height);
			_legend_loffset_fld->setEnabled(false);
			_legend_toffset_fld->setEnabled(false);
			break;
		case QPUI_OUTSIDE_TOP_RIGHT:
			_legend_loffset_fld->setValue(1.0);
			_legend_toffset_fld->setValue(0.0);
			_legend_loffset_fld->setEnabled(false);
			_legend_toffset_fld->setEnabled(false);
			break;
		case QPUI_USER_DEFINED:
			_legend_loffset_fld->setEnabled(true);
			_legend_toffset_fld->setEnabled(true);
			break;
	}

    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_ALIGN, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendLoffset(double choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendLoffset " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_L_OFFSET, 0, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendToffset(double choice)
{
    if( QuSlotDebug() ) cout << " QpLegendTab::setLegendToffset " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Panel->Tree()->setLegendOption(LEGEND_T_OFFSET, 0, choice);
}


// -------------------------------------

void QpLegendTab::updateNumTraces()
{
	if(QuSlotDebug()) cout << "QpLegendTab::updateNumTraces " << endl;

	// updates number of rows when trace number changed  
	int num_traces = I_Panel->numberTraces();
	if(num_traces > 0)
	{
		int num_rows = (int)ceil((float)num_traces / _legend_num_cols_fld->value());
		_legend_num_rows_fld->setValue(num_rows);
	}
	
}

QDomElement QpLegendTab::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Legend_Tab");
    xml.appendChild(createTextXMLNode(doc, "Show_Legend", _show_legend_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Border", _legend_border_fld->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "Background", _legend_background_fld->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "Num_Rows", _legend_num_rows_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Num_Cols", _legend_num_cols_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Char_Size", _legend_char_size_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Max_Length", _legend_max_char_length_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Line_Length", _legend_line_length_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Justification", _legend_just_fld->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "Alignment", _legend_align_fld->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "Left_Offset", _legend_loffset_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Top_Offset", _legend_toffset_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Width", _legend_width_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Height", _legend_height_fld->value()));

    PLINT family, style, weight;
    _legend_font_fld->getFontParams(family, style, weight);
    xml.appendChild(createTextXMLNode(doc, "Font_Family", family));
    xml.appendChild(createTextXMLNode(doc, "Font_Style", style));
    xml.appendChild(createTextXMLNode(doc, "Font_Weight", weight));
    
    return xml;
}

void QpLegendTab::fromXMLElement(QDomElement& el)
{
    QString s;
    
    if(getXMLNodeText(el, "Show_Legend", s))
    {
        _show_legend_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Border", s))
    {
        _legend_border_fld->setCurrentIndex(s.toInt());
    }
    if(getXMLNodeText(el, "Background", s))
    {
        _legend_background_fld->setCurrentIndex(s.toInt());
    }
    if(getXMLNodeText(el, "Num_Rows", s))
    {
        _legend_num_rows_fld->setValue(s.toInt());
    }
    if(getXMLNodeText(el, "Num_Cols", s))
    {
        _legend_num_cols_fld->setValue(s.toInt());
    }
    if(getXMLNodeText(el, "Char_Size", s))
    {
        _legend_char_size_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Max_Length", s))
    {
        _legend_max_char_length_fld->setValue(s.toInt());
    }
    if(getXMLNodeText(el, "Line_Length", s))
    {
        _legend_line_length_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Justification", s))
    {
        _legend_just_fld->setCurrentIndex(s.toInt());
    }
    if(getXMLNodeText(el, "Alignment", s))
    {
        _legend_align_fld->setCurrentIndex(s.toInt());
    }
    if(getXMLNodeText(el, "Left_Offset", s))
    {
        _legend_loffset_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Top_Offset", s))
    {
        _legend_toffset_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Width", s))
    {
        _legend_width_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Height", s))
    {
        _legend_height_fld->setValue(s.toDouble());
    }

    int family(PL_FCI_SANS), style(0), weight(0);
    if(getXMLNodeText(el, "Font_Family", s))
    {
        family=s.toInt();
    }
    if(getXMLNodeText(el, "Font_Size", s))
    {
        style=s.toInt();
    }
    if(getXMLNodeText(el, "Font_Weight", s))
    {
        weight=s.toInt();
    }
    switch(family)
    {
        case PL_FCI_SERIF: _legend_font_fld->setSerifFont(); break;
        case PL_FCI_MONO: _legend_font_fld->setMonotypeFont(); break;
        case PL_FCI_SCRIPT: _legend_font_fld->setDecorativeFont(); break;
        default: _legend_font_fld->setSansFont(); break;
    }
    _legend_font_fld->setItalic(style==PL_FCI_ITALIC);
    _legend_font_fld->setBold(weight==PL_FCI_BOLD);
    
}
