
#include <stdio.h>
#include <math.h>

#ifdef HAVE_IEEEFP_H
 #include <ieeefp.h>
#endif

#include <qglobal.h>
#include <qapplication.h>
#include <qmainwindow.h>
#include <qpalette.h>
#include <qlineedit.h>
#include <qinputdialog.h>
#include <qstatusbar.h>
#include <qregexp.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qdatetime.h> 
#include <qstring.h> 
#include <qwidget.h>
#include <qcheckbox.h>

#include "Xrefs.h"
#include "qdutil.h"
#include "DvUnit.h"
#include "qwui.h"
#include "qprofile.h"

using namespace QSAS;
 qwuiWindow* qsasWindow;

// This is the actual DVOS data object list (Working List)
DvList Wlist;

static double CLOSE_ENOUGH = 1.e-8;
int qsas_restore_release;
double _PI = 4.*atan(1.0);

// ------------------------------------------

int QuEpochType(DvObject_var &obj){
	
	// It is not obvious we ever get epoch types now as we convert to DvTime
	
	if(obj.is_nil()) return 0;
	
	if( !obj->xref_exists(SI_CONVERSION) ) return 0;
	
	DvObject_var attr = obj->get_xref(SI_CONVERSION);
	
	if(attr->not_str()) return 0;
	
	string base_str1 = DvUnitGetBaseSI(attr->str().c_str());
	string base_str2 = DvUnitGetBaseSI("1.e-3>s");

	if(base_str1 == base_str2) return 1;

	string base_str3 = DvUnitGetBaseSI("1.e-12>s");
	
	if(base_str1 == base_str3) return 2;

	return false;
	
}


bool QuIsMatch(double value, double fill){
	// used by Filters in Calc and Filter plugins
	// Does not check strip toggles on main window

    if( fill == 0 && value == fill) return true;
    if( fill == -1 && value == fill) return true;
    else return isFill(value, fill);
}


bool QuSlotDebug(){

	return (QSAS_profiles->current()->Debug);
}


string QuCommonLabl(string str1, string str2){
	string commonStr="";
	string units="";
	string units2="";
  
	// remove mod signs "|"
	size_t pos = str1.find("|");
	if(pos != string::npos) {
    	str1 = str1.substr(pos+1, str1.size() - pos -1);
		pos = str1.find("|");
		if(pos != string::npos) {
       	str1 = str1.substr(0,pos);
		}
  	}
  
	pos = str2.find("|");
	if(pos != string::npos) {
    	str2 = str2.substr(pos+1, str2.size() - pos -1);
		pos = str2.find("|");
		if(pos != string::npos) {
      	 str2 = str2.substr(0,pos);
		}
  	}

	pos = 0;
	
	// remove and keep units for later
    for(int i=0; i<str2.size(); i++){
		if(str2[i] == '(') 
		{
			pos = i;
			do
			{
				units2 += str2[i];
				if (str2[i] == ')' ) break;
				i++;
			}
			while(i < str2.size());
			break;
		}
	}
	if(pos > 0) str2 = str2.substr(0,pos);

    for(int i=0; i<str1.size(); i++){
		if(str1[i] == '(') 
		{
			pos = i;
			do
			{
				units += str1[i];
				if (str1[i] == ')' ) break;
				i++;
			}
			while(i < str1.size());
			break;
		}
	}
	if(pos > 0) str1 = str1.substr(0,pos);

  if (str1 == str2) 
  {
  	if(units == units2) str1 += units;
  	else if(units == "") str1 += units2;
  	else if(units2 == "") str1 += units;
  	return str1;
  }
  
  // break into stem and component
/*  pos =  str2.find("_");
  if(pos != string::npos) {
    string s1 = str2.substr(0,pos);
    string s2 = str2.substr(pos+1, str2.size() - pos -1);
	pos = s2.find(" ");
	if(pos != string::npos) s2 = s2.substr(0,pos);
	
	pos = str1.find(s1);
	if(pos != string::npos) commonStr = s1;
	pos = str1.find(s2);
	if(pos != string::npos) commonStr = s2;
  }
  else */
  {
    for(int i=0; i<str1.size(); i++){
	// handle escaped characters first
	if( str1[i] == '#' )
	{
		if( strncmp(&(str1[i]), &(str2[i]), 3) != 0 ) break;
		else {
			commonStr.append(&(str1[i]), 3);
			i+=2; // add only 2 as i++ adds one
			continue;
		}
	}
	
	// if we get here char was not escaped
	if(str1[i] == str2[i] ) commonStr += str1[i];
	  else break;
	}
  }
  // remove leading and trailing '-'
  if(commonStr[0] == '_') commonStr = commonStr.substr(1);
  if(commonStr[commonStr.size()-1] == '_') commonStr = commonStr.substr(0, commonStr.size()-1);

	if(units == units2) commonStr += units;
  	else if(units == "") commonStr += units2;
  	else if(units2 == "") commonStr += units;
  
  return commonStr;
}



 /***********************************************************************/

dimTripleStack  getDimTripleStack(const char* Txt){

   DvString compTxt(Txt);
  
   dimTripleStack arrayTriples;
   compTxt = compTxt.after("["); // remove "[" 

   bool notEmpty = true;
   while (notEmpty){
     dimTriple nextDim = getDimTriple(compTxt);
	 if (nextDim.size() == 3) arrayTriples.push_back(nextDim);
	 else notEmpty = false;
   }

   return arrayTriples;
 }


dimTriple  getDimTriple(DvString &compTxt){

   dimTriple newTriple;

   // get the next dimension only and cut from input str

   DvString thisDim = compTxt.before(")"); // deep copy of first n=ptr characters
   if(thisDim.empty()) return newTriple;
   
   compTxt = compTxt.after(")"); // remove ")" for next call
   
    int option = 0;  // default is elements (or slice done below)
    if( thisDim.contains("Sum") ){
        option = 1;   // sum
        thisDim = thisDim.after("Sum");
    }
    else if(thisDim.contains("Ave") ){
        option = 3;   // ave
        thisDim = thisDim.after("Ave");
    }
    else if(thisDim.contains("AtT") ){
        option = 4;   // alias to time
        thisDim = thisDim.after("AtT");
    }
	  

   thisDim = thisDim.after("("); // // remove " ("
   if(thisDim.empty() ) return newTriple; // no more dimensions, newTriple will have size()< 3

   DvString token = thisDim.front(",");
   thisDim = thisDim.after(",");
   int start;
   int stop;
   if (thisDim.empty()) {
     // one index, slice requested
     start = token.toInt();
	 stop = start;
	 option = 2;
     newTriple.push_back(option);
     newTriple.push_back(start);
     newTriple.push_back(stop);
     
   }
   else{
     start = token.toInt();
     stop = thisDim.toInt();
	 newTriple.push_back(option);
     newTriple.push_back(start);
     newTriple.push_back(stop);
   }
   return newTriple;

}


dimTriple getSliceTriple(DvString &compTxt)
{
    if ( QuSlotDebug() ) printf( "getSliceTriple this = %s \n", compTxt.c_str() );

    dimTriple sliceTriple;
    int option = 0; // default is subset
    int start;
    int stop;

    compTxt = compTxt.after(" <");
    if(compTxt.empty()) return sliceTriple;  // no T slice section found, sliceTriple is empty vector
    
    compTxt = compTxt.before(">"); // remove ">"

    if(compTxt.contains("Ave")) {
        option = 1;   // average
        compTxt = compTxt.after("Ave");
    }

    DvString token = compTxt.front(",");
    compTxt = compTxt.after(",");
    
    if (compTxt.empty()) {
      // one index, slice requested
        start = token.toInt();
	    stop = start;
	    option = 2;
	    sliceTriple.push_back(option);
	    sliceTriple.push_back(start);
	    sliceTriple.push_back(stop);
	  
    }
    else{
        start = token.toInt();
        stop = compTxt.toInt();
        
        sliceTriple.push_back(option);
        sliceTriple.push_back(start);
	    sliceTriple.push_back(stop);
  }

  return sliceTriple;
 }

DvString compText(const char* objTxt)
{
   DvString compTxt(objTxt); 
   DvString comp = compTxt.betweenIncl('[', ']');
   DvString slice = compTxt.betweenIncl('<', '>');
   
   compTxt = comp;
   compTxt += " ";
   compTxt += slice;

   return compTxt.trimmed();
}


DvString baseObjectText(const char* objTxt)
{
   DvString baseTxt(objTxt);
   baseTxt = baseTxt.front(" [");
   baseTxt = baseTxt.front(" <");
   
   return baseTxt.trimmed();
}

// assemble component change string
// unopened ) indicates unchanged comp
//
DvString getCompChange(const char *oldComp, const char *newComp)
{
	DvString compChange("");
	
	DvString oldC = oldComp;
	DvString newC = newComp;
	
	if(newC.contains(R_STR_CMP)) compChange += R_STR_CMP;
	else if(newC.contains(X_STR_CMP)) compChange += X_STR_CMP;
	else if(newC.contains(Y_STR_CMP)) compChange += Y_STR_CMP;
	else if(newC.contains(Z_STR_CMP)) compChange += Z_STR_CMP;
	else if(newC.contains(Ld_STR_CMP)) compChange += Ld_STR_CMP;
	else if(newC.contains(Lr_STR_CMP)) compChange += Lr_STR_CMP;
	else if(newC.contains(Td_STR_CMP)) compChange += Td_STR_CMP;
	else if(newC.contains(Tr_STR_CMP)) compChange += Tr_STR_CMP;
	else if(newC.contains(Pd_STR_CMP)) compChange += Pd_STR_CMP;
	else if(newC.contains(Pr_STR_CMP)) compChange += Pr_STR_CMP;
	else if(newC.contains(Lt_STR_CMP)) compChange += Lt_STR_CMP;
	else if(newC.contains(T_STR_START)) compChange += T_STR_START;
	else if(newC.contains(T_STR_CENTRE)) compChange += T_STR_CENTRE;
	else if(newC.contains(T_STR_END)) compChange += T_STR_END;
	else if(newC.contains(T_STR_0_START)) {
		compChange += T_STR_0_START;
		return compChange; // <> included
	}
	else if(newC.contains("=>")) {
		compChange = newComp;
		return compChange; // can't do anything else with metadata yet
	}
	else{
		DvString oldArrC = oldC.between('[', ']').trimmed();
		DvString newArrC = newC.between('[', ']').trimmed();
			
		if( oldArrC != newArrC){
			compChange += " [ ";
			DvString oldToken = oldArrC.before(')').trimmed();
			DvString newToken = newArrC.before(')').trimmed();
			oldArrC = oldArrC.after(')');
			newArrC = newArrC.after(')');
		
			while( !oldToken.empty() ){
				if(oldToken != newToken ) compChange += newToken;
				compChange += ") "; // add this for both changed and unchanged comp
		
				// find next
				oldToken = oldArrC.before(')').trimmed();
				newToken = newArrC.before(')').trimmed();
				oldArrC = oldArrC.after(')');
				newArrC = newArrC.after(')');
			}
			compChange += " ] ";
		}
	}
	
	// time component
	DvString oldTimeC = oldC.betweenIncl('<', '>');
	DvString newTimeC = newC.betweenIncl('<', '>');
	
	if( oldTimeC != newTimeC){
		compChange += newTimeC;
	}
 
   
   return compChange;
}

// assemble new component from change string and old comp
// unopened ) indicates unchanged comp
//
DvString getNewComp(const char *oldComp, const char *changes)
{
	DvString changedComp("");
	
	DvString oldC = oldComp;
	DvString newC = changes;
	
	if(newC.contains(R_STR_CMP)) changedComp += R_STR_CMP;
	else if(newC.contains(X_STR_CMP)) changedComp += X_STR_CMP;
	else if(newC.contains(Y_STR_CMP)) changedComp += Y_STR_CMP;
	else if(newC.contains(Z_STR_CMP)) changedComp += Z_STR_CMP;
	else if(newC.contains(Ld_STR_CMP)) changedComp += Ld_STR_CMP;
	else if(newC.contains(Lr_STR_CMP)) changedComp += Lr_STR_CMP;
	else if(newC.contains(Td_STR_CMP)) changedComp += Td_STR_CMP;
	else if(newC.contains(Tr_STR_CMP)) changedComp += Tr_STR_CMP;
	else if(newC.contains(Pd_STR_CMP)) changedComp += Pd_STR_CMP;
	else if(newC.contains(Pr_STR_CMP)) changedComp += Pr_STR_CMP;
	else if(newC.contains(Lt_STR_CMP)) changedComp += Lt_STR_CMP;
	else if(newC.contains(T_STR_START)) changedComp += T_STR_START;
	else if(newC.contains(T_STR_CENTRE)) changedComp += T_STR_CENTRE;
	else if(newC.contains(T_STR_END)) changedComp += T_STR_END;
	else if(newC.contains(T_STR_0_START)) {
		changedComp += T_STR_0_START;
		return changedComp; // <> included
	}
	else if(newC.contains("=>")) {
		changedComp = changes;
		return changedComp; // can't do anything else with metadata yet
	}
	else{	
		DvString oldArrC = oldC.between('[', ']').trimmed();
		DvString newArrC = newC.between('[', ']').trimmed();

		if( newArrC.empty()){
			changedComp += " [ ";
			changedComp += oldArrC;
			changedComp += " ] ";
		}
		else{
			changedComp += " [ ";
			DvString oldToken = oldArrC.before(')').trimmed();
			DvString newToken = newArrC.before(')').trimmed();
			oldArrC = oldArrC.after(')');
			newArrC = newArrC.after(')');
		
			while( !oldToken.empty() ){
				if( newToken.empty() ) changedComp += oldToken;
				else changedComp += newToken.c_str();
				changedComp += ") "; // add this for both changed and unchanged comp

				// find next
				oldToken = oldArrC.before(')').trimmed();
				newToken = newArrC.before(')').trimmed();
				oldArrC = oldArrC.after(')');
				newArrC = newArrC.after(')');
			}
			changedComp += " ] ";
		}
	}
	
	// time component
	DvString oldTimeC = oldC.betweenIncl('<', '>');
	DvString newTimeC = newC.betweenIncl('<', '>');
	
	changedComp += ' ';
	if( newTimeC.empty())	changedComp += oldTimeC;
	else changedComp += newTimeC;
 
   
   return changedComp;
}



/// \fn QguComp compEntry(const char* cmpTxt, const char* cmpType)
/// \brief Method that returns the component entry in the dropdown box from the type of component and the component text
/// \param     cmpTxt of type const char*
/// \param     cmpType of type const char*
/// \exception None
/// \return    comp of type QguComp
///

QguComp compEntry(const char* cmpTxt, const char* cmpType)
{
  QguComp comp=UNKNOWN_COMP;
  if(strcmp(cmpType,"VECTOR") ==0)
  {
    if(strcmp(cmpTxt,R_STR_CMP)==0)
      comp=R_COMP;
    else if(strcmp(cmpTxt,X_STR_CMP)==0)
      comp=X_COMP;
    else if(strcmp(cmpTxt,Y_STR_CMP)==0)
      comp=Y_COMP;
    else if(strcmp(cmpTxt,Z_STR_CMP)==0)
      comp=Z_COMP;
    else if(strcmp(cmpTxt,R_STR_CMP)==0)
      comp=R_COMP;
    else if(strcmp(cmpTxt,Ld_STR_CMP)==0)
      comp=LAT_DEG_COMP;
    else if(strcmp(cmpTxt,Td_STR_CMP)==0)
      comp=THETA_DEG_COMP;
    else if(strcmp(cmpTxt,Pd_STR_CMP)==0)
      comp=PHI_DEG_COMP;
    else if(strcmp(cmpTxt,Lr_STR_CMP)==0)
      comp=LAT_RAD_COMP;
    else if(strcmp(cmpTxt,Tr_STR_CMP)==0)
      comp=THETA_RAD_COMP;
    else if(strcmp(cmpTxt,Pr_STR_CMP)==0)
      comp=PHI_RAD_COMP;
    else
      comp=UNKNOWN_COMP;
  }
  else if(strcmp(cmpType,"TIME") ==0)
  {
    if (strcmp(cmpTxt,T_STR_START)==0)
      comp=T_START_COMP;
    else if (strcmp(cmpTxt,T_STR_CENTRE)==0)
       comp=T_CENTRE_COMP;
    else if (strcmp(cmpTxt,T_STR_END)==0)
      comp=T_END_COMP;
    else
      comp=UNKNOWN_COMP;
  }
  else if(strcmp(cmpType,"ARRAY") ==0)
    comp=ARRAY_COMP;
  else if(strcmp(cmpType,"XREF")==0)
    comp=XREF_OBJ;
  else if(strcmp(cmpType,"SLICE") ==0)
    comp=SLICE_COMP;
  else
    comp=UNKNOWN_COMP;

  return comp;
}

/// \fn QguComp compEntry(const char* cmpTxt)
/// \brief Method that returns the component entry in the dropdown box from the component text
/// \param     cmpTxt of type const char*
/// \exception None
/// \return    comp of type QguComp
///

QguComp compEntry(const char* cmpTxt)
{
  QguComp comp=UNKNOWN_COMP;
  if(strcmp(cmpTxt,R_STR_CMP)==0)
    comp=R_COMP;
  else if(strcmp(cmpTxt,X_STR_CMP)==0)
    comp=X_COMP;
  else if(strcmp(cmpTxt,Y_STR_CMP)==0)
    comp=Y_COMP;
  else if(strcmp(cmpTxt,Z_STR_CMP)==0)
    comp=Z_COMP;
  else if(strcmp(cmpTxt,R_STR_CMP)==0)
    comp=R_COMP;
  else if(strcmp(cmpTxt,Ld_STR_CMP)==0)
    comp=LAT_DEG_COMP;
  else if(strcmp(cmpTxt,Td_STR_CMP)==0)
    comp=THETA_DEG_COMP;
  else if(strcmp(cmpTxt,Pd_STR_CMP)==0)
    comp=PHI_DEG_COMP;
  else if(strcmp(cmpTxt,Lr_STR_CMP)==0)
    comp=LAT_RAD_COMP;
  else if(strcmp(cmpTxt,Tr_STR_CMP)==0)
    comp=THETA_RAD_COMP;
  else if(strcmp(cmpTxt,Pr_STR_CMP)==0)
    comp=PHI_RAD_COMP;
  else if (strcmp(cmpTxt,T_STR_START)==0)
    comp=T_START_COMP;
  else if (strcmp(cmpTxt,T_STR_CENTRE)==0)
   comp=T_CENTRE_COMP;
  else if (strcmp(cmpTxt,T_STR_END)==0)
    comp=T_END_COMP;
  else if (strncmp(cmpTxt, " [=>",4) == 0)
    comp=XREF_OBJ;
  else if (strncmp(cmpTxt, " [",2)==0)
    comp=ARRAY_COMP;
  else if (strncmp(cmpTxt, " <",2)==0)
    comp=SLICE_COMP;
  else
    comp=WL_OBJ;
  return comp;
}


/***********************************************************************/

void QuDeleteObject(const char* name){

	Wlist.remove(DvString(name));    
		
	return;
}

/***********************************************************************/

void QuReplaceObject(const char* name, DvObject_var &dobj){
	
	// safe against old object not existing
	DvNode *node = Wlist.find(name);
	if (node == NULL )    {
        Wlist.append(DvString(name), dobj);
	}
	else node->setObj(dobj);
  
 return;    

}

/***********************************************************************/

void QuRenameObject(const char* oldName, const char* newName){
  
	DvNode *node = Wlist.find(oldName);
	if (node == NULL )  return;

	DvString name(newName);
	node->setName(name);

}

//*********************************************************

bool QuSetObjByName(DvObject_var ToSetObj, const char* name){
  
	DvString objName(name);

    // Find if objects are to be cleaned before putting on WL

	if ( qsasWindow->StripBadData->isChecked() || qsasWindow->StripFills->isChecked() ){
        
		ToSetObj = ToSetObj->cleanObject();
	}

    if( ToSetObj.is_nil() ) {
		QuMsgBar("Empty object: browse object to see errors (double click)", true);
	}
	
	// ensure vectors are cartesian
	DvString rep = ToSetObj->getRep();
	if(rep == "RTP"){
		ToSetObj = ToSetObj->getVecFromRTP();
	}
	else if(rep == "RLP"){
		ToSetObj = ToSetObj->getVecFromRLP();
	}
	
  	DvNode *node = Wlist.find(objName);
	if (node == NULL )  {
		Wlist.append(objName, ToSetObj);
	}
	else{

		node->setObj(ToSetObj);
	}
	 
    DvString folder = objName.front("/");
	objName = objName.back("/");
	ToSetObj->change_xref("ObjectName", objName);
   
	// catch old qsas created time tags
	if(ToSetObj->xref_exists("TIME_TAGS") && !ToSetObj->xref_exists(DEPEND_0))
	{
		DvObject_var tt = ToSetObj->get_xref("TIME_TAGS");
		ToSetObj->change_xref(DEPEND_0, tt);
		ToSetObj->delete_xref("TIME_TAGS");
	}

    if(!ToSetObj->xref_exists(DEPEND_0) && ToSetObj->not_time() && folder != "Temp" && ToSetObj->seqSize() > 1){
        DvObject_var dep0 = new DvObject((int)0, ToSetObj->seqSize());
        dep0->assign_regular(1);
        ToSetObj->change_xref(DEPEND_0, dep0);
    }
    
   return true;
   

}



/***********************************************************************/

bool QuIsBadData(double value, double fill){
  if( qsasWindow->StripFills->isChecked()){
    if( fill == 0 && value == fill) return true;
    else if ( isFill(value, fill) ) return true;
  }

  if(qsasWindow->StripBadData->isChecked() ){
    if ( !isfinite(value) ) return true; // finds NaN, Infinity, -Infinity
  }

  return false;
}

/***********************************************************************/

bool QuIsBad(double value, double fill){

	// base level test, does not ask if strip is set
	if( fill == 0 && value == fill) return true;
	else if ( isFill(value, fill) ) return true;
  
	if ( !isfinite(value) ) return true; // finds NaN, Infinity, -Infinity

	return false;
}

/***********************************************************************/

void QuMsgBar(const char* text, bool error){

  if(qsasWindow==NULL) return;
  
  if(error)
  {
	  QPalette palette;
	  palette.setColor(qsasWindow->statusBar()->foregroundRole(), QColor("#ff0000"));
	  qsasWindow->statusBar()->setPalette(palette);
  }
  else
  {
	  QPalette palette;
	  palette.setColor(qsasWindow->statusBar()->foregroundRole(), qApp->palette().color(QPalette::WindowText));
	  qsasWindow->statusBar()->setPalette(palette);
  }
  
  qsasWindow->statusBar()->showMessage(text);

}

/***********************************************************************/

void QuMsgBar(const char* text1, const char* text2, bool error){
  static QString SStxt;

  SStxt = text1;
  SStxt += " ";
  SStxt += text2;
  QuMsgBar(SStxt.toLatin1(), error);

}

/***********************************************************************/

void QuSetOK(){
  static QString OKtxt = "  ok  ";

  // restore colour palette as last write may have been in error colours
  QPalette *palette = new QPalette(qApp->palette());
  
  if(qsasWindow!=NULL)
  {
	  qsasWindow->statusBar()->setPalette(*palette);
	  qsasWindow->statusBar()->showMessage(OKtxt,10000);
  }

}

/***********************************************************************/

int QuStrcpy(char * To, const char * From, int ToLen){

  /* ToLen is the size of the To string available
     (i.e. malloc as To[ToLen+1] ) */

 size_t FromLen;

   FromLen = strlen(From);
   if (FromLen > ToLen){
     strncpy(To, From, ToLen);
     To[ToLen] = '\0';
     QuMsgBar("String truncated on cpy", From, true);
   }
   else strcpy(To, From);

   return 0;
}

/***********************************************************************/

int QuStrcat(char * To, const char * From, int ToLen){

  /* ToLen is the remaining size of the To string available 
     (i.e. malloc as To[UsedLen + ToLen + 1] ) */
  
 size_t FromLen;
   
   FromLen = strlen(From);
   if (FromLen > ToLen){
     strncat(To, From, ToLen);
     To[ToLen] = '\0';
     QuMsgBar("String truncated on cat", From, true);
   }
   else strcat(To, From);
   
   return 0;
}


/***********************************************************************/

void QuParseSIC(string &str, double &multiplier){
	// returns units portion in str
	multiplier = 0;
	size_t ptr = str.find(">");
	if(ptr == string::npos) return;

	multiplier = strtod(str.c_str(), NULL);
	
	str = str.substr(ptr+1);
	
}


double ConvertThetaRad(double theta){
  const double PiByTwo = 2.0 * atan(1.0);

  // works for colatitude to latitude and vice versa

  return (double) (PiByTwo - theta);
}

// -------------------------------------------------------

double ConvertThetaDeg(double theta){

  // works for colatitude to latitude and vice versa

  return (double)(90.0 - theta);
}

// -------------------------------------------------------


string QuSanitizeName ( const char * oldName){

  string newName;

  int n_sl = 0;
  for(int i=0; i<strlen(oldName); i++)
    if(oldName[i] == '/') n_sl++;
    
  for(int i=0; i<strlen(oldName); i++){
    if(oldName[i] == '/'){
      newName += "_sl_";
    }
    else{
      newName += oldName[i];
    }
  }

  return newName;
}
/***********************************************************************/

string QuToUpper(const char *InStr)
{
  int i;
  string upper="";
  int s_length;

  s_length = strlen(InStr);

  for (int i=0; i < s_length; i++){
    upper += toupper(InStr[i]);
  }

  return upper;
}
/***********************************************************************/

string QuToUpper(string InStr)
{
	return QuToUpper(InStr.c_str()); 
}

/***********************************************************************/

bool QuStrCmp(const char *A, const char *B){
  
  string strA = QuToUpper(A);
  string strB = QuToUpper(B);
  
  if( strA == strB) return true;

  return false;

}




// -------------------------------------------------------------------

void QuNow(const char * txt, bool reset){
	static long start = clock(); // initial call
	if(reset) start = clock();
	
	printf(" >>>>seconds ------------------- %s at %.6f \n",txt, ((double)(clock()-start))/CLOCKS_PER_SEC);
}

// -------------------------------------------------------------------

bool QuSet_Array(int ndata, double *dArray, DvObject_var &obj, DvTime offset){
	// Note that QuLength call ensures we cannot run off array
	
    if( obj->seqSize() == ndata ) {
        for(int i=0; i<ndata; i++){
            dArray[i] = obj->asDouble(i);
        }
        return true;
    }
	
    if( obj->seqSize() == 1 && obj->arraySize() != ndata) {
        dArray[0] = obj->asDouble(0);
        return true;
    }
	
    if( obj->arraySize() == ndata) {
        for(int i=0; i<ndata; i++){
			dArray[i] = obj->asDouble(i);
        }
        return true;
    }
	
    if( obj->is_time() && obj->seqSize() == ndata) {
		
        for(int i=0; i<ndata; i++){
            dArray[i] = obj->time(i).getDiffSecs(offset);
        }
        return true;
    }

    return false;
}
// --------------------------------------------------------------------



void setRestoreVersion(const char*version)
{
	int ptr=0;
	int len = strlen(version);
	
	while( ptr < len && !isdigit(version[ptr]) ) ptr++;
	if(ptr == len) 
	{ 
		qsas_restore_release = 0;
		return;
	}
	
	int r1 = strtol( &(version[ptr]), NULL, 10);
	while( ptr < len &&  version[ptr] != '.' ) ptr++;
	ptr++;
	if(ptr >= len) 
	{ 
		qsas_restore_release = r1*10000;
		return;
	}
	
	int r2 = strtol( &(version[ptr]), NULL, 10);
	while( ptr < len && version[ptr] != '.' ) ptr++;
	ptr++;
	if(ptr >= len ) 
	{ 
		qsas_restore_release = r1*10000 + r2*100;
		return;
	}

	int r3 = strtol( &(version[ptr]), NULL, 10);
	
	qsas_restore_release = r1*10000 + r2*100 + r3;
	return;
}



// -----------------------------------------------------------------

void cleanWLtext(string &text)
{
	for(int i=0; i<text.size(); i++)
	{
		if(text[i] == '}') text[i] = '/';
	}
}


// --------------------------------------------------------------------


bool validObjName(DvString objName){

	DvNode *node = Wlist.find(objName);
	if(node == NULL) return false;
	return true;

}


DvObject *getWLObjExact(DvString Objname){
    
    DvNode *node = Wlist.find(Objname);
    if (node != NULL) {
        return node->obj().ptr();
    }
    return new DvObject();
}

DvObject *getWLObject(DvString Objname){
    
    Objname = Objname.front("[");
    Objname = Objname.front("<");
    Objname = Objname.trimmed();
    
    DvNode *node = Wlist.find(Objname);
    if (node != NULL) {
        return node->obj().ptr();
    }
    return new DvObject();
}

DvObject *getWLObject(const char * Objname){
	
	return getWLObject(DvString(Objname));
}


// ---------------

DvObject_var getObjectAndComp(DvString Objname){

	// should only be used if slot is no longer available
	
	if(Objname.empty()) return new DvObject();
	
	DvObject_var baseObj = getWLObject(Objname);

 	if(baseObj.is_nil()) {
		// try time string
		DvTime tobj(Objname.c_str());
		DvObject_var obj = new DvObject(tobj);
		
		if(tobj.base_day == MJD2000 && fabs(tobj.time_sec) < 1.e-10)
		{
			// not time, try double
			if(Objname.isInt()){
				obj = new DvObject(Objname.toInt());
			}
            else if(Objname.isNumber()){
				obj = new DvObject(Objname.toDouble());
			}
			else
			{
				// not double, keep string
				obj = new DvObject(Objname);
			}
		}
		
		return obj;
	}
	
	DvString compStr = Objname.betweenIncl("[", "]");
	DvString recStr = Objname.betweenIncl("<", ">");
	compStr += " ";
	compStr += recStr;
	
	DvObject_var retObj = baseObj->getObjComponent(compStr);
	
	return retObj;
}

DvObject_var getObjectAndComp(const char * Objname){
	
	return getObjectAndComp(DvString(Objname));
}



